@@ -22,6 +22,7 @@ from abc import abstractmethod
 from collections import deque
 from typing import (
     TYPE_CHECKING,
+    Any,
     Generator,
     Generic,
     Iterable,
@@ -71,14 +72,14 @@ InProgressPhysicalPlan = Iterator[Union[None, PartitionTask[PartitionT], Partiti
 MaterializedPhysicalPlan = Iterator[Union[None, PartitionTask[PartitionT], MaterializedResult[PartitionT]]]
 
 
-def _stage_id_counter():
+def _stage_id_counter() -> Iterator[int]:
     counter = 0
     while True:
         counter += 1
         yield counter
 
 
-stage_id_counter = _stage_id_counter()
+stage_id_counter: Iterator[int] = _stage_id_counter()
 
 
 def partition_read(
@@ -207,7 +208,7 @@ def lance_write(
     base_path: str,
     mode: str,
     io_config: IOConfig | None,
-    kwargs: dict | None,
+    kwargs: dict[str, Any] | None,
 ) -> InProgressPhysicalPlan[PartitionT]:
     """Write the results of `child_plan` into lance data files described by `write_info`."""
     yield from (
@@ -227,7 +228,7 @@ def lance_write(
 
 def data_sink_write(
     child_plan: InProgressPhysicalPlan[PartitionT],
-    sink: DataSink,
+    sink: DataSink[Any],
 ) -> InProgressPhysicalPlan[PartitionT]:
     """Write the results of `child_plan` into a custom write sink described by `sink`."""
     yield from (
@@ -239,7 +240,7 @@ def data_sink_write(
 def pipeline_instruction(
     child_plan: InProgressPhysicalPlan[PartitionT],
     pipeable_instruction: Instruction,
-    resource_request: execution_step.ResourceRequest,
+    resource_request: ResourceRequest,
 ) -> InProgressPhysicalPlan[PartitionT]:
     """Apply an instruction to the results of `child_plan`."""
     yield from (
@@ -278,7 +279,7 @@ def actor_pool_project(
     child_plan: InProgressPhysicalPlan[PartitionT],
     projection: ExpressionsProjection,
     actor_pool_manager: ActorPoolManager,
-    resource_request: execution_step.ResourceRequest,
+    resource_request: ResourceRequest,
     num_actors: int,
 ) -> InProgressPhysicalPlan[PartitionT]:
     stage_id = next(stage_id_counter)
@@ -640,7 +641,7 @@ def cross_join(
     left_plan: InProgressPhysicalPlan[PartitionT],
     right_plan: InProgressPhysicalPlan[PartitionT],
     outer_loop_side: JoinSide,
-):
+) -> InProgressPhysicalPlan[PartitionT]:
     stage_id = next(stage_id_counter)
 
     outer_plan, inner_plan = (left_plan, right_plan) if outer_loop_side == JoinSide.Left else (right_plan, left_plan)
@@ -1759,7 +1760,9 @@ def top_n(
     yield from global_limit(child_plan=child_plan, limit_rows=limit, eager=False, num_partitions=num_partitions)
 
 
-def fanout_random(child_plan: InProgressPhysicalPlan[PartitionT], num_partitions: int):
+def fanout_random(
+    child_plan: InProgressPhysicalPlan[PartitionT], num_partitions: int
+) -> InProgressPhysicalPlan[PartitionT]:
     """Splits the results of `child_plan` randomly into a list of `node.num_partitions()` number of partitions."""
     seed = 0
     for step in child_plan:
