@@ -33,7 +33,7 @@ class DataCatalogTable:
     table_name: str
     catalog_id: Optional[str] = None
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         import warnings
 
         warnings.warn(
