@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import functools
-from typing import TYPE_CHECKING, Callable
+from typing import TYPE_CHECKING, Any, Callable
 
 from daft.context import get_context
 from daft.daft import (
@@ -32,14 +32,16 @@ if TYPE_CHECKING:
     from daft.runners.partitioning import PartitionCacheEntry
 
 
-def _apply_daft_planning_config_to_initializer(classmethod_func: Callable[..., LogicalPlanBuilder]):
+def _apply_daft_planning_config_to_initializer(
+    classmethod_func: Callable[..., LogicalPlanBuilder],
+) -> Callable[..., LogicalPlanBuilder]:
     """Decorator to be applied to any @classmethod instantiation method on LogicalPlanBuilder.
 
     This decorator ensures that the current DaftPlanningConfig is applied to the instantiated LogicalPlanBuilder
     """
 
     @functools.wraps(classmethod_func)
-    def wrapper(cls: type[LogicalPlanBuilder], *args, **kwargs):
+    def wrapper(cls: type[LogicalPlanBuilder], *args: Any, **kwargs: Any) -> LogicalPlanBuilder:
         instantiated_logical_plan_builder = classmethod_func(cls, *args, **kwargs)
 
         # Parametrize the builder with the current DaftPlanningConfig
@@ -269,7 +271,7 @@ class LogicalPlanBuilder:
         builder = self._builder.pivot(group_by_pyexprs, pivot_col._expr, value_col._expr, agg_fn._expr, names)
         return LogicalPlanBuilder(builder)
 
-    def join(  # type: ignore[override]
+    def join(
         self,
         right: LogicalPlanBuilder,
         left_on: list[Expression],
@@ -290,7 +292,7 @@ class LogicalPlanBuilder:
         )
         return LogicalPlanBuilder(builder)
 
-    def concat(self, other: LogicalPlanBuilder) -> LogicalPlanBuilder:  # type: ignore[override]
+    def concat(self, other: LogicalPlanBuilder) -> LogicalPlanBuilder:
         builder = self._builder.concat(other._builder)
         return LogicalPlanBuilder(builder)
 
@@ -382,7 +384,7 @@ class LogicalPlanBuilder:
         path: str | pathlib.Path,
         mode: str,
         io_config: IOConfig,
-        kwargs: dict | None,
+        kwargs: dict[str, Any] | None,
     ) -> LogicalPlanBuilder:
         columns_name = self.schema().column_names()
         builder = self._builder.lance_write(
@@ -394,6 +396,6 @@ class LogicalPlanBuilder:
         )
         return LogicalPlanBuilder(builder)
 
-    def write_datasink(self, name: str, sink: DataSink) -> LogicalPlanBuilder:
+    def write_datasink(self, name: str, sink: DataSink[Any]) -> LogicalPlanBuilder:
         builder = self._builder.datasink_write(name, sink)
         return LogicalPlanBuilder(builder)
