@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import logging
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Callable
 
 from daft.daft import (
     CsvConvertOptions,
@@ -115,7 +115,7 @@ class MicroPartition:
         return MicroPartition._from_record_batches([table])
 
     @staticmethod
-    def from_pydict(data: dict) -> MicroPartition:
+    def from_pydict(data: dict[str, Any]) -> MicroPartition:
         table = RecordBatch.from_pydict(data)
         return MicroPartition._from_record_batches([table])
 
@@ -157,7 +157,7 @@ class MicroPartition:
         else:
             return self.to_record_batch().to_arrow_table()
 
-    def to_pydict(self) -> dict[str, list]:
+    def to_pydict(self) -> dict[str, list[Any]]:
         return self.to_record_batch().to_pydict()
 
     def to_pylist(self) -> list[dict[str, Any]]:
@@ -180,10 +180,6 @@ class MicroPartition:
     # Compute methods (MicroPartition -> MicroPartition)
     ###
 
-    def cast_to_schema(self, schema: Schema) -> MicroPartition:
-        """Casts a MicroPartition into the provided schema."""
-        return MicroPartition._from_pymicropartition(self._micropartition.cast_to_schema(schema._schema))
-
     def eval_expression_list(self, exprs: ExpressionsProjection) -> MicroPartition:
         assert all(isinstance(e, Expression) for e in exprs)
         pyexprs = [e._expr for e in exprs]
@@ -440,7 +436,7 @@ class MicroPartition:
             raise TypeError(f"Expected a bool, list[bool] or None for `nulls_first` but got {type(nulls_first)}")
         return Series._from_pyseries(self._micropartition.argsort(pyexprs, descending, nulls_first))
 
-    def __reduce__(self) -> tuple:
+    def __reduce__(self) -> tuple[Callable[[dict[str, Any]], MicroPartition], tuple[dict[str, Series]]]:
         names = self.column_names()
         return MicroPartition.from_pydict, ({name: self.get_column_by_name(name) for name in names},)
 
