@@ -8,12 +8,13 @@ from typing import TYPE_CHECKING, Any, Generic, TypeVar
 from uuid import uuid4
 
 from daft.datatype import TimeUnit
-from daft.table import MicroPartition
+from daft.recordbatch import MicroPartition
 
 if TYPE_CHECKING:
     import pandas as pd
     import pyarrow as pa
 
+    from daft.daft import PyMicroPartitionSet
     from daft.expressions.expressions import Expression
     from daft.logical.schema import Schema
 
@@ -276,6 +277,13 @@ class LocalPartitionSet(PartitionSet[MicroPartition]):
         super().__init__()
         self._partitions = {}
 
+    @staticmethod
+    def _from_micropartition_set(pset: PyMicroPartitionSet) -> LocalPartitionSet:
+        s = LocalPartitionSet()
+        for idx in range(0, pset.num_partitions()):
+            s.set_partition_from_table(idx=idx, part=MicroPartition._from_pymicropartition(pset.get_partition(idx)))
+        return s
+
     def items(self) -> list[tuple[PartID, MaterializedResult[MicroPartition]]]:
         return sorted(self._partitions.items())
 
