@@ -3,6 +3,7 @@ from __future__ import annotations
 import contextlib
 import dataclasses
 import logging
+import os
 import threading
 import time
 import uuid
@@ -18,12 +19,15 @@ import ray.experimental  # noqa: TID253
 
 from daft.arrow_utils import ensure_array
 from daft.context import execution_config_ctx, get_context
-from daft.daft import PyTable as _PyTable
+from daft.daft import DistributedPhysicalPlan, RayPartitionRef
+from daft.daft import PyRecordBatch as _PyRecordBatch
 from daft.dependencies import np
+from daft.recordbatch import RecordBatch
 from daft.runners import ray_tracing
 from daft.runners.progress_bar import ProgressBar
+from daft.scarf_telemetry import track_runner_on_scarf
 from daft.series import Series, item_to_series
-from daft.table import Table
+from daft.utils import SyncFromAsyncIterator
 
 logger = logging.getLogger(__name__)
 
@@ -31,7 +35,7 @@ try:
     import ray
 except ImportError:
     logger.error(
-        "Error when importing Ray. Please ensure that getdaft was installed with the Ray extras tag: getdaft[ray] (https://www.getdaft.io/projects/docs/en/latest/learn/install.html)"
+        "Error when importing Ray. Please ensure that daft was installed with the Ray extras tag: daft[ray] (https://www.getdaft.io/projects/docs/en/latest/learn/install.html)"
     )
     raise
 
@@ -56,6 +60,7 @@ from daft.execution.execution_step import (
 from daft.execution.physical_plan import ActorPoolManager
 from daft.expressions import ExpressionsProjection
 from daft.filesystem import glob_path_with_stats
+from daft.recordbatch import MicroPartition
 from daft.runners import runner_io
 from daft.runners.partitioning import (
     LocalPartitionSet,
@@ -68,7 +73,6 @@ from daft.runners.partitioning import (
 )
 from daft.runners.profiler import profiler
 from daft.runners.runner import Runner
-from daft.table import MicroPartition
 
 if TYPE_CHECKING:
     import dask
@@ -147,7 +151,7 @@ def _make_ray_block_from_micropartition(partition: MicroPartition) -> RayDataset
         # Convert arrays to Ray Data's native ArrowTensorType arrays
         new_arrs = {}
         for idx, field in enumerate(arrow_tbl.schema):
-            if daft_schema[field.name].dtype._is_fixed_shape_tensor_type():
+            if daft_schema[field.name].dtype.is_fixed_shape_tensor():
                 assert isinstance(field.type, pa.FixedShapeTensorType)
                 new_dtype = ArrowTensorType(field.type.shape, field.type.value_type)
                 arrow_arr = arrow_tbl[field.name].combine_chunks()
@@ -164,9 +168,12 @@ def _make_ray_block_from_micropartition(partition: MicroPartition) -> RayDataset
                     field.name,
                     pa.ExtensionArray.from_storage(new_dtype, new_storage_arr),
                 )
-            elif daft_schema[field.name].dtype._is_tensor_type():
+            elif daft_schema[field.name].dtype.is_tensor():
                 assert isinstance(field.type, pa.ExtensionType)
-                new_arrs[idx] = (field.name, ArrowTensorArray.from_numpy(partition.get_column(field.name).to_pylist()))
+                new_arrs[idx] = (
+                    field.name,
+                    ArrowTensorArray.from_numpy(partition.get_column_by_name(field.name).to_pylist()),
+                )
         for idx, (field_name, arr) in new_arrs.items():
             arrow_tbl = arrow_tbl.set_column(idx, pa.field(field_name, arr.type), arr)
 
@@ -202,7 +209,7 @@ def _micropartition_from_arrow_with_ray_data_extensions(arrow_table: pa.Table) -
     non_native_fields = []
     for arrow_field in arrow_table.schema:
         dt = _from_arrow_type_with_ray_data_extensions(arrow_field.type)
-        if dt == DataType.python() or dt._is_tensor_type() or dt._is_fixed_shape_tensor_type():
+        if dt == DataType.python() or dt.is_tensor() or dt.is_fixed_shape_tensor():
             non_native_fields.append(arrow_field.name)
     if non_native_fields:
         # If there are any contained Arrow types that are not natively supported, convert each
@@ -216,7 +223,9 @@ def _micropartition_from_arrow_with_ray_data_extensions(arrow_table: pa.Table) -
                 else item_to_series(name, column)
             )
             series_dict[name] = series._series
-        return MicroPartition._from_tables([Table._from_pytable(_PyTable.from_pylist_series(series_dict))])
+        return MicroPartition._from_record_batches(
+            [RecordBatch._from_pyrecordbatch(_PyRecordBatch.from_pylist_series(series_dict))]
+        )
     return MicroPartition.from_arrow(arrow_table)
 
 
@@ -561,9 +570,9 @@ def _ray_num_cpus_provider(ttl_seconds: int = 1) -> Generator[int, None, None]:
     Used as a generator as it provides a guard against calling ray.cluster_resources()
     more than once per `ttl_seconds`.
 
-    Example:
-    >>> p = _ray_num_cpus_provider()
-    >>> next(p)
+    Examples:
+        >>> p = _ray_num_cpus_provider()
+        >>> next(p)
     """
     last_checked_time = time.time()
     last_num_cpus_queried = int(ray.cluster_resources().get("CPU", 0))
@@ -1045,6 +1054,7 @@ def _build_partitions(
             *task.inputs,
         )
 
+    task.inputs.clear()
     metadatas_accessor = PartitionMetadataAccessor(metadatas_ref)
     task.set_result(
         [
@@ -1176,6 +1186,11 @@ class RayRunner(Runner[ray.ObjectRef]):
     ) -> None:
         super().__init__()
 
+        # ray.init does not accept "ray://" prefix for some reason.
+        # We remove it here to avoid issues.
+        if address is not None and address.startswith("ray://"):
+            address = address.replace("ray://", "")
+
         self.ray_address = address
 
         if ray.is_initialized():
@@ -1268,6 +1283,8 @@ class RayRunner(Runner[ray.ObjectRef]):
     def run_iter(
         self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
     ) -> Iterator[RayMaterializedResult]:
+        track_runner_on_scarf(runner=self.name)
+
         # Grab and freeze the current DaftExecutionConfig
         daft_execution_config = get_context().daft_execution_config
 
@@ -1277,30 +1294,91 @@ class RayRunner(Runner[ray.ObjectRef]):
         if daft_execution_config.enable_aqe:
             adaptive_planner = builder.to_adaptive_physical_plan_scheduler(daft_execution_config)
             while not adaptive_planner.is_done():
-                source_id, plan_scheduler = adaptive_planner.next()
+                stage_id, plan_scheduler = adaptive_planner.next()
+                start_time = time.time()
                 # don't store partition sets in variable to avoid reference
                 result_uuid = self._start_plan(
                     plan_scheduler, daft_execution_config, results_buffer_size=results_buffer_size
                 )
                 del plan_scheduler
                 results_iter = self._stream_plan(result_uuid)
-                # if source_id is None that means this is the final stage
-                if source_id is None:
-                    yield from results_iter
+                # if stage_id is None that means this is the final stage
+                if stage_id is None:
+                    num_rows_processed = 0
+                    bytes_processed = 0
+
+                    for result in results_iter:
+                        num_rows_processed += result.metadata().num_rows
+                        size_bytes = result.metadata().size_bytes
+                        if size_bytes is not None:
+                            bytes_processed += size_bytes
+                        yield result
+                    adaptive_planner.update_stats(
+                        time.time() - start_time, bytes_processed, num_rows_processed, stage_id
+                    )
                 else:
                     cache_entry = self._collect_into_cache(results_iter)
-                    adaptive_planner.update(source_id, cache_entry)
+                    adaptive_planner.update_stats(
+                        time.time() - start_time, cache_entry.size_bytes(), cache_entry.num_rows(), stage_id
+                    )
+                    adaptive_planner.update(stage_id, cache_entry)
                     del cache_entry
-        else:
-            # Finalize the logical plan and get a physical plan scheduler for translating the
-            # physical plan to executable tasks.
-            plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
 
-            result_uuid = self._start_plan(
-                plan_scheduler, daft_execution_config, results_buffer_size=results_buffer_size
+            enable_explain_analyze = os.getenv("DAFT_DEV_ENABLE_EXPLAIN_ANALYZE")
+            ray_logs_location = ray_tracing.get_log_location()
+            should_explain_analyze = (
+                ray_logs_location.exists()
+                and enable_explain_analyze is not None
+                and enable_explain_analyze in ["1", "true"]
             )
+            if should_explain_analyze:
+                explain_analyze_dir = ray_tracing.get_daft_trace_location(ray_logs_location)
+                explain_analyze_dir.mkdir(exist_ok=True, parents=True)
+                adaptive_planner.explain_analyze(str(explain_analyze_dir))
+        elif daft_execution_config.flotilla:
+            try:
+                distributed_planner = DistributedPhysicalPlan.from_logical_plan_builder(
+                    builder._builder, daft_execution_config
+                )
+            except Exception as e:
+                logger.error("Failed to build distributed plan, falling back to regular execution. Error: %s", str(e))
+                # Fallback to regular execution
+                self._execute_plan(builder, daft_execution_config, results_buffer_size)
+            else:
+                # If plan building succeeds, execute it
+                from functools import partial
 
-            yield from self._stream_plan(result_uuid)
+                psets = {
+                    k: [
+                        RayPartitionRef(v.partition(), v.metadata().num_rows, v.metadata().size_bytes or 0)
+                        for v in v.values()
+                    ]
+                    for k, v in self._part_set_cache.get_all_partition_sets().items()
+                }
+                # SyncFromAsyncIterator is used to convert an async iterator to a sync iterator.
+                # This is because the distributed planner returns an async iterator.
+                # Note that the async iterator is created lazily upon first iteration, this is because the `run_plan`
+                # method needs to capture the current python event loop.
+                sync_iter = SyncFromAsyncIterator(partial(distributed_planner.run_plan, psets))
+                for obj, size_bytes, num_rows in sync_iter:
+                    metadata_accessor = PartitionMetadataAccessor.from_metadata_list(
+                        [PartitionMetadata(num_rows, size_bytes)]
+                    )
+                    materialized_result = RayMaterializedResult(
+                        partition=obj,
+                        metadatas=metadata_accessor,
+                        metadata_idx=0,
+                    )
+                    yield materialized_result
+        else:
+            yield from self._execute_plan(builder, daft_execution_config, results_buffer_size)
+
+    def _execute_plan(self, builder, daft_execution_config, results_buffer_size):
+        # Finalize the logical plan and get a physical plan scheduler for translating the
+        # physical plan to executable tasks.
+        plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
+        result_uuid = self._start_plan(plan_scheduler, daft_execution_config, results_buffer_size=results_buffer_size)
+        yield from self._stream_plan(result_uuid)
 
     def run_iter_tables(
         self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
