@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import warnings
-from typing import Any, Iterator, Literal, TypeVar
+from typing import TYPE_CHECKING, Any, Callable, Iterator, Literal, TypeVar
 
 import daft.daft as native
 from daft.arrow_utils import ensure_array, ensure_chunked_array
@@ -10,6 +10,9 @@ from daft.datatype import DataType, TimeUnit, _ensure_registered_super_ext_type
 from daft.dependencies import np, pa, pd
 from daft.utils import pyarrow_supports_fixed_shape_tensor
 
+if TYPE_CHECKING:
+    import builtins
+
 
 class SeriesIterable:
     """Iterable wrapper for Series that efficiently handles different data types."""
@@ -21,19 +24,19 @@ class SeriesIterable:
         dt = self.series.datatype()
         if dt == DataType.python():
 
-            def yield_pylist():
+            def yield_pylist() -> Iterator[Any]:
                 yield from self.series._series.to_pylist()
 
             return yield_pylist()
         elif dt._should_cast_to_python():
 
-            def yield_pylist():
+            def yield_pylist() -> Iterator[Any]:
                 yield from self.series._series.cast(DataType.python()._dtype).to_pylist()
 
             return yield_pylist()
         else:
 
-            def arrow_to_py():
+            def arrow_to_py() -> Iterator[Any]:
                 # We directly call .to_arrow() on the internal PySeries object since the case
                 # above has already captured the fixed shape tensor case.
                 arrow_data = self.series._series.to_arrow()
@@ -95,7 +98,11 @@ class Series:
             raise TypeError(f"expected either PyArrow Array or Chunked Array, got {type(array)}")
 
     @staticmethod
-    def from_pylist(data: list, name: str = "list_series", pyobj: str = "allow") -> Series:
+    def from_pylist(
+        data: list[Any],
+        name: str = "list_series",
+        pyobj: Literal["allow", "disallow", "force"] = "allow",
+    ) -> Series:
         """Construct a Series from a Python list.
 
         The resulting type depends on the setting of pyobjects:
@@ -262,7 +269,7 @@ class Series:
 
         return arrow_arr
 
-    def to_pylist(self) -> list:
+    def to_pylist(self) -> list[Any]:
         """Convert this Series to a Python list."""
         if self.datatype().is_python():
             return self._series.to_pylist()
@@ -712,7 +719,29 @@ class Series:
     def partitioning(self) -> SeriesPartitioningNamespace:
         return SeriesPartitioningNamespace.from_series(self)
 
-    def __reduce__(self) -> tuple:
+    def __reduce__(
+        self,
+    ) -> (
+        tuple[
+            Callable[
+                [
+                    builtins.list[Any],
+                    builtins.str,
+                    Literal["allow", "disallow", "force"],
+                ],
+                Series,
+            ],
+            tuple[
+                builtins.list[Any],
+                builtins.str,
+                Literal["allow", "disallow", "force"],
+            ],
+        ]
+        | tuple[
+            Callable[[pa.Array | pa.ChunkedArray, builtins.str], Series],
+            tuple[pa.Array | pa.ChunkedArray, builtins.str],
+        ]
+    ):
         if self.datatype().is_python():
             return (Series.from_pylist, (self.to_pylist(), self.name(), "force"))
         else:
@@ -725,7 +754,12 @@ class Series:
     def _debug_bincode_deserialize(cls, b: bytes) -> Series:
         return Series._from_pyseries(PySeries._debug_bincode_deserialize(b))
 
-    def _eval_expressions(self, func_name, *others: Series, **kwargs) -> Series:
+    def _eval_expressions(
+        self,
+        func_name: builtins.str,
+        *others: Series | None,
+        **kwargs: Any,
+    ) -> Series:
         from daft.expressions.expressions import lit
 
         name = self._series.name()
@@ -734,8 +768,18 @@ class Series:
         col_names = []
         for i, other in enumerate(others):
             col_name = f"c{i}"
-            other_series_list.append(other._series.rename(col_name))
-            col_names.append(col_name)
+
+            if other is None:
+                continue
+            if not isinstance(other, Series):
+                try:
+                    other_series_list.append(Series.from_pylist([other])._series.rename(col_name))
+                    col_names.append(col_name)
+                except AttributeError:
+                    raise ValueError(f"expected another Series but got {type(other)}")
+            else:
+                other_series_list.append(other._series.rename(col_name))
+                col_names.append(col_name)
 
         rb = PyRecordBatch.from_pyseries_list([s] + other_series_list)
 
@@ -781,7 +825,7 @@ class SeriesNamespace:
         ns._series = series._series
         return ns
 
-    def _eval_expressions(self, func_name: str, *others: Series, **kwargs) -> Series:
+    def _eval_expressions(self, func_name: builtins.str, *others: Series | None, **kwargs: Any) -> Series:
         s = Series._from_pyseries(self._series)
         return s._eval_expressions(func_name, *others, **kwargs)
 
@@ -802,34 +846,20 @@ class SeriesFloatNamespace(SeriesNamespace):
 
 class SeriesStringNamespace(SeriesNamespace):
     def endswith(self, suffix: Series) -> Series:
-        if not isinstance(suffix, Series):
-            raise ValueError(f"expected another Series but got {type(suffix)}")
-        assert self._series is not None and suffix._series is not None
-        return Series._from_pyseries(self._series.utf8_endswith(suffix._series))
+        return self._eval_expressions("ends_with", suffix)
 
     def startswith(self, prefix: Series) -> Series:
-        if not isinstance(prefix, Series):
-            raise ValueError(f"expected another Series but got {type(prefix)}")
-        assert self._series is not None and prefix._series is not None
-        return Series._from_pyseries(self._series.utf8_startswith(prefix._series))
+        return self._eval_expressions("starts_with", prefix)
 
     def contains(self, pattern: Series) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_contains(pattern._series))
+        return self._eval_expressions("utf8_contains", pattern)
 
     def match(self, pattern: Series) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_match(pattern._series))
+        return self._eval_expressions("regexp_match", pattern)
 
     def split(self, pattern: Series, regex: bool = False) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_split(pattern._series, regex))
+        f_name = "regexp_split" if regex else "split"
+        return self._eval_expressions(f_name, pattern)
 
     def concat(self, other: Series) -> Series:
         if not isinstance(other, Series):
@@ -838,133 +868,71 @@ class SeriesStringNamespace(SeriesNamespace):
         return Series._from_pyseries(self._series) + other
 
     def extract(self, pattern: Series, index: int = 0) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_extract(pattern._series, index))
+        return self._eval_expressions("regexp_extract", pattern, index=index)
 
     def extract_all(self, pattern: Series, index: int = 0) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_extract_all(pattern._series, index))
+        return self._eval_expressions("regexp_extract_all", pattern, index=index)
 
     def replace(self, pattern: Series, replacement: Series, regex: bool = False) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        if not isinstance(replacement, Series):
-            raise ValueError(f"expected another Series but got {type(replacement)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_replace(pattern._series, replacement._series, regex))
+        f_name = "regexp_replace" if regex else "replace"
+        return self._eval_expressions(f_name, pattern, replacement)
 
     def length(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_length())
+        return self._eval_expressions("length")
 
     def length_bytes(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_length_bytes())
+        return self._eval_expressions("length_bytes")
 
     def lower(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_lower())
+        return self._eval_expressions("lower")
 
     def upper(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_upper())
+        return self._eval_expressions("upper")
 
     def lstrip(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_lstrip())
+        return self._eval_expressions("lstrip")
 
     def rstrip(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_rstrip())
+        return self._eval_expressions("rstrip")
 
     def reverse(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_reverse())
+        return self._eval_expressions("reverse")
 
     def capitalize(self) -> Series:
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_capitalize())
+        return self._eval_expressions("capitalize")
 
     def left(self, nchars: Series) -> Series:
-        if not isinstance(nchars, Series):
-            raise ValueError(f"expected another Series but got {type(nchars)}")
-        assert self._series is not None and nchars._series is not None
-        return Series._from_pyseries(self._series.utf8_left(nchars._series))
+        return self._eval_expressions("left", nchars)
 
     def right(self, nchars: Series) -> Series:
-        if not isinstance(nchars, Series):
-            raise ValueError(f"expected another Series but got {type(nchars)}")
-        assert self._series is not None and nchars._series is not None
-        return Series._from_pyseries(self._series.utf8_right(nchars._series))
+        return self._eval_expressions("right", nchars)
 
     def find(self, substr: Series) -> Series:
-        if not isinstance(substr, Series):
-            raise ValueError(f"expected another Series but got {type(substr)}")
-        assert self._series is not None and substr._series is not None
-        return Series._from_pyseries(self._series.utf8_find(substr._series))
+        return self._eval_expressions("find", substr)
 
     def rpad(self, length: Series, pad: Series) -> Series:
-        if not isinstance(length, Series):
-            raise ValueError(f"expected another Series but got {type(length)}")
-        if not isinstance(pad, Series):
-            raise ValueError(f"expected another Series but got {type(pad)}")
-        assert self._series is not None and length._series is not None and pad._series is not None
-        return Series._from_pyseries(self._series.utf8_rpad(length._series, pad._series))
+        return self._eval_expressions("rpad", length, pad)
 
     def lpad(self, length: Series, pad: Series) -> Series:
-        if not isinstance(length, Series):
-            raise ValueError(f"expected another Series but got {type(length)}")
-        if not isinstance(pad, Series):
-            raise ValueError(f"expected another Series but got {type(pad)}")
-        assert self._series is not None and length._series is not None and pad._series is not None
-        return Series._from_pyseries(self._series.utf8_lpad(length._series, pad._series))
+        return self._eval_expressions("lpad", length, pad)
 
     def repeat(self, n: Series) -> Series:
-        if not isinstance(n, Series):
-            raise ValueError(f"expected another Series but got {type(n)}")
-        assert self._series is not None and n._series is not None
-        return Series._from_pyseries(self._series.utf8_repeat(n._series))
+        return self._eval_expressions("repeat", n)
 
     def like(self, pattern: Series) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_like(pattern._series))
+        return self._eval_expressions("like", pattern)
 
     def ilike(self, pattern: Series) -> Series:
-        if not isinstance(pattern, Series):
-            raise ValueError(f"expected another Series but got {type(pattern)}")
-        assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_ilike(pattern._series))
+        return self._eval_expressions("ilike", pattern)
 
     def to_date(self, format: str) -> Series:
-        if not isinstance(format, str):
-            raise ValueError(f"expected str for format but got {type(format)}")
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_to_date(format))
+        return self._eval_expressions("to_date", format=format)
 
     def to_datetime(self, format: str, timezone: str | None = None) -> Series:
-        if not isinstance(format, str):
-            raise ValueError(f"expected str for format but got {type(format)}")
-        if timezone is not None and not isinstance(timezone, str):
-            raise ValueError(f"expected str for timezone but got {type(timezone)}")
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_to_datetime(format, timezone))
+        return self._eval_expressions("to_datetime", format=format, timezone=timezone)
 
     def substr(self, start: Series, length: Series | None = None) -> Series:
-        if not isinstance(start, Series):
-            raise ValueError(f"expected another Series but got {type(start)}")
-        if length is not None and not isinstance(length, Series):
-            raise ValueError(f"expected another Series but got {type(length)}")
-        if length is None:
-            length = Series.from_arrow(pa.array([None]))
-
-        assert self._series is not None and start._series is not None
-        return Series._from_pyseries(self._series.utf8_substr(start._series, length._series))
+        return self._eval_expressions("substr", start, length)
 
     def normalize(
         self,
@@ -974,26 +942,27 @@ class SeriesStringNamespace(SeriesNamespace):
         nfd_unicode: bool = False,
         white_space: bool = False,
     ) -> Series:
-        if not isinstance(remove_punct, bool):
-            raise ValueError(f"expected bool for remove_punct but got {type(remove_punct)}")
-        if not isinstance(lowercase, bool):
-            raise ValueError(f"expected bool for lowercase but got {type(lowercase)}")
-        if not isinstance(nfd_unicode, bool):
-            raise ValueError(f"expected bool for nfd_unicode but got {type(nfd_unicode)}")
-        if not isinstance(white_space, bool):
-            raise ValueError(f"expected bool for white_space but got {type(white_space)}")
-        assert self._series is not None
-        return Series._from_pyseries(self._series.utf8_normalize(remove_punct, lowercase, nfd_unicode, white_space))
+        return self._eval_expressions(
+            "normalize",
+            remove_punct=remove_punct,
+            lowercase=lowercase,
+            nfd_unicode=nfd_unicode,
+            white_space=white_space,
+        )
 
-    def count_matches(self, patterns: Series, whole_words: bool = False, case_sensitive: bool = True) -> Series:
-        if not isinstance(patterns, Series):
-            raise ValueError(f"expected another Series but got {type(patterns)}")
-        if not isinstance(whole_words, bool):
-            raise ValueError(f"expected bool for whole_word but got {type(whole_words)}")
-        if not isinstance(case_sensitive, bool):
-            raise ValueError(f"expected bool for case_sensitive but got {type(case_sensitive)}")
-        assert self._series is not None and patterns._series is not None
-        return Series._from_pyseries(self._series.utf8_count_matches(patterns._series, whole_words, case_sensitive))
+    def count_matches(
+        self,
+        patterns: Series,
+        *,
+        whole_words: bool = False,
+        case_sensitive: bool = True,
+    ) -> Series:
+        return self._eval_expressions(
+            "count_matches",
+            patterns=patterns,
+            whole_words=whole_words,
+            case_sensitive=case_sensitive,
+        )
 
 
 class SeriesDateNamespace(SeriesNamespace):
@@ -1093,23 +1062,16 @@ class SeriesListNamespace(SeriesNamespace):
             category=DeprecationWarning,
         )
 
-        return Series._from_pyseries(self._series.list_count(CountMode.All))
+        return self._eval_expressions("list_count", count_mode=CountMode.All)
 
     def length(self) -> Series:
-        return Series._from_pyseries(self._series.list_count(CountMode.All))
+        return self._eval_expressions("list_count", count_mode=CountMode.All)
 
     def get(self, idx: Series, default: Series) -> Series:
-        return Series._from_pyseries(self._series.list_get(idx._series, default._series))
+        return self._eval_expressions("list_get", idx=idx, default=default)
 
     def sort(self, desc: bool | Series = False, nulls_first: bool | Series | None = None) -> Series:
-        if isinstance(desc, bool):
-            desc = Series.from_pylist([desc], name="desc")
-        if nulls_first is None:
-            nulls_first = desc
-        elif isinstance(nulls_first, bool):
-            nulls_first = Series.from_pylist([nulls_first], name="nulls_first")
-
-        return Series._from_pyseries(self._series.list_sort(desc._series, nulls_first._series))
+        return self._eval_expressions("list_sort", desc=desc, nulls_first=nulls_first)
 
 
 class SeriesMapNamespace(SeriesNamespace):
