@@ -67,6 +67,10 @@ class SQLConnection:
         limit: int | None = None,
         partition_bounds: tuple[str, str] | None = None,
     ) -> str:
+        # If all options are None, just return the original sql
+        if projection is None and predicate is None and limit is None and partition_bounds is None:
+            return sql
+
         import sqlglot
 
         target_dialect = self.dialect
@@ -82,7 +86,7 @@ class SQLConnection:
                 f"Unsupported dialect: {target_dialect}, please refer to the documentation for supported dialects."
             )
 
-        query = sqlglot.subquery(sql, "subquery")
+        query = sqlglot.subquery(sql, "subquery", dialect=target_dialect)
 
         if projection is not None:
             query = query.select(*projection)
