@@ -2,16 +2,20 @@ from __future__ import annotations
 
 import dataclasses
 import warnings
-from typing import Callable
+from typing import TYPE_CHECKING, Callable, Literal
 from urllib.parse import urlparse
 
 import unitycatalog
 
 from daft.io import AzureConfig, IOConfig, S3Config
 
+if TYPE_CHECKING:
+    from unitycatalog.types import TableInfo
+
 
 @dataclasses.dataclass(frozen=True)
 class UnityCatalogTable:
+    table_info: TableInfo
     table_uri: str
     io_config: IOConfig | None
 
@@ -90,12 +94,20 @@ class UnityCatalog:
 
         return self._paginate_to_completion(_paginated_list_tables)
 
-    def load_table(self, table_name: str, new_table_storage_path: str | None = None) -> UnityCatalogTable:
+    def load_table(
+        self,
+        table_name: str,
+        new_table_storage_path: str | None = None,
+        operation: Literal["READ" | "READ_WRITE"] = "READ_WRITE",
+        table_type: Literal["EXTERNAL" | "MANAGED"] = "EXTERNAL",
+    ) -> UnityCatalogTable:
         """Loads an existing Unity Catalog table. If the table is not found, and information is provided in the method to create a new table, a new table will be attempted to be registered.
 
         Args:
             table_name (str): Name of the table in Unity Catalog in the form of dot-separated, 3-level namespace
             new_table_storage_path (str, optional): Cloud storage path URI to register a new external table using this path. Unity Catalog will validate if the path is valid and authorized for the principal, else will raise an exception.
+            operation ("READ" or "READ_WRITE", optional): The intended use of the table, which impacts authorization. Defaults to "READ_WRITE".
+                table_type ("EXTERNAL" or "MANAGED", optional): The table type impacts the creation of the table if it does not exist. If "MANAGED" the table is created as managed table. If "EXTERNAL" the table is created based on the new_tabke_storage_path.
 
         Returns:
             UnityCatalogTable
@@ -108,10 +120,12 @@ class UnityCatalog:
                     f"Table {table_name} is an existing storage table with a valid storage path. The 'new_table_storage_path' argument provided will be ignored."
                 )
         except unitycatalog.NotFoundError:
-            if not new_table_storage_path:
+            if table_type == "EXTERNAL" and not new_table_storage_path:
                 raise ValueError(
-                    f"Table {table_name} is not an existing table. If a new table needs to be created, provide 'new_table_storage_path' value."
+                    f"Table {table_name} is not an existing table. If a new table needs to be created, provide 'new_table_storage_path' value or set table_type to 'MANAGED'."
                 )
+            elif table_type == "MANAGED":
+                warnings.warn(f"Table {table_name} is being created as managed table in Unity Catalog.")
             try:
                 three_part_namesplit = table_name.split(".")
                 if len(three_part_namesplit) != 3 or not all(three_part_namesplit):
@@ -125,7 +139,7 @@ class UnityCatalog:
                     "name": three_part_namesplit[2],
                     "columns": None,
                     "data_source_format": "DELTA",
-                    "table_type": "EXTERNAL",
+                    "table_type": table_type,
                     "storage_location": new_table_storage_path,
                     "comment": None,
                 }
@@ -137,9 +151,7 @@ class UnityCatalog:
         table_id = table_info.table_id
         storage_location = table_info.storage_location
         # Grab credentials from Unity catalog and place it into the Table
-        temp_table_credentials = self._client.temporary_table_credentials.create(
-            operation="READ_WRITE", table_id=table_id
-        )
+        temp_table_credentials = self._client.temporary_table_credentials.create(operation=operation, table_id=table_id)
 
         scheme = urlparse(storage_location).scheme
         if scheme == "s3" or scheme == "s3a":
@@ -168,6 +180,7 @@ class UnityCatalog:
             io_config = None
 
         return UnityCatalogTable(
+            table_info=table_info,
             table_uri=storage_location,
             io_config=io_config,
         )
