@@ -17,6 +17,14 @@ pub enum DaftError {
     ComputeError(String),
     #[error("DaftError::ArrowError {0}")]
     ArrowError(arrow2::error::Error),
+    #[cfg(feature = "arrow")]
+    #[error("DaftError::ArrowRsError {0}")]
+    ArrowRsError(#[from] arrow_schema::ArrowError),
+    // TODO(desmond): We can't currently implement this as a From<parquet::errors::ParquetError>
+    // because this results in infinite nesting of types in `fixed_size_binary_op` in arithmetic.rs.
+    #[cfg(feature = "arrow")]
+    #[error("DaftError::ParquetError {0}")]
+    ParquetError(String),
     #[error("DaftError::ValueError {0}")]
     ValueError(String),
     #[cfg(feature = "python")]
