@@ -19,37 +19,44 @@ use crate::{
         ExtensionArray, FixedSizeBinaryArray, Float32Array, Float64Array, IntervalArray, NullArray,
         Utf8Array,
     },
-    kernels::search_sorted::{build_compare_with_nulls, cmp_float},
+    kernels::search_sorted::{build_nulls_first_compare_with_nulls, cmp_float},
     series::Series,
-    utils::{ensure_nulls_first, ensure_nulls_first_arr},
 };
 
 pub fn build_multi_array_compare(
     arrays: &[Series],
     descending: &[bool],
+    nulls_first: &[bool],
 ) -> DaftResult<DynComparator> {
-    build_multi_array_bicompare(arrays, arrays, descending)
+    build_multi_array_bicompare(arrays, arrays, descending, nulls_first)
 }
 
 pub fn build_multi_array_bicompare(
     left: &[Series],
     right: &[Series],
     descending: &[bool],
+    nulls_first: &[bool],
 ) -> DaftResult<DynComparator> {
     let mut cmp_list = Vec::with_capacity(left.len());
 
-    for ((l, r), desc) in left.iter().zip(right.iter()).zip(descending.iter()) {
-        cmp_list.push(build_compare_with_nulls(
+    for (((l, r), desc), nf) in left
+        .iter()
+        .zip(right.iter())
+        .zip(descending.iter())
+        .zip(nulls_first.iter())
+    {
+        cmp_list.push(build_nulls_first_compare_with_nulls(
             l.to_arrow().as_ref(),
             r.to_arrow().as_ref(),
             *desc,
+            *nf,
         )?);
     }
 
     let combined_comparator = Box::new(move |a_idx: usize, b_idx: usize| -> std::cmp::Ordering {
         for comparator in &cmp_list {
             match comparator(a_idx, b_idx) {
-                std::cmp::Ordering::Equal => continue,
+                std::cmp::Ordering::Equal => {}
                 other => return other,
             }
         }
@@ -68,8 +75,6 @@ where
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first(descending, nulls_first)?;
-
         let arrow_array = self.as_arrow();
 
         let result =
@@ -77,7 +82,7 @@ where
                 I::Native,
                 T::Native,
                 _,
-            >(arrow_array, ord::total_cmp, descending);
+            >(arrow_array, ord::total_cmp, descending, nulls_first);
 
         Ok(DataArray::<I>::from((self.name(), Box::new(result))))
     }
@@ -92,11 +97,11 @@ where
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first_arr(descending, nulls_first)?;
         let arrow_array = self.as_arrow();
         let first_desc = *descending.first().unwrap();
+        let first_nulls_first = *nulls_first.first().unwrap();
 
-        let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+        let others_cmp = build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
         let values = arrow_array.values().as_slice();
 
@@ -115,7 +120,7 @@ where
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         } else {
             multi_column_idx_sort(
@@ -132,7 +137,7 @@ where
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         };
 
@@ -164,7 +169,6 @@ impl Float32Array {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first(descending, nulls_first)?;
         let arrow_array = self.as_arrow();
 
         let result =
@@ -172,7 +176,7 @@ impl Float32Array {
                 I::Native,
                 f32,
                 _,
-            >(arrow_array, cmp_float::<f32>, descending);
+            >(arrow_array, cmp_float::<f32>, descending, nulls_first);
 
         Ok(DataArray::<I>::from((self.name(), Box::new(result))))
     }
@@ -187,12 +191,11 @@ impl Float32Array {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first_arr(descending, nulls_first)?;
-
         let arrow_array = self.as_arrow();
         let first_desc = *descending.first().unwrap();
+        let first_nulls_first = *nulls_first.first().unwrap();
 
-        let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+        let others_cmp = build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
         let values = arrow_array.values().as_slice();
 
@@ -211,7 +214,7 @@ impl Float32Array {
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         } else {
             multi_column_idx_sort(
@@ -228,7 +231,7 @@ impl Float32Array {
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         };
 
@@ -260,7 +263,6 @@ impl Float64Array {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first(descending, nulls_first)?;
         let arrow_array = self.as_arrow();
 
         let result =
@@ -268,7 +270,7 @@ impl Float64Array {
                 I::Native,
                 f64,
                 _,
-            >(arrow_array, cmp_float::<f64>, descending);
+            >(arrow_array, cmp_float::<f64>, descending, nulls_first);
 
         Ok(DataArray::<I>::from((self.name(), Box::new(result))))
     }
@@ -283,12 +285,11 @@ impl Float64Array {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first_arr(descending, nulls_first)?;
-
         let arrow_array = self.as_arrow();
         let first_desc = *descending.first().unwrap();
+        let first_nulls_first = *nulls_first.first().unwrap();
 
-        let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+        let others_cmp = build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
         let values = arrow_array.values().as_slice();
 
@@ -307,7 +308,7 @@ impl Float64Array {
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         } else {
             multi_column_idx_sort(
@@ -324,7 +325,7 @@ impl Float64Array {
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         };
 
@@ -356,7 +357,6 @@ impl Decimal128Array {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first(descending, nulls_first)?;
         let arrow_array = self.as_arrow();
 
         let result =
@@ -364,7 +364,7 @@ impl Decimal128Array {
                 I::Native,
                 i128,
                 _,
-            >(arrow_array, ord::total_cmp, descending);
+            >(arrow_array, ord::total_cmp, descending, nulls_first);
 
         Ok(DataArray::<I>::from((self.name(), Box::new(result))))
     }
@@ -379,11 +379,11 @@ impl Decimal128Array {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first_arr(descending, nulls_first)?;
         let arrow_array = self.as_arrow();
         let first_desc = *descending.first().unwrap();
+        let first_nulls_first = *nulls_first.first().unwrap();
 
-        let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+        let others_cmp = build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
         let values = arrow_array.values().as_slice();
 
@@ -402,7 +402,7 @@ impl Decimal128Array {
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         } else {
             multi_column_idx_sort(
@@ -419,7 +419,7 @@ impl Decimal128Array {
                 },
                 &others_cmp,
                 arrow_array.len(),
-                first_desc,
+                first_nulls_first,
             )
         };
 
@@ -464,10 +464,9 @@ impl NullArray {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first_arr(descending, nulls_first)?;
-        let first_desc = *descending.first().unwrap();
+        let first_nulls_first = *nulls_first.first().unwrap();
 
-        let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+        let others_cmp = build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
         let result = multi_column_idx_sort(
             self.data().validity(),
@@ -478,7 +477,7 @@ impl NullArray {
             },
             &others_cmp,
             self.len(),
-            first_desc,
+            first_nulls_first,
         );
 
         Ok(DataArray::<I>::from((self.name(), Box::new(result))))
@@ -516,10 +515,10 @@ impl BooleanArray {
         I: DaftIntegerType,
         <I as DaftNumericType>::Native: arrow2::types::Index,
     {
-        ensure_nulls_first_arr(descending, nulls_first)?;
         let first_desc = *descending.first().unwrap();
+        let first_nulls_first = *nulls_first.first().unwrap();
 
-        let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+        let others_cmp = build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
         let values = self
             .data()
@@ -543,7 +542,7 @@ impl BooleanArray {
                 },
                 &others_cmp,
                 self.len(),
-                first_desc,
+                first_nulls_first,
             )
         } else {
             multi_column_idx_sort(
@@ -560,7 +559,7 @@ impl BooleanArray {
                 },
                 &others_cmp,
                 self.len(),
-                first_desc,
+                first_nulls_first,
             )
         };
 
@@ -615,10 +614,11 @@ macro_rules! impl_binary_like_sort {
                 I: DaftIntegerType,
                 <I as DaftNumericType>::Native: arrow2::types::Index,
             {
-                ensure_nulls_first_arr(descending, nulls_first)?;
                 let first_desc = *descending.first().unwrap();
+                let first_nulls_first = *nulls_first.first().unwrap();
 
-                let others_cmp = build_multi_array_compare(others, &descending[1..])?;
+                let others_cmp =
+                    build_multi_array_compare(others, &descending[1..], &nulls_first[1..])?;
 
                 let values = self.as_arrow();
 
@@ -637,7 +637,7 @@ macro_rules! impl_binary_like_sort {
                         },
                         &others_cmp,
                         self.len(),
-                        first_desc,
+                        first_nulls_first,
                     )
                 } else {
                     multi_column_idx_sort(
@@ -654,7 +654,7 @@ macro_rules! impl_binary_like_sort {
                         },
                         &others_cmp,
                         self.len(),
-                        first_desc,
+                        first_nulls_first,
                     )
                 };
 
