@@ -16,7 +16,7 @@ use pyo3::{
 
 use crate::{
     array::{
-        ops::{as_arrow::AsArrow, DaftLogical, Utf8NormalizeOptions},
+        ops::{as_arrow::AsArrow, DaftLogical},
         pseudo_arrow::PseudoArrowArray,
         DataArray,
     },
@@ -345,156 +345,6 @@ impl PySeries {
         Ok(self.series.data_type().clone().into())
     }
 
-    pub fn utf8_endswith(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_endswith(&pattern.series)?.into())
-    }
-
-    pub fn utf8_startswith(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_startswith(&pattern.series)?.into())
-    }
-
-    pub fn utf8_contains(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_contains(&pattern.series)?.into())
-    }
-
-    pub fn utf8_match(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_match(&pattern.series)?.into())
-    }
-
-    pub fn utf8_split(&self, pattern: &Self, regex: bool) -> PyResult<Self> {
-        Ok(self.series.utf8_split(&pattern.series, regex)?.into())
-    }
-
-    pub fn utf8_extract(&self, pattern: &Self, index: usize) -> PyResult<Self> {
-        Ok(self.series.utf8_extract(&pattern.series, index)?.into())
-    }
-
-    pub fn utf8_extract_all(&self, pattern: &Self, index: usize) -> PyResult<Self> {
-        Ok(self.series.utf8_extract_all(&pattern.series, index)?.into())
-    }
-
-    pub fn utf8_replace(&self, pattern: &Self, replacement: &Self, regex: bool) -> PyResult<Self> {
-        Ok(self
-            .series
-            .utf8_replace(&pattern.series, &replacement.series, regex)?
-            .into())
-    }
-
-    pub fn utf8_length(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_length()?.into())
-    }
-
-    pub fn utf8_length_bytes(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_length_bytes()?.into())
-    }
-
-    pub fn utf8_lower(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_lower()?.into())
-    }
-
-    pub fn utf8_upper(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_upper()?.into())
-    }
-
-    pub fn utf8_lstrip(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_lstrip()?.into())
-    }
-
-    pub fn utf8_rstrip(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_rstrip()?.into())
-    }
-
-    pub fn utf8_reverse(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_reverse()?.into())
-    }
-
-    pub fn utf8_capitalize(&self) -> PyResult<Self> {
-        Ok(self.series.utf8_capitalize()?.into())
-    }
-
-    pub fn utf8_left(&self, nchars: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_left(&nchars.series)?.into())
-    }
-
-    pub fn utf8_right(&self, nchars: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_right(&nchars.series)?.into())
-    }
-
-    pub fn utf8_find(&self, substr: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_find(&substr.series)?.into())
-    }
-
-    pub fn utf8_rpad(&self, length: &Self, character: &Self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .utf8_rpad(&length.series, &character.series)?
-            .into())
-    }
-
-    pub fn utf8_lpad(&self, length: &Self, character: &Self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .utf8_lpad(&length.series, &character.series)?
-            .into())
-    }
-
-    pub fn utf8_repeat(&self, n: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_repeat(&n.series)?.into())
-    }
-
-    pub fn utf8_like(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_like(&pattern.series)?.into())
-    }
-
-    pub fn utf8_ilike(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_ilike(&pattern.series)?.into())
-    }
-
-    pub fn utf8_substr(&self, start: &Self, length: &Self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .utf8_substr(&start.series, &length.series)?
-            .into())
-    }
-
-    pub fn utf8_to_date(&self, format: &str) -> PyResult<Self> {
-        Ok(self.series.utf8_to_date(format)?.into())
-    }
-
-    #[pyo3(signature = (format, timezone=None))]
-    pub fn utf8_to_datetime(&self, format: &str, timezone: Option<&str>) -> PyResult<Self> {
-        Ok(self.series.utf8_to_datetime(format, timezone)?.into())
-    }
-
-    pub fn utf8_normalize(
-        &self,
-        remove_punct: bool,
-        lowercase: bool,
-        nfd_unicode: bool,
-        white_space: bool,
-    ) -> PyResult<Self> {
-        let opts = Utf8NormalizeOptions {
-            remove_punct,
-            lowercase,
-            nfd_unicode,
-            white_space,
-        };
-
-        Ok(self.series.utf8_normalize(opts)?.into())
-    }
-
-    pub fn utf8_count_matches(
-        &self,
-        patterns: &Self,
-        whole_word: bool,
-        case_sensitive: bool,
-    ) -> PyResult<Self> {
-        Ok(self
-            .series
-            .utf8_count_matches(&patterns.series, whole_word, case_sensitive)?
-            .into())
-    }
-
     pub fn dt_date(&self) -> PyResult<Self> {
         Ok(self.series.dt_date()?.into())
     }
@@ -607,25 +457,6 @@ impl PySeries {
         Ok(self.series.murmur3_32()?.into_series().into())
     }
 
-    pub fn list_count(&self, mode: CountMode) -> PyResult<Self> {
-        Ok(self.series.list_count(mode)?.into_series().into())
-    }
-
-    pub fn list_get(&self, idx: &Self, default: &Self) -> PyResult<Self> {
-        Ok(self.series.list_get(&idx.series, &default.series)?.into())
-    }
-
-    pub fn list_slice(&self, start: &Self, end: &Self) -> PyResult<Self> {
-        Ok(self.series.list_slice(&start.series, &end.series)?.into())
-    }
-
-    pub fn list_sort(&self, desc: &Self, nulls_first: &Self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .list_sort(&desc.series, &nulls_first.series)?
-            .into())
-    }
-
     pub fn map_get(&self, key: &Self) -> PyResult<Self> {
         Ok(self.series.map_get(&key.series)?.into())
     }
