@@ -1,11 +1,10 @@
-mod channel;
 mod pipeline_node;
 mod plan;
 #[cfg(feature = "python")]
 pub mod python;
-mod runtime;
 mod scheduling;
 mod stage;
+pub(crate) mod utils;
 
 #[cfg(feature = "python")]
 pub use python::register_modules;
