@@ -202,6 +202,18 @@ pub fn lit(item: Bound<PyAny>) -> PyResult<PyExpr> {
     } else if let Ok(pybytes) = item.downcast::<PyBytes>() {
         let bytes = pybytes.as_bytes();
         Ok(crate::lit(bytes).into())
+    } else if item.is_instance_of::<common_io_config::python::IOConfig>() {
+        let py_ioconfig = item.extract::<common_io_config::python::IOConfig>()?;
+        Ok(crate::lit(py_ioconfig.config).into())
+    } else if item.is_instance_of::<ImageMode>() {
+        let image_mode = item.extract::<ImageMode>()?;
+        Ok(crate::lit(image_mode).into())
+    } else if item.is_instance_of::<ImageFormat>() {
+        let fmt = item.extract::<ImageFormat>()?;
+        Ok(crate::lit(fmt).into())
+    } else if item.is_instance_of::<CountMode>() {
+        let mode = item.extract::<CountMode>()?;
+        Ok(crate::lit(mode).into())
     } else if item.is_none() {
         Ok(crate::null_lit().into())
     } else {
