@@ -4,11 +4,9 @@
 )]
 pub mod binary;
 pub mod coalesce;
-pub mod count_matches;
 pub mod distance;
 pub mod float;
 pub mod hash;
-pub mod list;
 pub mod minhash;
 pub mod numeric;
 #[cfg(feature = "python")]
@@ -18,7 +16,6 @@ pub mod temporal;
 pub mod to_struct;
 pub mod tokenize;
 pub mod uri;
-pub mod utf8;
 
 use common_error::DaftError;
 #[cfg(feature = "python")]
