@@ -1,12 +1,12 @@
-use common_error::{ensure, DaftError, DaftResult};
+use common_error::{DaftError, DaftResult};
 use daft_core::{
     datatypes::InferDataType,
-    prelude::{Field, Schema},
+    prelude::{DataType, Field, Schema},
     series::{IntoSeries, Series},
     with_match_numeric_daft_types,
 };
 use daft_dsl::{
-    functions::{ScalarFunction, ScalarUDF},
+    functions::{FunctionArgs, ScalarFunction, ScalarUDF},
     ExprRef,
 };
 use serde::{Deserialize, Serialize};
@@ -14,37 +14,33 @@ use serde::{Deserialize, Serialize};
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub struct Clip;
 
+#[derive(FunctionArgs)]
+struct ClipArgs<T> {
+    input: T,
+    min: Option<T>,
+    max: Option<T>,
+}
+
 #[typetag::serde]
 impl ScalarUDF for Clip {
-    fn evaluate(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
-        ensure!(
-            inputs.len() == 2 || inputs.len() == 3,
-            ComputeError: "clip takes exactly two or three arguments"
-        );
+    fn evaluate(&self, inputs: FunctionArgs<Series>) -> DaftResult<Series> {
+        let ClipArgs { input, min, max } = inputs.try_into()?;
 
-        let arr = inputs.required((0, "input"))?;
-        let arr_dtype = arr.data_type();
-        let null_lit = |name: &str| Series::full_null(name, arr_dtype, 1);
+        let input_dtype = input.data_type();
+        let null_lit = |name: &str| Series::full_null(name, input_dtype, 1);
 
-        let min = inputs
-            .optional((1, "min"))?
-            .cloned()
-            .unwrap_or_else(|| null_lit("min"));
-
-        let max = inputs
-            .optional((2, "max"))?
-            .cloned()
-            .unwrap_or_else(|| null_lit("max"));
+        let min = min.unwrap_or_else(|| null_lit("min"));
+        let max = max.unwrap_or_else(|| null_lit("max"));
 
         let output_type = InferDataType::clip_op(
-            &InferDataType::from(arr.data_type()),
+            &InferDataType::from(input.data_type()),
             &InferDataType::from(min.data_type()),
             &InferDataType::from(max.data_type()),
         )?;
         match &output_type {
             output_type if output_type.is_numeric() => {
                 with_match_numeric_daft_types!(output_type, |$T| {
-                    let self_casted = arr.cast(output_type)?;
+                    let self_casted = input.cast(output_type)?;
                     let min_casted = min.cast(output_type)?;
                     let max_casted = max.cast(output_type)?;
 
@@ -65,23 +61,28 @@ impl ScalarUDF for Clip {
         "clip"
     }
 
-    fn to_field(&self, inputs: &[ExprRef], schema: &Schema) -> DaftResult<Field> {
-        ensure!(
-            inputs.len() == 3,
-            SchemaMismatch: "clip takes exactly three arguments (input, min, max) got {}", inputs.len()
-        );
+    fn function_args_to_field(
+        &self,
+        inputs: FunctionArgs<ExprRef>,
+        schema: &Schema,
+    ) -> DaftResult<Field> {
+        let ClipArgs { input, min, max } = inputs.try_into()?;
 
-        let array_field = inputs[0].to_field(schema)?;
-        let min_field = inputs[1].to_field(schema)?;
-        let max_field = inputs[2].to_field(schema)?;
+        let input_field = input.to_field(schema)?;
+        let min_field = min.map_or(Ok(Field::new("min", DataType::Null)), |min| {
+            min.to_field(schema)
+        })?;
+        let max_field = max.map_or(Ok(Field::new("max", DataType::Null)), |max| {
+            max.to_field(schema)
+        })?;
 
         let output_type = InferDataType::clip_op(
-            &InferDataType::from(&array_field.dtype),
+            &InferDataType::from(&input_field.dtype),
             &InferDataType::from(&min_field.dtype),
             &InferDataType::from(&max_field.dtype),
         )?;
 
-        Ok(Field::new(array_field.name, output_type))
+        Ok(Field::new(input_field.name, output_type))
     }
     fn docstring(&self) -> &'static str {
         "Clips a number to a specified range. If left bound is None, no lower clipping is applied. If right bound is None, no upper clipping is applied. Panics if right bound < left bound."
