@@ -1,34 +1,37 @@
-use common_error::{ensure, DaftResult};
+use common_error::DaftResult;
 use daft_core::{
     prelude::{Field, Schema},
     series::Series,
 };
 use daft_dsl::{
-    functions::{ScalarFunction, ScalarUDF},
+    functions::{FunctionArgs, ScalarFunction, ScalarUDF, UnaryArg},
     ExprRef,
 };
 use serde::{Deserialize, Serialize};
 
-use super::to_field_single_numeric;
+use super::to_field_numeric;
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub struct Floor;
 
 #[typetag::serde]
 impl ScalarUDF for Floor {
-    fn evaluate(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
-        ensure!(inputs.len() == 1, "floor expects 1 argument");
-        let s = inputs.required((0, "input"))?;
-
-        s.floor()
+    fn evaluate(&self, inputs: FunctionArgs<Series>) -> DaftResult<Series> {
+        let UnaryArg { input } = inputs.try_into()?;
+        input.floor()
     }
 
     fn name(&self) -> &'static str {
         "floor"
     }
 
-    fn to_field(&self, inputs: &[ExprRef], schema: &Schema) -> DaftResult<Field> {
-        to_field_single_numeric(self, inputs, schema)
+    fn function_args_to_field(
+        &self,
+        inputs: FunctionArgs<ExprRef>,
+        schema: &Schema,
+    ) -> DaftResult<Field> {
+        let UnaryArg { input } = inputs.try_into()?;
+        to_field_numeric(self, &input, schema)
     }
 
     fn docstring(&self) -> &'static str {
