@@ -15,13 +15,11 @@ macro_rules! simple_python_wrapper {
 mod binary;
 mod coalesce;
 mod distance;
-mod list;
 mod misc;
 mod sequence;
 mod temporal;
 mod tokenize;
 mod uri;
-mod utf8;
 
 use std::sync::Arc;
 
@@ -105,25 +103,7 @@ pub fn register(parent: &Bound<PyModule>) -> PyResult<()> {
     add!(binary::try_encode);
     add!(binary::try_decode);
 
-    add!(list::list_chunk);
-    add!(list::list_count);
-    add!(list::explode);
-    add!(list::list_get);
-    add!(list::list_join);
-    add!(list::list_max);
-    add!(list::list_mean);
-    add!(list::list_min);
-    add!(list::list_slice);
-    add!(list::list_sort);
-    add!(list::list_sum);
-    add!(list::list_count_distinct);
-    add!(list::list_value_counts);
-    add!(list::list_distinct);
-    add!(list::list_bool_and);
-    add!(list::list_bool_or);
-
     add!(misc::to_struct);
-    add!(misc::utf8_count_matches);
     add!(misc::hash);
     add!(misc::minhash);
 
@@ -156,34 +136,5 @@ pub fn register(parent: &Bound<PyModule>) -> PyResult<()> {
     add!(uri::url_download);
     add!(uri::url_upload);
 
-    add!(utf8::utf8_capitalize);
-    add!(utf8::utf8_contains);
-    add!(utf8::utf8_endswith);
-    add!(utf8::utf8_extract);
-    add!(utf8::utf8_extract_all);
-    add!(utf8::utf8_find);
-    add!(utf8::utf8_ilike);
-    add!(utf8::utf8_left);
-    add!(utf8::utf8_length);
-    add!(utf8::utf8_length_bytes);
-    add!(utf8::utf8_like);
-    add!(utf8::utf8_lower);
-    add!(utf8::utf8_lpad);
-    add!(utf8::utf8_lstrip);
-    add!(utf8::utf8_match);
-    add!(utf8::utf8_repeat);
-    add!(utf8::utf8_replace);
-    add!(utf8::utf8_reverse);
-    add!(utf8::utf8_right);
-    add!(utf8::utf8_rpad);
-    add!(utf8::utf8_rstrip);
-    add!(utf8::utf8_split);
-    add!(utf8::utf8_startswith);
-    add!(utf8::utf8_substr);
-    add!(utf8::utf8_upper);
-    add!(utf8::utf8_normalize);
-    add!(utf8::utf8_to_date);
-    add!(utf8::utf8_to_datetime);
-
     Ok(())
 }
