@@ -1,14 +1,14 @@
 use std::sync::Arc;
 
-use common_runtime::RuntimeRef;
 use daft_dsl::ExprRef;
 use daft_micropartition::MicroPartition;
-use tracing::instrument;
+use tracing::{instrument, Span};
 
 use super::intermediate_op::{
     IntermediateOpExecuteResult, IntermediateOpState, IntermediateOperator,
     IntermediateOperatorResult,
 };
+use crate::ExecutionTaskSpawner;
 
 pub struct FilterOperator {
     predicate: ExprRef,
@@ -26,21 +26,28 @@ impl IntermediateOperator for FilterOperator {
         &self,
         input: Arc<MicroPartition>,
         state: Box<dyn IntermediateOpState>,
-        runtime: &RuntimeRef,
+        task_spawner: &ExecutionTaskSpawner,
     ) -> IntermediateOpExecuteResult {
         let predicate = self.predicate.clone();
-        runtime
-            .spawn(async move {
-                let out = input.filter(&[predicate])?;
-                Ok((
-                    state,
-                    IntermediateOperatorResult::NeedMoreInput(Some(Arc::new(out))),
-                ))
-            })
+        task_spawner
+            .spawn(
+                async move {
+                    let out = input.filter(&[predicate])?;
+                    Ok((
+                        state,
+                        IntermediateOperatorResult::NeedMoreInput(Some(Arc::new(out))),
+                    ))
+                },
+                Span::current(),
+            )
             .into()
     }
 
+    fn multiline_display(&self) -> Vec<String> {
+        vec![format!("Filter: {}", self.predicate)]
+    }
+
     fn name(&self) -> &'static str {
-        "FilterOperator"
+        "Filter"
     }
 }
