@@ -11,13 +11,22 @@ use super::source::Source;
 use crate::sources::source::SourceStream;
 
 pub struct InMemorySource {
-    data: PartitionSetRef<MicroPartitionRef>,
+    data: Option<PartitionSetRef<MicroPartitionRef>>,
+    size_bytes: usize,
     schema: SchemaRef,
 }
 
 impl InMemorySource {
-    pub fn new(data: PartitionSetRef<MicroPartitionRef>, schema: SchemaRef) -> Self {
-        Self { data, schema }
+    pub fn new(
+        data: Option<PartitionSetRef<MicroPartitionRef>>,
+        schema: SchemaRef,
+        size_bytes: usize,
+    ) -> Self {
+        Self {
+            data,
+            size_bytes,
+            schema,
+        }
     }
     pub fn arced(self) -> Arc<dyn Source> {
         Arc::new(self) as Arc<dyn Source>
@@ -32,11 +41,25 @@ impl Source for InMemorySource {
         _maintain_order: bool,
         _io_stats: IOStatsRef,
     ) -> DaftResult<SourceStream<'static>> {
-        Ok(self.data.clone().to_partition_stream())
+        Ok(self
+            .data
+            .as_ref()
+            .unwrap_or_else(|| panic!("No data in InMemorySource"))
+            .clone()
+            .to_partition_stream())
     }
     fn name(&self) -> &'static str {
-        "InMemory"
+        "InMemorySource"
     }
+
+    fn multiline_display(&self) -> Vec<String> {
+        let mut res = vec![];
+        res.push("InMemorySource:".to_string());
+        res.push(format!("Schema = {}", self.schema.short_string()));
+        res.push(format!("Size bytes = {}", self.size_bytes));
+        res
+    }
+
     fn schema(&self) -> &SchemaRef {
         &self.schema
     }
