@@ -505,6 +505,7 @@ async fn stream_scan_task(
 
     Ok(table_stream.map(move |table| {
         let table = table?;
+        #[allow(deprecated)]
         let casted_table = table.cast_to_schema_with_fill(
             scan_task.materialized_schema().as_ref(),
             scan_task
@@ -513,10 +514,20 @@ async fn stream_scan_task(
                 .map(|pspec| pspec.to_fill_map())
                 .as_ref(),
         )?;
+
+        let stats = scan_task
+            .statistics
+            .as_ref()
+            .map(|stats| {
+                #[allow(deprecated)]
+                stats.cast_to_schema(&scan_task.materialized_schema())
+            })
+            .transpose()?;
+
         let mp = Arc::new(MicroPartition::new_loaded(
             scan_task.materialized_schema(),
             Arc::new(vec![casted_table]),
-            scan_task.statistics.clone(),
+            stats,
         ));
         Ok(mp)
     }))
