@@ -1,14 +1,21 @@
 #[allow(unused)]
 mod plan;
+#[cfg(feature = "python")]
+mod python;
 mod translate;
-
 #[cfg(feature = "python")]
 pub use plan::CatalogWrite;
 #[cfg(feature = "python")]
+pub use plan::DataSink;
+#[cfg(feature = "python")]
 pub use plan::LanceWrite;
 pub use plan::{
     ActorPoolProject, Concat, CrossJoin, EmptyScan, Explode, Filter, HashAggregate, HashJoin,
     InMemoryScan, Limit, LocalPhysicalPlan, LocalPhysicalPlanRef, MonotonicallyIncreasingId,
-    PhysicalScan, PhysicalWrite, Pivot, Project, Sample, Sort, UnGroupedAggregate, Unpivot,
+    PhysicalScan, PhysicalWrite, Pivot, Project, Sample, Sort, TopN, UnGroupedAggregate, Unpivot,
+    WindowOrderByOnly, WindowPartitionAndDynamicFrame, WindowPartitionAndOrderBy,
+    WindowPartitionOnly,
 };
+#[cfg(feature = "python")]
+pub use python::{register_modules, PyLocalPhysicalPlan};
 pub use translate::translate;
