@@ -1,16 +1,17 @@
-use std::sync::Arc;
+use std::{cmp::max, sync::Arc};
 
 use common_resource_request::ResourceRequest;
 use common_scan_info::{Pushdowns, ScanTaskLikeRef};
 use daft_core::prelude::*;
-use daft_dsl::{AggExpr, ExprRef};
+use daft_dsl::{AggExpr, ExprRef, WindowExpr, WindowFrame};
 use daft_logical_plan::{
     stats::{PlanStats, StatsState},
     InMemoryInfo, OutputFileInfo,
 };
+use serde::{Deserialize, Serialize};
 
 pub type LocalPhysicalPlanRef = Arc<LocalPhysicalPlan>;
-#[derive(Debug, strum::IntoStaticStr)]
+#[derive(Debug, strum::IntoStaticStr, Serialize, Deserialize)]
 pub enum LocalPhysicalPlan {
     InMemoryScan(InMemoryScan),
     PhysicalScan(PhysicalScan),
@@ -22,6 +23,7 @@ pub enum LocalPhysicalPlan {
     Explode(Explode),
     Unpivot(Unpivot),
     Sort(Sort),
+    TopN(TopN),
     // Split(Split),
     Sample(Sample),
     MonotonicallyIncreasingId(MonotonicallyIncreasingId),
@@ -46,6 +48,12 @@ pub enum LocalPhysicalPlan {
     CatalogWrite(CatalogWrite),
     #[cfg(feature = "python")]
     LanceWrite(LanceWrite),
+    #[cfg(feature = "python")]
+    DataSink(DataSink),
+    WindowPartitionOnly(WindowPartitionOnly),
+    WindowPartitionAndOrderBy(WindowPartitionAndOrderBy),
+    WindowPartitionAndDynamicFrame(WindowPartitionAndDynamicFrame),
+    WindowOrderByOnly(WindowOrderByOnly),
 }
 
 impl LocalPhysicalPlan {
@@ -72,6 +80,7 @@ impl LocalPhysicalPlan {
             | Self::Explode(Explode { stats_state, .. })
             | Self::Unpivot(Unpivot { stats_state, .. })
             | Self::Sort(Sort { stats_state, .. })
+            | Self::TopN(TopN { stats_state, .. })
             | Self::Sample(Sample { stats_state, .. })
             | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { stats_state, .. })
             | Self::UnGroupedAggregate(UnGroupedAggregate { stats_state, .. })
@@ -80,10 +89,18 @@ impl LocalPhysicalPlan {
             | Self::Concat(Concat { stats_state, .. })
             | Self::HashJoin(HashJoin { stats_state, .. })
             | Self::CrossJoin(CrossJoin { stats_state, .. })
-            | Self::PhysicalWrite(PhysicalWrite { stats_state, .. }) => stats_state,
+            | Self::PhysicalWrite(PhysicalWrite { stats_state, .. })
+            | Self::WindowPartitionOnly(WindowPartitionOnly { stats_state, .. })
+            | Self::WindowPartitionAndOrderBy(WindowPartitionAndOrderBy { stats_state, .. })
+            | Self::WindowPartitionAndDynamicFrame(WindowPartitionAndDynamicFrame {
+                stats_state,
+                ..
+            })
+            | Self::WindowOrderByOnly(WindowOrderByOnly { stats_state, .. }) => stats_state,
             #[cfg(feature = "python")]
             Self::CatalogWrite(CatalogWrite { stats_state, .. })
-            | Self::LanceWrite(LanceWrite { stats_state, .. }) => stats_state,
+            | Self::LanceWrite(LanceWrite { stats_state, .. })
+            | Self::DataSink(DataSink { stats_state, .. }) => stats_state,
         }
     }
 
@@ -228,6 +245,98 @@ impl LocalPhysicalPlan {
         .arced()
     }
 
+    pub(crate) fn window_partition_only(
+        input: LocalPhysicalPlanRef,
+        partition_by: Vec<ExprRef>,
+        schema: SchemaRef,
+        stats_state: StatsState,
+        aggregations: Vec<AggExpr>,
+        aliases: Vec<String>,
+    ) -> LocalPhysicalPlanRef {
+        Self::WindowPartitionOnly(WindowPartitionOnly {
+            input,
+            partition_by,
+            schema,
+            stats_state,
+            aggregations,
+            aliases,
+        })
+        .arced()
+    }
+
+    #[allow(clippy::too_many_arguments)]
+    pub(crate) fn window_partition_and_order_by(
+        input: LocalPhysicalPlanRef,
+        partition_by: Vec<ExprRef>,
+        order_by: Vec<ExprRef>,
+        descending: Vec<bool>,
+        schema: SchemaRef,
+        stats_state: StatsState,
+        functions: Vec<WindowExpr>,
+        aliases: Vec<String>,
+    ) -> LocalPhysicalPlanRef {
+        Self::WindowPartitionAndOrderBy(WindowPartitionAndOrderBy {
+            input,
+            partition_by,
+            order_by,
+            descending,
+            schema,
+            stats_state,
+            functions,
+            aliases,
+        })
+        .arced()
+    }
+
+    #[allow(clippy::too_many_arguments)]
+    pub(crate) fn window_partition_and_dynamic_frame(
+        input: LocalPhysicalPlanRef,
+        partition_by: Vec<ExprRef>,
+        order_by: Vec<ExprRef>,
+        descending: Vec<bool>,
+        frame: WindowFrame,
+        min_periods: usize,
+        schema: SchemaRef,
+        stats_state: StatsState,
+        aggregations: Vec<AggExpr>,
+        aliases: Vec<String>,
+    ) -> LocalPhysicalPlanRef {
+        Self::WindowPartitionAndDynamicFrame(WindowPartitionAndDynamicFrame {
+            input,
+            partition_by,
+            order_by,
+            descending,
+            frame,
+            min_periods,
+            schema,
+            stats_state,
+            aggregations,
+            aliases,
+        })
+        .arced()
+    }
+
+    pub(crate) fn window_order_by_only(
+        input: LocalPhysicalPlanRef,
+        order_by: Vec<ExprRef>,
+        descending: Vec<bool>,
+        schema: SchemaRef,
+        stats_state: StatsState,
+        functions: Vec<WindowExpr>,
+        aliases: Vec<String>,
+    ) -> LocalPhysicalPlanRef {
+        Self::WindowOrderByOnly(WindowOrderByOnly {
+            input,
+            order_by,
+            descending,
+            schema,
+            stats_state,
+            functions,
+            aliases,
+        })
+        .arced()
+    }
+
     pub(crate) fn unpivot(
         input: LocalPhysicalPlanRef,
         ids: Vec<ExprRef>,
@@ -292,6 +401,27 @@ impl LocalPhysicalPlan {
         .arced()
     }
 
+    pub(crate) fn top_n(
+        input: LocalPhysicalPlanRef,
+        sort_by: Vec<ExprRef>,
+        descending: Vec<bool>,
+        nulls_first: Vec<bool>,
+        limit: i64,
+        stats_state: StatsState,
+    ) -> LocalPhysicalPlanRef {
+        let schema = input.schema().clone();
+        Self::TopN(TopN {
+            input,
+            sort_by,
+            descending,
+            nulls_first,
+            limit,
+            schema,
+            stats_state,
+        })
+        .arced()
+    }
+
     pub(crate) fn sample(
         input: LocalPhysicalPlanRef,
         fraction: f64,
@@ -433,6 +563,22 @@ impl LocalPhysicalPlan {
         .arced()
     }
 
+    #[cfg(feature = "python")]
+    pub(crate) fn data_sink(
+        input: LocalPhysicalPlanRef,
+        data_sink_info: daft_logical_plan::DataSinkInfo,
+        file_schema: SchemaRef,
+        stats_state: StatsState,
+    ) -> LocalPhysicalPlanRef {
+        Self::DataSink(DataSink {
+            input,
+            data_sink_info,
+            file_schema,
+            stats_state,
+        })
+        .arced()
+    }
+
     pub fn schema(&self) -> &SchemaRef {
         match self {
             Self::PhysicalScan(PhysicalScan { schema, .. })
@@ -445,30 +591,45 @@ impl LocalPhysicalPlan {
             | Self::HashAggregate(HashAggregate { schema, .. })
             | Self::Pivot(Pivot { schema, .. })
             | Self::Sort(Sort { schema, .. })
+            | Self::TopN(TopN { schema, .. })
             | Self::Sample(Sample { schema, .. })
             | Self::HashJoin(HashJoin { schema, .. })
             | Self::CrossJoin(CrossJoin { schema, .. })
             | Self::Explode(Explode { schema, .. })
             | Self::Unpivot(Unpivot { schema, .. })
             | Self::Concat(Concat { schema, .. })
-            | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { schema, .. }) => schema,
+            | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { schema, .. })
+            | Self::WindowPartitionOnly(WindowPartitionOnly { schema, .. })
+            | Self::WindowPartitionAndOrderBy(WindowPartitionAndOrderBy { schema, .. })
+            | Self::WindowPartitionAndDynamicFrame(WindowPartitionAndDynamicFrame {
+                schema, ..
+            })
+            | Self::WindowOrderByOnly(WindowOrderByOnly { schema, .. }) => schema,
             Self::PhysicalWrite(PhysicalWrite { file_schema, .. }) => file_schema,
             Self::InMemoryScan(InMemoryScan { info, .. }) => &info.source_schema,
             #[cfg(feature = "python")]
             Self::CatalogWrite(CatalogWrite { file_schema, .. }) => file_schema,
             #[cfg(feature = "python")]
             Self::LanceWrite(LanceWrite { file_schema, .. }) => file_schema,
+            #[cfg(feature = "python")]
+            Self::DataSink(DataSink { file_schema, .. }) => file_schema,
+            Self::WindowPartitionOnly(WindowPartitionOnly { schema, .. }) => schema,
+            Self::WindowPartitionAndOrderBy(WindowPartitionAndOrderBy { schema, .. }) => schema,
         }
     }
+
+    pub fn estimated_memory_cost(&self) -> usize {
+        todo!("Implement estimated memory cost for local physical plan");
+    }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct InMemoryScan {
     pub info: InMemoryInfo,
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct PhysicalScan {
     pub scan_tasks: Arc<Vec<ScanTaskLikeRef>>,
     pub pushdowns: Pushdowns,
@@ -476,13 +637,13 @@ pub struct PhysicalScan {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct EmptyScan {
     pub schema: SchemaRef,
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Project {
     pub input: LocalPhysicalPlanRef,
     pub projection: Vec<ExprRef>,
@@ -490,7 +651,7 @@ pub struct Project {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct ActorPoolProject {
     pub input: LocalPhysicalPlanRef,
     pub projection: Vec<ExprRef>,
@@ -498,7 +659,7 @@ pub struct ActorPoolProject {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Filter {
     pub input: LocalPhysicalPlanRef,
     pub predicate: ExprRef,
@@ -506,7 +667,7 @@ pub struct Filter {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Limit {
     pub input: LocalPhysicalPlanRef,
     pub num_rows: i64,
@@ -514,7 +675,7 @@ pub struct Limit {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Explode {
     pub input: LocalPhysicalPlanRef,
     pub to_explode: Vec<ExprRef>,
@@ -522,7 +683,7 @@ pub struct Explode {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Sort {
     pub input: LocalPhysicalPlanRef,
     pub sort_by: Vec<ExprRef>,
@@ -532,7 +693,18 @@ pub struct Sort {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
+pub struct TopN {
+    pub input: LocalPhysicalPlanRef,
+    pub sort_by: Vec<ExprRef>,
+    pub descending: Vec<bool>,
+    pub nulls_first: Vec<bool>,
+    pub limit: i64,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Sample {
     pub input: LocalPhysicalPlanRef,
     pub fraction: f64,
@@ -542,7 +714,7 @@ pub struct Sample {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct MonotonicallyIncreasingId {
     pub input: LocalPhysicalPlanRef,
     pub column_name: String,
@@ -550,7 +722,7 @@ pub struct MonotonicallyIncreasingId {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct UnGroupedAggregate {
     pub input: LocalPhysicalPlanRef,
     pub aggregations: Vec<ExprRef>,
@@ -558,7 +730,7 @@ pub struct UnGroupedAggregate {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct HashAggregate {
     pub input: LocalPhysicalPlanRef,
     pub aggregations: Vec<ExprRef>,
@@ -567,7 +739,7 @@ pub struct HashAggregate {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Unpivot {
     pub input: LocalPhysicalPlanRef,
     pub ids: Vec<ExprRef>,
@@ -578,7 +750,7 @@ pub struct Unpivot {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Pivot {
     pub input: LocalPhysicalPlanRef,
     pub group_by: Vec<ExprRef>,
@@ -590,7 +762,7 @@ pub struct Pivot {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct HashJoin {
     pub left: LocalPhysicalPlanRef,
     pub right: LocalPhysicalPlanRef,
@@ -602,7 +774,7 @@ pub struct HashJoin {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct CrossJoin {
     pub left: LocalPhysicalPlanRef,
     pub right: LocalPhysicalPlanRef,
@@ -610,7 +782,7 @@ pub struct CrossJoin {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct Concat {
     pub input: LocalPhysicalPlanRef,
     pub other: LocalPhysicalPlanRef,
@@ -618,7 +790,7 @@ pub struct Concat {
     pub stats_state: StatsState,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct PhysicalWrite {
     pub input: LocalPhysicalPlanRef,
     pub data_schema: SchemaRef,
@@ -628,7 +800,7 @@ pub struct PhysicalWrite {
 }
 
 #[cfg(feature = "python")]
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct CatalogWrite {
     pub input: LocalPhysicalPlanRef,
     pub catalog_type: daft_logical_plan::CatalogType,
@@ -638,7 +810,7 @@ pub struct CatalogWrite {
 }
 
 #[cfg(feature = "python")]
-#[derive(Debug)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct LanceWrite {
     pub input: LocalPhysicalPlanRef,
     pub lance_info: daft_logical_plan::LanceCatalogInfo,
@@ -646,3 +818,59 @@ pub struct LanceWrite {
     pub file_schema: SchemaRef,
     pub stats_state: StatsState,
 }
+
+#[cfg(feature = "python")]
+#[derive(Debug, Serialize, Deserialize)]
+pub struct DataSink {
+    pub input: LocalPhysicalPlanRef,
+    pub data_sink_info: daft_logical_plan::DataSinkInfo,
+    pub file_schema: SchemaRef,
+    pub stats_state: StatsState,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+pub struct WindowPartitionOnly {
+    pub input: LocalPhysicalPlanRef,
+    pub partition_by: Vec<ExprRef>,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+    pub aggregations: Vec<AggExpr>,
+    pub aliases: Vec<String>,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+pub struct WindowPartitionAndOrderBy {
+    pub input: LocalPhysicalPlanRef,
+    pub partition_by: Vec<ExprRef>,
+    pub order_by: Vec<ExprRef>,
+    pub descending: Vec<bool>,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+    pub functions: Vec<WindowExpr>,
+    pub aliases: Vec<String>,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+pub struct WindowPartitionAndDynamicFrame {
+    pub input: LocalPhysicalPlanRef,
+    pub partition_by: Vec<ExprRef>,
+    pub order_by: Vec<ExprRef>,
+    pub descending: Vec<bool>,
+    pub frame: WindowFrame,
+    pub min_periods: usize,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+    pub aggregations: Vec<AggExpr>,
+    pub aliases: Vec<String>,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+pub struct WindowOrderByOnly {
+    pub input: LocalPhysicalPlanRef,
+    pub order_by: Vec<ExprRef>,
+    pub descending: Vec<bool>,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+    pub functions: Vec<WindowExpr>,
+    pub aliases: Vec<String>,
+}
