@@ -6,14 +6,18 @@ use std::{
 };
 
 use common_display::ascii::AsciiTreeDisplay;
-use common_error::DaftError;
-use daft_dsl::{optimization::get_required_columns, Subquery, SubqueryPlan};
-use daft_schema::schema::SchemaRef;
+use common_error::{DaftError, DaftResult};
+use common_treenode::{TreeNode, TreeNodeRecursion};
+use daft_core::join::JoinSide;
+use daft_dsl::{
+    optimization::get_required_columns, Column, Expr, ResolvedColumn, Subquery, SubqueryPlan,
+};
+use daft_schema::{field::Field, schema::SchemaRef};
 use indexmap::IndexSet;
 use snafu::Snafu;
 
 pub use crate::ops::*;
-use crate::stats::PlanStats;
+use crate::stats::{PlanStats, StatsState};
 
 /// Logical plan for a Daft query.
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
@@ -37,10 +41,39 @@ pub enum LogicalPlan {
     Sink(Sink),
     Sample(Sample),
     MonotonicallyIncreasingId(MonotonicallyIncreasingId),
+    SubqueryAlias(SubqueryAlias),
+    Window(Window),
+    TopN(TopN),
 }
 
 pub type LogicalPlanRef = Arc<LogicalPlan>;
 
+#[derive(Clone, Debug, PartialEq, Eq, Hash)]
+pub struct SubqueryAlias {
+    pub plan_id: Option<usize>,
+    pub input: LogicalPlanRef,
+    pub name: Arc<str>,
+}
+
+impl SubqueryAlias {
+    pub fn new(input: LogicalPlanRef, name: impl Into<Arc<str>>) -> Self {
+        Self {
+            plan_id: None,
+            input,
+            name: name.into(),
+        }
+    }
+
+    pub fn with_plan_id(mut self, plan_id: usize) -> Self {
+        self.plan_id = Some(plan_id);
+        self
+    }
+
+    pub fn multiline_display(&self) -> Vec<String> {
+        vec![format!("Alias"), format!("name = {}", self.name)]
+    }
+}
+
 impl LogicalPlan {
     pub fn arced(self) -> Arc<Self> {
         Arc::new(self)
@@ -75,6 +108,9 @@ impl LogicalPlan {
             Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { schema, .. }) => {
                 schema.clone()
             }
+            Self::SubqueryAlias(SubqueryAlias { input, .. }) => input.schema(),
+            Self::Window(Window { schema, .. }) => schema.clone(),
+            Self::TopN(TopN { input, .. }) => input.schema(),
         }
     }
 
@@ -137,10 +173,8 @@ impl LogicalPlan {
                 let res = distinct
                     .input
                     .schema()
-                    .fields
-                    .iter()
-                    .map(|(name, _)| name)
-                    .cloned()
+                    .field_names()
+                    .map(ToString::to_string)
                     .collect();
                 vec![res]
             }
@@ -165,18 +199,56 @@ impl LogicalPlan {
                 vec![res]
             }
             Self::Join(join) => {
-                let left = join.left_on.iter().flat_map(get_required_columns).collect();
-                let right = join
-                    .right_on
-                    .iter()
-                    .flat_map(get_required_columns)
-                    .collect();
+                let mut left = IndexSet::new();
+                let mut right = IndexSet::new();
+
+                if let Some(pred) = join.on.inner() {
+                    pred.apply(|e| {
+                        match e.as_ref() {
+                            Expr::Column(Column::Resolved(ResolvedColumn::JoinSide(
+                                Field { name, .. },
+                                JoinSide::Left,
+                            ))) => {
+                                left.insert(name.clone());
+                            }
+                            Expr::Column(Column::Resolved(ResolvedColumn::JoinSide(
+                                Field { name, .. },
+                                JoinSide::Right,
+                            ))) => {
+                                right.insert(name.clone());
+                            }
+                            _ => {}
+                        }
+
+                        Ok(TreeNodeRecursion::Continue)
+                    })
+                    .unwrap();
+                }
                 vec![left, right]
             }
             Self::Intersect(_) => vec![IndexSet::new(), IndexSet::new()],
             Self::Union(_) => vec![IndexSet::new(), IndexSet::new()],
             Self::Source(_) => todo!(),
             Self::Sink(_) => todo!(),
+            Self::SubqueryAlias(SubqueryAlias { input, .. }) => input.required_columns(),
+            Self::Window(window) => {
+                let res = window
+                    .window_spec
+                    .partition_by
+                    .iter()
+                    .chain(window.window_spec.order_by.iter())
+                    .flat_map(get_required_columns)
+                    .collect();
+                vec![res]
+            }
+            Self::TopN(top_n) => {
+                let res = top_n
+                    .sort_by
+                    .iter()
+                    .flat_map(get_required_columns)
+                    .collect();
+                vec![res]
+            }
         }
     }
 
@@ -201,10 +273,13 @@ impl LogicalPlan {
             Self::Sink(..) => "Sink",
             Self::Sample(..) => "Sample",
             Self::MonotonicallyIncreasingId(..) => "MonotonicallyIncreasingId",
+            Self::SubqueryAlias(..) => "Alias",
+            Self::Window(..) => "Window",
+            Self::TopN(..) => "TopN",
         }
     }
 
-    pub fn materialized_stats(&self) -> &PlanStats {
+    pub fn stats_state(&self) -> &StatsState {
         match self {
             Self::Source(Source { stats_state, .. })
             | Self::Project(Project { stats_state, .. })
@@ -222,18 +297,25 @@ impl LogicalPlan {
             | Self::Join(Join { stats_state, .. })
             | Self::Sink(Sink { stats_state, .. })
             | Self::Sample(Sample { stats_state, .. })
-            | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { stats_state, .. }) => {
-                stats_state.materialized_stats()
-            }
+            | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { stats_state, .. })
+            | Self::Window(Window { stats_state, .. })
+            | Self::TopN(TopN { stats_state, .. }) => stats_state,
             Self::Intersect(_) => {
                 panic!("Intersect nodes should be optimized away before stats are materialized")
             }
             Self::Union(_) => {
                 panic!("Union nodes should be optimized away before stats are materialized")
             }
+            Self::SubqueryAlias(_) => {
+                panic!("Alias nodes should be optimized away before stats are materialized")
+            }
         }
     }
 
+    pub fn materialized_stats(&self) -> &PlanStats {
+        self.stats_state().materialized_stats()
+    }
+
     // Materializes stats over logical plans. If stats are already materialized, this function recomputes stats, which might be
     // useful if stats become stale during query planning.
     pub fn with_materialized_stats(self) -> Self {
@@ -257,12 +339,17 @@ impl LogicalPlan {
             Self::Union(_) => {
                 panic!("Union should be optimized away before stats are derived")
             }
+            Self::SubqueryAlias(_) => {
+                panic!("Alias should be optimized away before stats are derived")
+            }
             Self::Join(plan) => Self::Join(plan.with_materialized_stats()),
             Self::Sink(plan) => Self::Sink(plan.with_materialized_stats()),
             Self::Sample(plan) => Self::Sample(plan.with_materialized_stats()),
             Self::MonotonicallyIncreasingId(plan) => {
                 Self::MonotonicallyIncreasingId(plan.with_materialized_stats())
             }
+            Self::Window(plan) => Self::Window(plan.with_materialized_stats()),
+            Self::TopN(plan) => Self::TopN(plan.with_materialized_stats()),
         }
     }
 
@@ -289,6 +376,9 @@ impl LogicalPlan {
             Self::MonotonicallyIncreasingId(monotonically_increasing_id) => {
                 monotonically_increasing_id.multiline_display()
             }
+            Self::SubqueryAlias(alias) => alias.multiline_display(),
+            Self::Window(window) => window.multiline_display(),
+            Self::TopN(top_n) => top_n.multiline_display(),
         }
     }
 
@@ -315,6 +405,9 @@ impl LogicalPlan {
             Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { input, .. }) => {
                 vec![input]
             }
+            Self::SubqueryAlias(SubqueryAlias { input, .. }) => vec![input],
+            Self::Window(Window { input, .. }) => vec![input],
+            Self::TopN(TopN { input, .. }) => vec![input],
         }
     }
 
@@ -330,15 +423,25 @@ impl LogicalPlan {
                 Self::Limit(Limit { limit, eager, .. }) => Self::Limit(Limit::new(input.clone(), *limit, *eager)),
                 Self::Explode(Explode { to_explode, .. }) => Self::Explode(Explode::try_new(input.clone(), to_explode.clone()).unwrap()),
                 Self::Sort(Sort { sort_by, descending, nulls_first, .. }) => Self::Sort(Sort::try_new(input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone()).unwrap()),
-                Self::Repartition(Repartition {  repartition_spec: scheme_config, .. }) => Self::Repartition(Repartition::try_new(input.clone(), scheme_config.clone()).unwrap()),
+                Self::Repartition(Repartition {  repartition_spec: scheme_config, .. }) => Self::Repartition(Repartition::new(input.clone(), scheme_config.clone())),
                 Self::Distinct(_) => Self::Distinct(Distinct::new(input.clone())),
                 Self::Aggregate(Aggregate { aggregations, groupby, ..}) => Self::Aggregate(Aggregate::try_new(input.clone(), aggregations.clone(), groupby.clone()).unwrap()),
                 Self::Pivot(Pivot { group_by, pivot_column, value_column, aggregation, names, ..}) => Self::Pivot(Pivot::try_new(input.clone(), group_by.clone(), pivot_column.clone(), value_column.clone(), aggregation.into(), names.clone()).unwrap()),
                 Self::Sink(Sink { sink_info, .. }) => Self::Sink(Sink::try_new(input.clone(), sink_info.clone()).unwrap()),
-                Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId {column_name, .. }) => Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId::new(input.clone(), Some(column_name))),
+                Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId {column_name, .. }) => Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId::try_new(input.clone(), Some(column_name)).unwrap()),
                 Self::Unpivot(Unpivot {ids, values, variable_name, value_name, output_schema, ..}) =>
                     Self::Unpivot(Unpivot::new(input.clone(), ids.clone(), values.clone(), variable_name.clone(), value_name.clone(), output_schema.clone())),
                 Self::Sample(Sample {fraction, with_replacement, seed, ..}) => Self::Sample(Sample::new(input.clone(), *fraction, *with_replacement, *seed)),
+                Self::SubqueryAlias(SubqueryAlias { name: id, .. }) => Self::SubqueryAlias(SubqueryAlias::new(input.clone(), id.clone())),
+                Self::Window(Window { window_functions, aliases, window_spec, .. }) => Self::Window(Window::try_new(
+                    input.clone(),
+                    window_functions.clone(),
+                    aliases.clone(),
+                    window_spec.clone(),
+                ).unwrap()),
+                Self::TopN(TopN { sort_by, descending, nulls_first, limit, .. }) => Self::TopN(TopN::try_new(
+                    input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone(), *limit
+                ).unwrap()),
                 Self::Concat(_) => panic!("Concat ops should never have only one input, but got one"),
                 Self::Intersect(_) => panic!("Intersect ops should never have only one input, but got one"),
                 Self::Union(_) => panic!("Union ops should never have only one input, but got one"),
@@ -348,18 +451,13 @@ impl LogicalPlan {
                 Self::Source(_) => panic!("Source nodes don't have children, with_new_children() should never be called for Source ops"),
                 Self::Concat(_) => Self::Concat(Concat::try_new(input1.clone(), input2.clone()).unwrap()),
                 Self::Intersect(inner) => Self::Intersect(Intersect::try_new(input1.clone(), input2.clone(), inner.is_all).unwrap()),
-                Self::Union(inner) => Self::Union(Union::try_new(input1.clone(), input2.clone(), inner.is_all).unwrap()),
-                Self::Join(Join { left_on, right_on, null_equals_nulls, join_type, join_strategy, .. }) => Self::Join(Join::try_new(
+                Self::Union(inner) => Self::Union(Union::try_new(input1.clone(), input2.clone(), inner.quantifier, inner.strategy).unwrap()),
+                Self::Join(Join { on, join_type, join_strategy, .. }) => Self::Join(Join::try_new(
                     input1.clone(),
                     input2.clone(),
-                    left_on.clone(),
-                    right_on.clone(),
-                    null_equals_nulls.clone(),
+                    on.clone(),
                     *join_type,
                     *join_strategy,
-                    None,  // The suffix is already eagerly computed in the constructor
-                    None,  // the prefix is already eagerly computed in the constructor
-                    false // this is already eagerly computed in the constructor
                 ).unwrap()),
                 _ => panic!("Logical op {} has one input, but got two", self),
             },
@@ -394,6 +492,140 @@ impl LogicalPlan {
         self.fmt_tree_indent_style(0, &mut s).unwrap();
         s
     }
+
+    pub fn get_aliases(self: Arc<Self>) -> Vec<Arc<str>> {
+        use common_treenode::TreeNode;
+
+        let mut names = Vec::new();
+
+        self.apply(|node| {
+            if let Self::SubqueryAlias(SubqueryAlias { name, .. }) = node.as_ref() {
+                names.push(name.clone());
+                Ok(TreeNodeRecursion::Jump)
+            } else {
+                Ok(TreeNodeRecursion::Continue)
+            }
+        })
+        .unwrap();
+
+        names
+    }
+
+    pub fn get_schema_for_alias(self: &Arc<Self>, alias: &str) -> DaftResult<Option<SchemaRef>> {
+        use common_treenode::TreeNode;
+
+        let mut schema = None;
+
+        self.apply(|node| {
+            if let Self::SubqueryAlias(SubqueryAlias { name, .. }) = node.as_ref() {
+                if name.as_ref() == alias {
+                    if schema.is_some() {
+                        return Err(DaftError::ValueError(format!(
+                            "Plan must not have duplicate aliases in the same scope, found: {alias}"
+                        )));
+                    }
+
+                    schema = Some(node.schema());
+                }
+
+                Ok(TreeNodeRecursion::Jump)
+            } else {
+                Ok(TreeNodeRecursion::Continue)
+            }
+        })?;
+
+        Ok(schema)
+    }
+
+    pub fn get_schema_for_id(self: &Arc<Self>, id: usize) -> DaftResult<Option<SchemaRef>> {
+        use common_treenode::TreeNode;
+
+        let mut schema = None;
+
+        self.apply(|node| {
+            if let Some(plan_id) = node.plan_id() {
+                if plan_id == &id {
+                    if schema.is_some() {
+                        return Err(DaftError::ValueError(format!(
+                            "Plan must not have duplicate plan ids in the same scope, found: {id}"
+                        )));
+                    }
+
+                    schema = Some(node.schema());
+
+                    Ok(TreeNodeRecursion::Jump)
+                } else {
+                    Ok(TreeNodeRecursion::Continue)
+                }
+            } else {
+                Ok(TreeNodeRecursion::Continue)
+            }
+        })?;
+
+        Ok(schema)
+    }
+
+    pub fn plan_id(&self) -> &Option<usize> {
+        match self {
+            Self::Source(Source { plan_id, .. })
+            | Self::Project(Project { plan_id, .. })
+            | Self::ActorPoolProject(ActorPoolProject { plan_id, .. })
+            | Self::Filter(Filter { plan_id, .. })
+            | Self::Limit(Limit { plan_id, .. })
+            | Self::Explode(Explode { plan_id, .. })
+            | Self::Unpivot(Unpivot { plan_id, .. })
+            | Self::Sort(Sort { plan_id, .. })
+            | Self::Repartition(Repartition { plan_id, .. })
+            | Self::Distinct(Distinct { plan_id, .. })
+            | Self::Aggregate(Aggregate { plan_id, .. })
+            | Self::Pivot(Pivot { plan_id, .. })
+            | Self::Concat(Concat { plan_id, .. })
+            | Self::Intersect(Intersect { plan_id, .. })
+            | Self::Union(Union { plan_id, .. })
+            | Self::Join(Join { plan_id, .. })
+            | Self::Sink(Sink { plan_id, .. })
+            | Self::Sample(Sample { plan_id, .. })
+            | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { plan_id, .. })
+            | Self::SubqueryAlias(SubqueryAlias { plan_id, .. })
+            | Self::Window(Window { plan_id, .. })
+            | Self::TopN(TopN { plan_id, .. }) => plan_id,
+        }
+    }
+
+    pub fn with_plan_id(self: Arc<Self>, plan_id: usize) -> Self {
+        match self.as_ref() {
+            Self::Source(source) => Self::Source(source.clone().with_plan_id(plan_id)),
+            Self::Project(project) => Self::Project(project.clone().with_plan_id(plan_id)),
+            Self::ActorPoolProject(project) => {
+                Self::ActorPoolProject(project.clone().with_plan_id(plan_id))
+            }
+            Self::Filter(filter) => Self::Filter(filter.clone().with_plan_id(plan_id)),
+            Self::Limit(limit) => Self::Limit(limit.clone().with_plan_id(plan_id)),
+            Self::Explode(explode) => Self::Explode(explode.clone().with_plan_id(plan_id)),
+            Self::Unpivot(unpivot) => Self::Unpivot(unpivot.clone().with_plan_id(plan_id)),
+            Self::Sort(sort) => Self::Sort(sort.clone().with_plan_id(plan_id)),
+            Self::Repartition(repartition) => {
+                Self::Repartition(repartition.clone().with_plan_id(plan_id))
+            }
+            Self::Distinct(distinct) => Self::Distinct(distinct.clone().with_plan_id(plan_id)),
+            Self::Aggregate(aggregate) => Self::Aggregate(aggregate.clone().with_plan_id(plan_id)),
+            Self::Pivot(pivot) => Self::Pivot(pivot.clone().with_plan_id(plan_id)),
+            Self::Concat(concat) => Self::Concat(concat.clone().with_plan_id(plan_id)),
+            Self::Intersect(intersect) => Self::Intersect(intersect.clone().with_plan_id(plan_id)),
+            Self::Union(union) => Self::Union(union.clone().with_plan_id(plan_id)),
+            Self::Join(join) => Self::Join(join.clone().with_plan_id(plan_id)),
+            Self::Sink(sink) => Self::Sink(sink.clone().with_plan_id(plan_id)),
+            Self::Sample(sample) => Self::Sample(sample.clone().with_plan_id(plan_id)),
+            Self::MonotonicallyIncreasingId(monotonically_increasing_id) => {
+                Self::MonotonicallyIncreasingId(
+                    monotonically_increasing_id.clone().with_plan_id(plan_id),
+                )
+            }
+            Self::SubqueryAlias(alias) => Self::SubqueryAlias(alias.clone().with_plan_id(plan_id)),
+            Self::Window(window) => window.with_plan_id(Some(plan_id)),
+            Self::TopN(top_n) => Self::TopN(top_n.clone().with_plan_id(plan_id)),
+        }
+    }
 }
 
 impl SubqueryPlan for LogicalPlan {
@@ -417,7 +649,7 @@ impl SubqueryPlan for LogicalPlan {
         other
             .as_any()
             .downcast_ref::<Self>()
-            .map_or(false, |other| self == other)
+            .is_some_and(|other| self == other)
     }
 
     fn dyn_hash(&self, mut state: &mut dyn Hasher) {
@@ -495,3 +727,5 @@ impl_from_data_struct_for_logical_plan!(Join);
 impl_from_data_struct_for_logical_plan!(Sink);
 impl_from_data_struct_for_logical_plan!(Sample);
 impl_from_data_struct_for_logical_plan!(MonotonicallyIncreasingId);
+impl_from_data_struct_for_logical_plan!(Window);
+impl_from_data_struct_for_logical_plan!(TopN);
