@@ -4,7 +4,7 @@ mod concat;
 mod distinct;
 mod explode;
 mod filter;
-mod join;
+pub mod join;
 mod limit;
 mod monotonically_increasing_id;
 mod pivot;
@@ -15,7 +15,10 @@ mod set_operations;
 mod sink;
 mod sort;
 mod source;
+mod summarize;
+mod top_n;
 mod unpivot;
+mod window;
 
 pub use actor_pool_project::ActorPoolProject;
 pub use agg::Aggregate;
@@ -30,8 +33,11 @@ pub use pivot::Pivot;
 pub use project::Project;
 pub use repartition::Repartition;
 pub use sample::Sample;
-pub use set_operations::{Intersect, Union};
+pub use set_operations::{Except, Intersect, SetQuantifier, Union, UnionStrategy};
 pub use sink::Sink;
 pub use sort::Sort;
 pub use source::Source;
+pub use summarize::summarize;
+pub use top_n::TopN;
 pub use unpivot::Unpivot;
+pub use window::Window;
