@@ -1,16 +1,10 @@
 use std::sync::Arc;
 
-use common_error::DaftResult;
-use daft_dsl::ExprResolver;
-
-use crate::{
-    partitioning::{HashRepartitionConfig, RepartitionSpec},
-    stats::StatsState,
-    LogicalPlan,
-};
+use crate::{partitioning::RepartitionSpec, stats::StatsState, LogicalPlan};
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
 pub struct Repartition {
+    pub plan_id: Option<usize>,
     // Upstream node.
     pub input: Arc<LogicalPlan>,
     pub repartition_spec: RepartitionSpec,
@@ -18,28 +12,18 @@ pub struct Repartition {
 }
 
 impl Repartition {
-    pub(crate) fn try_new(
-        input: Arc<LogicalPlan>,
-        repartition_spec: RepartitionSpec,
-    ) -> DaftResult<Self> {
-        let repartition_spec = match repartition_spec {
-            RepartitionSpec::Hash(HashRepartitionConfig { num_partitions, by }) => {
-                let expr_resolver = ExprResolver::default();
-
-                let (resolved_by, _) = expr_resolver.resolve(by, &input.schema())?;
-                RepartitionSpec::Hash(HashRepartitionConfig {
-                    num_partitions,
-                    by: resolved_by,
-                })
-            }
-            RepartitionSpec::Random(_) | RepartitionSpec::IntoPartitions(_) => repartition_spec,
-        };
-
-        Ok(Self {
+    pub(crate) fn new(input: Arc<LogicalPlan>, repartition_spec: RepartitionSpec) -> Self {
+        Self {
+            plan_id: None,
             input,
             repartition_spec,
             stats_state: StatsState::NotMaterialized,
-        })
+        }
+    }
+
+    pub fn with_plan_id(mut self, plan_id: usize) -> Self {
+        self.plan_id = Some(plan_id);
+        self
     }
 
     pub(crate) fn with_materialized_stats(mut self) -> Self {
