@@ -11,7 +11,7 @@ use super::{
         MaterializeScans, OptimizerRule, PushDownAntiSemiJoin, PushDownFilter,
         PushDownJoinPredicate, PushDownLimit, PushDownProjection, ReorderJoins,
         SimplifyExpressionsRule, SimplifyNullFilteredJoin, SplitActorPoolProjects,
-        UnnestPredicateSubquery, UnnestScalarSubquery,
+        SplitGranularProjection, UnnestPredicateSubquery, UnnestScalarSubquery,
     },
 };
 use crate::LogicalPlan;
@@ -208,6 +208,14 @@ impl OptimizerBuilder {
         self
     }
 
+    pub fn split_granular_projections(mut self) -> Self {
+        self.rule_batches.push(RuleBatch::new(
+            vec![Box::new(SplitGranularProjection::new())],
+            RuleExecutionStrategy::Once,
+        ));
+        self
+    }
+
     pub fn with_optimizer_config(mut self, config: OptimizerConfig) -> Self {
         self.config = config;
         self
