@@ -4,19 +4,17 @@ use std::{
 };
 
 use common_error::DaftResult;
-use common_scan_info::{rewrite_predicate_for_partitioning, PredicateGroups};
+use common_scan_info::{rewrite_predicate_for_partitioning, PredicateGroups, ScanState};
 use common_treenode::{DynTreeNode, Transformed, TreeNode};
-use daft_algebra::boolean::{combine_conjunction, split_conjunction};
-use daft_core::join::JoinType;
+use daft_algebra::boolean::{combine_conjunction, split_conjunction, to_cnf};
 use daft_dsl::{
-    col,
     optimization::{get_required_columns, replace_columns_with_expressions},
-    ExprRef,
+    resolved_col, ExprRef,
 };
 
 use super::OptimizerRule;
 use crate::{
-    ops::{Concat, Filter, Project, Source},
+    ops::{Concat, Filter, Join, Project, Source},
     source_info::SourceInfo,
     LogicalPlan,
 };
@@ -89,6 +87,10 @@ impl PushDownFilter {
 
                     // Pushdown filter into the Source node
                     SourceInfo::Physical(external_info) => {
+                        // If the scan is materialized, we don't pushdown the filter.
+                        if matches!(external_info.scan_state, ScanState::Tasks(_)) {
+                            return Ok(Transformed::no(plan));
+                        }
                         let predicate = &filter.predicate;
                         let new_predicate = external_info
                             .pushdowns
@@ -179,7 +181,10 @@ impl PushDownFilter {
                 let projection_input_mapping = child_project
                     .projection
                     .iter()
-                    .filter_map(|e| e.input_mapping().map(|s| (e.name().to_string(), col(s))))
+                    .filter_map(|e| {
+                        e.input_mapping()
+                            .map(|s| (e.name().to_string(), resolved_col(s)))
+                    })
                     .collect::<HashMap<String, ExprRef>>();
                 // Split predicate expressions into those that don't depend on projection compute (can_push) and those
                 // that do (can_not_push).
@@ -248,32 +253,27 @@ impl PushDownFilter {
                     Concat::try_new(new_input.into(), new_other.into())?.into();
                 new_concat.into()
             }
-            LogicalPlan::Join(child_join) => {
+            LogicalPlan::Join(Join {
+                left,
+                right,
+                join_type,
+                on,
+                join_strategy,
+                ..
+            }) => {
                 // TODO(Kevin): add more filter pushdowns for joins
-                // Example 1:
+                // Example:
                 //      For `foo JOIN bar ON foo.a == (bar.b + 2) WHERE a > 0`, the filter `a > 0` is pushed down to the left side, but can also be pushed down to the right side as `(b + 2) > 0`
-                //
-                // Example 2:
-                //      A predicate `(a AND b) OR (c AND d)` is equivalent to `((a AND b) OR (c AND d)) AND (a OR c)`, and `a OR c` could potentially be pushed down.
-
-                // if a filter is pushed down on one side, would it preserve the output of the join+filter?
-                let (left_preserved, right_preserved) = match child_join.join_type {
-                    JoinType::Inner => (true, true),
-                    JoinType::Left => (true, false),
-                    JoinType::Right => (false, true),
-                    JoinType::Outer => (false, false),
-                    JoinType::Anti => (true, true),
-                    JoinType::Semi => (true, true),
-                };
 
                 let mut left_pushdowns = vec![];
                 let mut right_pushdowns = vec![];
                 let mut kept_predicates = vec![];
 
-                let left_cols = HashSet::<_>::from_iter(child_join.left.schema().names());
-                let right_cols = HashSet::<_>::from_iter(child_join.right.schema().names());
+                let left_cols = HashSet::<_>::from_iter(left.schema().names());
+                let right_cols = HashSet::<_>::from_iter(right.schema().names());
 
-                for predicate in split_conjunction(&filter.predicate) {
+                // TODO: simplify predicates, since they may be expanded with `to_cnf`
+                for predicate in split_conjunction(&to_cnf(filter.predicate.clone())) {
                     let pred_cols = HashSet::<_>::from_iter(get_required_columns(&predicate));
 
                     match (
@@ -290,17 +290,17 @@ impl PushDownFilter {
                             kept_predicates.push(predicate);
                         }
                         (true, false) => {
-                            if left_preserved {
-                                left_pushdowns.push(predicate);
-                            } else {
+                            if join_type.left_produces_nulls() {
                                 kept_predicates.push(predicate);
+                            } else {
+                                left_pushdowns.push(predicate);
                             }
                         }
                         (false, true) => {
-                            if right_preserved {
-                                right_pushdowns.push(predicate);
-                            } else {
+                            if join_type.right_produces_nulls() {
                                 kept_predicates.push(predicate);
+                            } else {
+                                right_pushdowns.push(predicate);
                             }
                         }
                     }
@@ -313,24 +313,30 @@ impl PushDownFilter {
                     let kept_predicates = combine_conjunction(kept_predicates);
 
                     let new_left = left_pushdowns.map_or_else(
-                        || child_join.left.clone(),
+                        || left.clone(),
                         |left_pushdowns| {
-                            Filter::try_new(child_join.left.clone(), left_pushdowns)
+                            Filter::try_new(left.clone(), left_pushdowns)
                                 .unwrap()
                                 .into()
                         },
                     );
 
                     let new_right = right_pushdowns.map_or_else(
-                        || child_join.right.clone(),
+                        || right.clone(),
                         |right_pushdowns| {
-                            Filter::try_new(child_join.right.clone(), right_pushdowns)
+                            Filter::try_new(right.clone(), right_pushdowns)
                                 .unwrap()
                                 .into()
                         },
                     );
 
-                    let new_join = child_plan.with_new_children(&[new_left, new_right]).into();
+                    let new_join = Arc::new(LogicalPlan::Join(Join::try_new(
+                        new_left,
+                        new_right,
+                        on.clone(),
+                        *join_type,
+                        *join_strategy,
+                    )?));
 
                     if let Some(kept_predicates) = kept_predicates {
                         Filter::try_new(new_join, kept_predicates).unwrap().into()
@@ -354,7 +360,8 @@ mod tests {
     use common_error::DaftResult;
     use common_scan_info::Pushdowns;
     use daft_core::prelude::*;
-    use daft_dsl::{col, lit};
+    use daft_dsl::{lit, resolved_col, unresolved_col};
+    use daft_functions::uri::download::UrlDownloadArgs;
     use rstest::rstest;
 
     use crate::{
@@ -393,7 +400,7 @@ mod tests {
         ]);
         let plan =
             dummy_scan_node_with_pushdowns(scan_op, Pushdowns::default().with_limit(Some(1)))
-                .filter(col("a").lt(lit(2)))?
+                .filter(resolved_col("a").lt(lit(2)))?
                 .build();
         // Plan should be unchanged after optimization.
         let expected = plan.clone();
@@ -404,6 +411,8 @@ mod tests {
     /// Tests combining of two Filters by merging their predicates.
     #[rstest]
     fn filter_combine_with_filter(#[values(false, true)] push_into_scan: bool) -> DaftResult<()> {
+        use daft_dsl::resolved_col;
+
         let scan_op = dummy_scan_operator(vec![
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Utf8),
@@ -412,8 +421,8 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_scan { None } else { Some(1) }),
         );
-        let p1 = col("a").lt(lit(2));
-        let p2 = col("b").eq(lit("foo"));
+        let p1 = resolved_col("a").lt(lit(2));
+        let p2 = resolved_col("b").eq(lit("foo"));
         let plan = scan_plan.filter(p1.clone())?.filter(p2.clone())?.build();
         let merged_filter = p2.and(p1);
         let expected = if push_into_scan {
@@ -434,7 +443,10 @@ mod tests {
     /// Tests that we can't pushdown a filter into a ScanOperator if it has an udf-ish expression.
     #[test]
     fn filter_with_udf_not_pushed_down_into_scan() -> DaftResult<()> {
-        let pred = daft_functions::uri::download(col("a"), 1, true, true, None);
+        let pred = daft_functions::uri::download(
+            resolved_col("a"),
+            Some(UrlDownloadArgs::new(1, true, true, None)),
+        );
         let plan = dummy_scan_node(dummy_scan_operator(vec![
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Utf8),
@@ -459,8 +471,8 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_scan { None } else { Some(1) }),
         );
-        let pred = col("a").lt(lit(2));
-        let proj = vec![col("a")];
+        let pred = resolved_col("a").lt(lit(2));
+        let proj = vec![resolved_col("a")];
         let plan = scan_plan
             .select(proj.clone())?
             .filter(pred.clone())?
@@ -488,8 +500,10 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_scan { None } else { Some(1) }),
         );
-        let pred = col("a").lt(lit(2)).and(col("b").eq(lit("foo")));
-        let proj = vec![col("a"), col("b")];
+        let pred = resolved_col("a")
+            .lt(lit(2))
+            .and(resolved_col("b").eq(lit("foo")));
+        let proj = vec![resolved_col("a"), resolved_col("b")];
         let plan = scan_plan
             .select(proj.clone())?
             .filter(pred.clone())?
@@ -512,8 +526,8 @@ mod tests {
             Field::new("b", DataType::Utf8),
         ]))
         // Projection involves compute on filtered column "a".
-        .select(vec![col("a").add(lit(1))])?
-        .filter(col("a").lt(lit(2)))?
+        .select(vec![resolved_col("a").add(lit(1))])?
+        .filter(resolved_col("a").lt(lit(2)))?
         .build();
         // Filter should NOT commute with Project, since this would involve redundant computation.
         let expected = plan.clone();
@@ -536,8 +550,8 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_scan { None } else { Some(1) }),
         );
-        let pred = col("a").lt(lit(2));
-        let proj = vec![col("a").add(lit(1))];
+        let pred = resolved_col("a").lt(lit(2));
+        let proj = vec![resolved_col("a").add(lit(1))];
         let plan = scan_plan
             // Projection involves compute on filtered column "a".
             .select(proj.clone())?
@@ -564,8 +578,8 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_scan { None } else { Some(1) }),
         );
-        let pred = col("a").lt(lit(2));
-        let sort_by = vec![col("a")];
+        let pred = resolved_col("a").lt(lit(2));
+        let sort_by = vec![resolved_col("a")];
         let descending = vec![true];
         let nulls_first = vec![false];
         let plan = scan_plan
@@ -597,9 +611,9 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_scan { None } else { Some(1) }),
         );
-        let pred = col("a").lt(lit(2));
+        let pred = resolved_col("a").lt(lit(2));
         let num_partitions = 1;
-        let repartition_by = vec![col("a")];
+        let repartition_by = vec![resolved_col("a")];
         let plan = scan_plan
             .hash_repartition(Some(num_partitions), repartition_by.clone())?
             .filter(pred.clone())?
@@ -634,7 +648,7 @@ mod tests {
             scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_right_scan { None } else { Some(1) }),
         );
-        let pred = col("a").lt(lit(2));
+        let pred = resolved_col("a").lt(lit(2));
         let plan = left_scan_plan
             .concat(&right_scan_plan)?
             .filter(pred.clone())?
@@ -671,7 +685,7 @@ mod tests {
             Field::new("b", DataType::Utf8),
         ]);
         let right_scan_op = dummy_scan_operator(vec![
-            Field::new("b", DataType::Utf8),
+            Field::new("right.b", DataType::Utf8),
             Field::new("c", DataType::Float64),
         ]);
         let left_scan_plan = dummy_scan_node_with_pushdowns(
@@ -679,24 +693,21 @@ mod tests {
             Pushdowns::default().with_limit(if push_into_left_scan { None } else { Some(1) }),
         );
         let right_scan_plan = dummy_scan_node(right_scan_op.clone());
-        let join_on = vec![col("b")];
-        let null_equals_nulls = if null_equals_null {
-            Some(vec![true])
+        let join_on = if null_equals_null {
+            unresolved_col("b").eq_null_safe(unresolved_col("right.b"))
         } else {
-            None
+            unresolved_col("b").eq(unresolved_col("right.b"))
         };
-        let pred = col("a").lt(lit(2));
+
+        let pred = resolved_col("a").lt(lit(2));
         let plan = left_scan_plan
-            .join_with_null_safe_equal(
+            .join(
                 &right_scan_plan,
-                join_on.clone(),
-                join_on.clone(),
-                null_equals_nulls.clone(),
+                join_on.clone().into(),
+                vec![],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .filter(pred.clone())?
             .build();
@@ -709,16 +720,13 @@ mod tests {
             left_scan_plan.filter(pred)?
         };
         let expected = expected_left_filter_scan
-            .join_with_null_safe_equal(
+            .join(
                 &right_scan_plan,
-                join_on.clone(),
-                join_on,
-                null_equals_nulls,
+                join_on.into(),
+                vec![],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .build();
         assert_optimized_plan_eq(plan, expected)?;
@@ -737,7 +745,7 @@ mod tests {
             Field::new("b", DataType::Utf8),
         ]);
         let right_scan_op = dummy_scan_operator(vec![
-            Field::new("b", DataType::Utf8),
+            Field::new("right.b", DataType::Utf8),
             Field::new("c", DataType::Float64),
         ]);
         let left_scan_plan = dummy_scan_node(left_scan_op.clone());
@@ -745,24 +753,20 @@ mod tests {
             right_scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_right_scan { None } else { Some(1) }),
         );
-        let join_on = vec![col("b")];
-        let null_equals_nulls = if null_equals_null {
-            Some(vec![true])
+        let join_on = if null_equals_null {
+            unresolved_col("b").eq_null_safe(unresolved_col("right.b"))
         } else {
-            None
+            unresolved_col("b").eq(unresolved_col("right.b"))
         };
-        let pred = col("c").lt(lit(2.0));
+        let pred = resolved_col("c").lt(lit(2.0));
         let plan = left_scan_plan
-            .join_with_null_safe_equal(
+            .join(
                 &right_scan_plan,
-                join_on.clone(),
-                join_on.clone(),
-                null_equals_nulls.clone(),
+                join_on.clone().into(),
+                vec![],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .filter(pred.clone())?
             .build();
@@ -775,16 +779,13 @@ mod tests {
             right_scan_plan.filter(pred)?
         };
         let expected = left_scan_plan
-            .join_with_null_safe_equal(
+            .join(
                 &expected_right_filter_scan,
-                join_on.clone(),
-                join_on,
-                null_equals_nulls,
+                join_on.into(),
+                vec![],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .build();
         assert_optimized_plan_eq(plan, expected)?;
@@ -796,7 +797,6 @@ mod tests {
     fn filter_commutes_with_join_on_join_key(
         #[values(false, true)] push_into_left_scan: bool,
         #[values(false, true)] push_into_right_scan: bool,
-        #[values(false, true)] null_equals_null: bool,
         #[values(
             JoinType::Inner,
             JoinType::Left,
@@ -824,24 +824,16 @@ mod tests {
             right_scan_op.clone(),
             Pushdowns::default().with_limit(if push_into_right_scan { None } else { Some(1) }),
         );
-        let join_on = vec![col("b")];
-        let null_equals_nulls = if null_equals_null {
-            Some(vec![true])
-        } else {
-            None
-        };
-        let pred = col("b").lt(lit(2));
+
+        let pred = resolved_col("b").is_null();
         let plan = left_scan_plan
-            .join_with_null_safe_equal(
+            .join(
                 &right_scan_plan,
-                join_on.clone(),
-                join_on.clone(),
-                null_equals_nulls.clone(),
+                None,
+                vec!["b".to_string()],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .filter(pred.clone())?
             .build();
@@ -862,16 +854,13 @@ mod tests {
             right_scan_plan.filter(pred)?
         };
         let expected = expected_left_filter_scan
-            .join_with_null_safe_equal(
+            .join(
                 &expected_right_filter_scan,
-                join_on.clone(),
-                join_on,
-                null_equals_nulls,
+                None,
+                vec!["b".to_string()],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .build();
         assert_optimized_plan_eq(plan, expected)?;
@@ -881,7 +870,7 @@ mod tests {
     /// Tests that Filter can be pushed into the left side of a Join.
     #[rstest]
     fn filter_does_not_commute_with_join_left_side(
-        #[values(false, true)] null_equal_null: bool,
+        #[values(false, true)] null_equals_null: bool,
         #[values(JoinType::Right, JoinType::Outer)] how: JoinType,
     ) -> DaftResult<()> {
         let left_scan_op = dummy_scan_operator(vec![
@@ -889,29 +878,25 @@ mod tests {
             Field::new("b", DataType::Utf8),
         ]);
         let right_scan_op = dummy_scan_operator(vec![
-            Field::new("b", DataType::Utf8),
+            Field::new("right.b", DataType::Utf8),
             Field::new("c", DataType::Float64),
         ]);
         let left_scan_plan = dummy_scan_node(left_scan_op.clone());
         let right_scan_plan = dummy_scan_node(right_scan_op.clone());
-        let join_on = vec![col("b")];
-        let null_equals_nulls = if null_equal_null {
-            Some(vec![true])
+        let join_on = if null_equals_null {
+            unresolved_col("b").eq_null_safe(unresolved_col("right.b"))
         } else {
-            None
+            unresolved_col("b").eq(unresolved_col("right.b"))
         };
-        let pred = col("a").lt(lit(2));
+        let pred = resolved_col("a").is_null();
         let plan = left_scan_plan
-            .join_with_null_safe_equal(
+            .join(
                 &right_scan_plan,
-                join_on.clone(),
-                join_on,
-                null_equals_nulls,
+                join_on.into(),
+                vec![],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .filter(pred)?
             .build();
@@ -924,7 +909,7 @@ mod tests {
     /// Tests that Filter can be pushed into the right side of a Join.
     #[rstest]
     fn filter_does_not_commute_with_join_right_side(
-        #[values(false, true)] null_equal_null: bool,
+        #[values(false, true)] null_equals_null: bool,
         #[values(JoinType::Left, JoinType::Outer)] how: JoinType,
     ) -> DaftResult<()> {
         let left_scan_op = dummy_scan_operator(vec![
@@ -932,29 +917,26 @@ mod tests {
             Field::new("b", DataType::Utf8),
         ]);
         let right_scan_op = dummy_scan_operator(vec![
-            Field::new("b", DataType::Utf8),
+            Field::new("right.b", DataType::Utf8),
             Field::new("c", DataType::Float64),
         ]);
         let left_scan_plan = dummy_scan_node(left_scan_op.clone());
         let right_scan_plan = dummy_scan_node(right_scan_op.clone());
-        let join_on = vec![col("b")];
-        let null_equals_nulls = if null_equal_null {
-            Some(vec![true])
+        let join_on = if null_equals_null {
+            unresolved_col("b").eq_null_safe(unresolved_col("right.b"))
         } else {
-            None
+            unresolved_col("b").eq(unresolved_col("right.b"))
         };
-        let pred = col("c").lt(lit(2.0));
+
+        let pred = resolved_col("c").is_null();
         let plan = left_scan_plan
-            .join_with_null_safe_equal(
+            .join(
                 &right_scan_plan,
-                join_on.clone(),
-                join_on,
-                null_equals_nulls,
+                join_on.into(),
+                vec![],
                 how,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
             .filter(pred)?
             .build();
@@ -963,4 +945,70 @@ mod tests {
         assert_optimized_plan_eq(plan, expected)?;
         Ok(())
     }
+
+    /// Tests that a complex predicate can be separated so that it can be pushed down into one side of the join.
+    /// Modeled after TPC-H Q7
+    #[rstest]
+    fn filter_commutes_with_join_complex() -> DaftResult<()> {
+        let left_scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Utf8)]);
+        let right_scan_op = dummy_scan_operator(vec![Field::new("b", DataType::Utf8)]);
+
+        let plan = dummy_scan_node(left_scan_op.clone())
+            .join(
+                dummy_scan_node(right_scan_op.clone()),
+                None,
+                vec![],
+                JoinType::Inner,
+                None,
+                Default::default(),
+            )?
+            .filter(
+                (resolved_col("a")
+                    .eq(lit("FRANCE"))
+                    .and(resolved_col("b").eq(lit("GERMANY"))))
+                .or(resolved_col("a")
+                    .eq(lit("GERMANY"))
+                    .and(resolved_col("b").eq(lit("FRANCE")))),
+            )?
+            .build();
+
+        let expected = dummy_scan_node_with_pushdowns(
+            left_scan_op,
+            Pushdowns::default().with_filters(Some(
+                resolved_col("a")
+                    .eq(lit("FRANCE"))
+                    .or(resolved_col("a").eq(lit("GERMANY"))),
+            )),
+        )
+        .join(
+            dummy_scan_node_with_pushdowns(
+                right_scan_op,
+                Pushdowns::default().with_filters(Some(
+                    resolved_col("b")
+                        .eq(lit("GERMANY"))
+                        .or(resolved_col("b").eq(lit("FRANCE"))),
+                )),
+            ),
+            None,
+            vec![],
+            JoinType::Inner,
+            None,
+            Default::default(),
+        )?
+        .filter(
+            (resolved_col("a")
+                .eq(lit("FRANCE"))
+                .or(resolved_col("b").eq(lit("FRANCE"))))
+            .and(
+                resolved_col("b")
+                    .eq(lit("GERMANY"))
+                    .or(resolved_col("a").eq(lit("GERMANY"))),
+            ),
+        )?
+        .build();
+
+        assert_optimized_plan_eq(plan, expected)?;
+
+        Ok(())
+    }
 }
