@@ -4,9 +4,9 @@ use common_error::DaftResult;
 use common_treenode::{DynTreeNode, Transformed, TreeNode};
 use daft_core::prelude::*;
 use daft_dsl::{
-    col, is_actor_pool_udf,
+    is_actor_pool_udf,
     optimization::{get_required_columns, replace_columns_with_expressions, requires_computation},
-    Expr, ExprRef,
+    resolved_col, Column, Expr, ExprRef, ResolvedColumn,
 };
 use indexmap::IndexSet;
 use itertools::Itertools;
@@ -44,7 +44,9 @@ impl PushDownProjection {
                     .iter()
                     .zip(upstream_schema.names().iter())
                     .all(|(expr, upstream_col)| match expr.as_ref() {
-                        Expr::Column(colname) => colname.as_ref() == upstream_col,
+                        Expr::Column(Column::Resolved(ResolvedColumn::Basic(colname))) => {
+                            colname.as_ref() == upstream_col
+                        }
                         _ => false,
                     })
         };
@@ -88,12 +90,13 @@ impl PushDownProjection {
                         // If it's a reference for a column that requires computation,
                         // record it.
                         if okay_to_merge
-                            && let Expr::Column(name) = expr.as_ref()
+                            && let Expr::Column(Column::Resolved(ResolvedColumn::Basic(name))) =
+                                expr.as_ref()
                             && upstream_computations.contains(name.as_ref())
                         {
                             okay_to_merge = okay_to_merge
                                 && upstream_computations_used.insert(name.to_string());
-                        };
+                        }
                         if okay_to_merge {
                             expr.children()
                         } else {
@@ -145,12 +148,11 @@ impl PushDownProjection {
                     SourceInfo::Physical(external_info) => {
                         if required_columns.len() < upstream_schema.names().len() {
                             let pruned_upstream_schema = upstream_schema
-                                .fields
-                                .iter()
-                                .filter(|&(name, _)| required_columns.contains(name))
-                                .map(|(_, field)| field.clone())
+                                .into_iter()
+                                .filter(|field| required_columns.contains(&field.name))
+                                .cloned()
                                 .collect::<Vec<_>>();
-                            let schema = Schema::new(pruned_upstream_schema)?;
+                            let schema = Schema::new(pruned_upstream_schema);
                             let new_source: LogicalPlan = Source::new(
                                 schema.into(),
                                 Arc::new(SourceInfo::Physical(external_info.with_pushdowns(
@@ -316,7 +318,7 @@ impl PushDownProjection {
                     let all_projections_are_just_colexprs =
                         pruned_upstream_projections.iter().all(|proj| {
                             !proj.exists(|e| match e.as_ref() {
-                                Expr::Column(_) => false,
+                                Expr::Column(Column::Resolved(..)) => false,
                                 // Check for existence of any non-ColumnExprs
                                 _ => true,
                             })
@@ -344,10 +346,10 @@ impl PushDownProjection {
             LogicalPlan::Sort(..)
             | LogicalPlan::Repartition(..)
             | LogicalPlan::Limit(..)
+            | LogicalPlan::TopN(..)
             | LogicalPlan::Filter(..)
             | LogicalPlan::Sample(..)
-            | LogicalPlan::Explode(..)
-            | LogicalPlan::Unpivot(..) => {
+            | LogicalPlan::Explode(..) => {
                 // Get required columns from projection and upstream.
                 let combined_dependencies = plan
                     .required_columns()
@@ -360,6 +362,7 @@ impl PushDownProjection {
                 // Skip optimization if no columns would be pruned.
                 let grand_upstream_plan = &upstream_plan.arc_children()[0];
                 let grand_upstream_columns = grand_upstream_plan.schema().names();
+
                 if grand_upstream_columns.len() == combined_dependencies.len() {
                     return Ok(Transformed::no(plan));
                 }
@@ -367,7 +370,7 @@ impl PushDownProjection {
                 let new_subprojection: LogicalPlan = {
                     let pushdown_column_exprs = combined_dependencies
                         .into_iter()
-                        .map(col)
+                        .map(resolved_col)
                         .collect::<Vec<_>>();
 
                     Project::try_new(grand_upstream_plan.clone(), pushdown_column_exprs)?.into()
@@ -381,6 +384,43 @@ impl PushDownProjection {
                     .or(Transformed::yes(new_plan));
                 Ok(new_plan)
             }
+            LogicalPlan::Unpivot(unpivot) => {
+                let combined_dependencies = plan
+                    .required_columns()
+                    .iter()
+                    .flatten()
+                    .chain(upstream_plan.required_columns().iter().flatten())
+                    .cloned()
+                    .collect::<IndexSet<_>>();
+
+                let grand_upstream_plan = &upstream_plan.arc_children()[0];
+                let grand_upstream_columns = grand_upstream_plan.schema().names();
+                let input_columns = unpivot
+                    .ids
+                    .iter()
+                    .chain(unpivot.values.iter())
+                    .map(|e| e.name().to_string())
+                    .collect::<IndexSet<_>>();
+
+                let can_be_pushed_down = input_columns
+                    .intersection(&combined_dependencies)
+                    .map(|e| resolved_col(e.as_str()))
+                    .collect::<Vec<_>>();
+
+                if grand_upstream_columns.len() == can_be_pushed_down.len() {
+                    return Ok(Transformed::no(plan));
+                }
+
+                let new_subprojection: LogicalPlan =
+                    Project::try_new(grand_upstream_plan.clone(), can_be_pushed_down)?.into();
+                let new_upstream = upstream_plan.with_new_children(&[new_subprojection.into()]);
+                let new_plan = Arc::new(plan.with_new_children(&[new_upstream.into()]));
+                // Retry optimization now that the upstream node is different.
+                let new_plan = self
+                    .try_optimize_node(new_plan.clone())?
+                    .or(Transformed::yes(new_plan));
+                Ok(new_plan)
+            }
             LogicalPlan::Concat(concat) => {
                 // Get required columns from projection and upstream.
                 let combined_dependencies = plan
@@ -400,7 +440,7 @@ impl PushDownProjection {
 
                 let pushdown_column_exprs: Vec<ExprRef> = combined_dependencies
                     .into_iter()
-                    .map(col)
+                    .map(resolved_col)
                     .collect::<Vec<_>>();
                 let new_left_subprojection: LogicalPlan = {
                     Project::try_new(concat.input.clone(), pushdown_column_exprs.clone())?.into()
@@ -437,7 +477,8 @@ impl PushDownProjection {
                     projection_dependencies: &IndexSet<String>,
                 ) -> DaftResult<Transformed<LogicalPlanRef>> {
                     let schema = side.schema();
-                    let upstream_names: IndexSet<String> = schema.fields.keys().cloned().collect();
+                    let upstream_names: IndexSet<String> =
+                        schema.field_names().map(ToString::to_string).collect();
 
                     let combined_dependencies: IndexSet<_> = side_dependencies
                         .union(
@@ -450,8 +491,10 @@ impl PushDownProjection {
                         .collect();
 
                     if combined_dependencies.len() < upstream_names.len() {
-                        let pushdown_column_exprs: Vec<ExprRef> =
-                            combined_dependencies.into_iter().map(col).collect();
+                        let pushdown_column_exprs: Vec<ExprRef> = combined_dependencies
+                            .into_iter()
+                            .map(resolved_col)
+                            .collect();
                         let new_project: LogicalPlan =
                             Project::try_new(side.clone(), pushdown_column_exprs)?.into();
                         Ok(Transformed::yes(new_project.into()))
@@ -502,9 +545,14 @@ impl PushDownProjection {
                 // Cannot push down past a Pivot/MonotonicallyIncreasingId because it changes the schema.
                 Ok(Transformed::no(plan))
             }
+            LogicalPlan::Window(_) => {
+                // Cannot push down past a Window because it changes the window calculation results
+                Ok(Transformed::no(plan))
+            }
             LogicalPlan::Sink(_) => {
                 panic!("Bad projection due to upstream sink node: {:?}", projection)
             }
+            LogicalPlan::SubqueryAlias(_) => unreachable!("Alias should have been optimized away"),
         }
     }
 
@@ -523,7 +571,7 @@ impl PushDownProjection {
             let new_subprojection: LogicalPlan = {
                 let pushdown_column_exprs = actor_pool_project_required_cols
                     .iter()
-                    .map(|s| col(s.as_str()))
+                    .map(|s| resolved_col(s.as_str()))
                     .collect::<Vec<_>>();
 
                 Project::try_new(upstream_plan.clone(), pushdown_column_exprs)?.into()
@@ -551,7 +599,7 @@ impl PushDownProjection {
             let new_subprojection: LogicalPlan = {
                 let pushdown_column_exprs = aggregation_required_cols
                     .iter()
-                    .map(|s| col(s.as_str()))
+                    .map(|s| resolved_col(s.as_str()))
                     .collect::<Vec<_>>();
 
                 Project::try_new(upstream_plan.clone(), pushdown_column_exprs)?.into()
@@ -580,11 +628,11 @@ impl PushDownProjection {
                 .expect("we expect 2 set of required columns for join");
             let right_schema = join.right.schema();
 
-            if right_required_cols.len() < right_schema.fields.len() {
+            if right_required_cols.len() < right_schema.len() {
                 let new_subprojection: LogicalPlan = {
                     let pushdown_column_exprs = right_required_cols
                         .iter()
-                        .map(|s| col(s.as_str()))
+                        .map(|s| resolved_col(s.as_str()))
                         .collect::<Vec<_>>();
 
                     Project::try_new(join.right.clone(), pushdown_column_exprs)?.into()
@@ -620,7 +668,7 @@ impl PushDownProjection {
             let new_subprojection: LogicalPlan = {
                 let pushdown_column_exprs = pivot_required_cols
                     .iter()
-                    .map(|s| col(s.as_str()))
+                    .map(|s| resolved_col(s.as_str()))
                     .collect::<Vec<_>>();
 
                 Project::try_new(upstream_plan.clone(), pushdown_column_exprs)?.into()
@@ -671,15 +719,15 @@ mod tests {
     use common_scan_info::Pushdowns;
     use daft_core::prelude::*;
     use daft_dsl::{
-        col,
         functions::{
             python::{MaybeInitializedUDF, PythonUDF, RuntimePyObject},
             FunctionExpr,
         },
-        lit, Expr, ExprRef,
+        lit, resolved_col, unresolved_col, Expr, ExprRef,
     };
 
     use crate::{
+        ops::{Project, Unpivot},
         optimization::{
             optimizer::{RuleBatch, RuleExecutionStrategy},
             rules::PushDownProjection,
@@ -729,7 +777,7 @@ mod tests {
     /// Projection merging: Ensure factored projections do not get merged.
     #[test]
     fn test_merge_does_not_unfactor() -> DaftResult<()> {
-        let a2 = col("a").add(col("a"));
+        let a2 = unresolved_col("a").add(unresolved_col("a"));
         let a4 = a2.clone().add(a2);
         let a8 = a4.clone().add(a4);
         let expressions = vec![a8.alias("x")];
@@ -749,20 +797,24 @@ mod tests {
             Field::new("b", DataType::Int64),
         ]);
         let proj1 = vec![
-            col("a").add(lit(1)),
-            col("b").add(lit(2)),
-            col("a").alias("c"),
+            unresolved_col("a").add(lit(1)),
+            unresolved_col("b").add(lit(2)),
+            unresolved_col("a").alias("c"),
+        ];
+        let proj2 = vec![
+            unresolved_col("a").add(lit(3)),
+            unresolved_col("b"),
+            unresolved_col("c").add(lit(4)),
         ];
-        let proj2 = vec![col("a").add(lit(3)), col("b"), col("c").add(lit(4))];
         let plan = dummy_scan_node(scan_op.clone())
             .select(proj1)?
             .select(proj2)?
             .build();
 
         let merged_proj = vec![
-            col("a").add(lit(1)).add(lit(3)),
-            col("b").add(lit(2)),
-            col("a").alias("c").add(lit(4)),
+            unresolved_col("a").add(lit(1)).add(lit(3)),
+            unresolved_col("b").add(lit(2)),
+            unresolved_col("a").alias("c").add(lit(4)),
         ];
         let expected = dummy_scan_node(scan_op).select(merged_proj)?.build();
 
@@ -778,7 +830,7 @@ mod tests {
             Field::new("b", DataType::Int64),
         ]);
         let plan = dummy_scan_node(scan_op.clone())
-            .select(vec![col("a"), col("b")])?
+            .select(vec![unresolved_col("a"), unresolved_col("b")])?
             .build();
 
         let expected = dummy_scan_node(scan_op).build();
@@ -795,7 +847,7 @@ mod tests {
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Int64),
         ]);
-        let proj = vec![col("b"), col("a")];
+        let proj = vec![unresolved_col("b"), unresolved_col("a")];
         let plan = dummy_scan_node(scan_op.clone())
             .select(proj.clone())?
             .build();
@@ -814,7 +866,7 @@ mod tests {
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Int64),
         ]);
-        let proj = vec![col("b").add(lit(3))];
+        let proj = vec![unresolved_col("b").add(lit(3))];
         let plan = dummy_scan_node(scan_op.clone())
             .select(proj.clone())?
             .build();
@@ -839,14 +891,22 @@ mod tests {
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Int64),
         ]);
-        let proj1 = vec![col("b").add(lit(3)), col("a"), col("a").alias("x")];
-        let proj2 = vec![col("a"), col("b"), col("b").alias("c")];
+        let proj1 = vec![
+            unresolved_col("b").add(lit(3)),
+            unresolved_col("a"),
+            unresolved_col("a").alias("x"),
+        ];
+        let proj2 = vec![
+            unresolved_col("a"),
+            unresolved_col("b"),
+            unresolved_col("b").alias("c"),
+        ];
         let plan = dummy_scan_node(scan_op.clone())
             .select(proj1)?
             .select(proj2.clone())?
             .build();
 
-        let new_proj1 = vec![col("b").add(lit(3)), col("a")];
+        let new_proj1 = vec![unresolved_col("b").add(lit(3)), unresolved_col("a")];
         let expected = dummy_scan_node(scan_op)
             .select(new_proj1)?
             .select(proj2)?
@@ -865,16 +925,16 @@ mod tests {
             Field::new("b", DataType::Int64),
             Field::new("c", DataType::Int64),
         ]);
-        let agg = vec![col("a").mean(), col("b").mean()];
-        let group_by = vec![col("c")];
-        let proj = vec![col("a")];
+        let agg = vec![unresolved_col("a").mean(), unresolved_col("b").mean()];
+        let group_by = vec![unresolved_col("c")];
+        let proj = vec![unresolved_col("a")];
         let plan = dummy_scan_node(scan_op.clone())
             .aggregate(agg, group_by.clone())?
             .select(proj.clone())?
             .build();
 
         let proj_pushdown = vec!["a".to_string(), "c".to_string()];
-        let new_agg = vec![col("a").mean()];
+        let new_agg = vec![unresolved_col("a").mean()];
         let expected = dummy_scan_node_with_pushdowns(
             scan_op,
             Pushdowns::default().with_columns(Some(Arc::new(proj_pushdown))),
@@ -896,8 +956,8 @@ mod tests {
             Field::new("b", DataType::Boolean),
             Field::new("c", DataType::Int64),
         ]);
-        let pred = col("b");
-        let proj = vec![col("a")];
+        let pred = unresolved_col("b");
+        let proj = vec![unresolved_col("a")];
         let plan = dummy_scan_node(scan_op.clone())
             .filter(pred.clone())?
             .select(proj.clone())?
@@ -926,7 +986,7 @@ mod tests {
         ]);
         let plan = dummy_scan_node(scan_op.clone())
             .add_monotonically_increasing_id(Some("id"))?
-            .select(vec![col("id")])?
+            .select(vec![unresolved_col("id")])?
             .build();
         let expected = plan.clone();
         assert_optimized_plan_eq(plan, expected)?;
@@ -945,17 +1005,21 @@ mod tests {
             Field::new("c", DataType::Int64),
         ]);
         let scan_node = dummy_scan_node(scan_op.clone());
-        let mock_udf = create_actor_pool_udf(vec![col("c")]);
+        let mock_udf = create_actor_pool_udf(vec![resolved_col("c")]);
 
         // Select the `udf_results` column, so the ActorPoolProject should apply column pruning to the other columns
         let actor_pool_project = LogicalPlan::ActorPoolProject(ActorPoolProject::try_new(
             scan_node.build(),
-            vec![col("a"), col("b"), mock_udf.alias("udf_results")],
+            vec![
+                resolved_col("a"),
+                resolved_col("b"),
+                mock_udf.alias("udf_results"),
+            ],
         )?)
         .arced();
         let project = LogicalPlan::Project(Project::try_new(
             actor_pool_project,
-            vec![col("udf_results")],
+            vec![resolved_col("udf_results")],
         )?)
         .arced();
 
@@ -984,21 +1048,25 @@ mod tests {
             Field::new("c", DataType::Int64),
         ]);
         let scan_node = dummy_scan_node(scan_op.clone()).build();
-        let mock_udf = create_actor_pool_udf(vec![col("a")]);
+        let mock_udf = create_actor_pool_udf(vec![resolved_col("a")]);
 
         // Select the `udf_results` column, so the ActorPoolProject should apply column pruning to the other columns
         let plan = LogicalPlan::ActorPoolProject(ActorPoolProject::try_new(
             scan_node,
-            vec![col("a"), col("b"), mock_udf.alias("udf_results_0")],
+            vec![
+                resolved_col("a"),
+                resolved_col("b"),
+                mock_udf.alias("udf_results_0"),
+            ],
         )?)
         .arced();
 
         let plan = LogicalPlan::ActorPoolProject(ActorPoolProject::try_new(
             plan,
             vec![
-                col("a"),
-                col("b"),
-                col("udf_results_0"),
+                resolved_col("a"),
+                resolved_col("b"),
+                resolved_col("udf_results_0"),
                 mock_udf.alias("udf_results_1"),
             ],
         )?)
@@ -1007,8 +1075,8 @@ mod tests {
         let plan = LogicalPlan::Project(Project::try_new(
             plan,
             vec![
-                col("udf_results_0").alias("udf_results_0_alias"),
-                col("udf_results_1"),
+                resolved_col("udf_results_0").alias("udf_results_0_alias"),
+                resolved_col("udf_results_1"),
             ],
         )?)
         .arced();
@@ -1020,14 +1088,14 @@ mod tests {
             )
             .build(),
             // col("b") is pruned
-            vec![mock_udf.alias("udf_results_0"), col("a")],
+            vec![mock_udf.alias("udf_results_0"), resolved_col("a")],
         )?)
         .arced();
         let expected = LogicalPlan::ActorPoolProject(ActorPoolProject::try_new(
             expected,
             vec![
                 // Absorbed a non-computational expression (alias) from the Projection
-                col("udf_results_0").alias("udf_results_0_alias"),
+                resolved_col("udf_results_0").alias("udf_results_0_alias"),
                 mock_udf.alias("udf_results_1"),
             ],
         )?)
@@ -1048,16 +1116,23 @@ mod tests {
             Field::new("c", DataType::Int64),
         ]);
         let scan_node = dummy_scan_node(scan_op.clone()).build();
-        let mock_udf = create_actor_pool_udf(vec![col("c")]);
+        let mock_udf = create_actor_pool_udf(vec![resolved_col("c")]);
 
         // Select only col("a"), so the ActorPoolProject node is now redundant and should be removed
         let actor_pool_project = LogicalPlan::ActorPoolProject(ActorPoolProject::try_new(
             scan_node,
-            vec![col("a"), col("b"), mock_udf.alias("udf_results")],
+            vec![
+                resolved_col("a"),
+                resolved_col("b"),
+                mock_udf.alias("udf_results"),
+            ],
+        )?)
+        .arced();
+        let project = LogicalPlan::Project(Project::try_new(
+            actor_pool_project,
+            vec![resolved_col("a")],
         )?)
         .arced();
-        let project =
-            LogicalPlan::Project(Project::try_new(actor_pool_project, vec![col("a")])?).arced();
 
         // Optimized plan will push the projection all the way down into the scan
         let expected_scan = dummy_scan_node_with_pushdowns(
@@ -1074,4 +1149,66 @@ mod tests {
         assert_optimized_plan_eq(project, expected_scan)?;
         Ok(())
     }
+
+    #[test]
+    fn test_projection_pushdown_with_unpivot() {
+        let scan_op = dummy_scan_operator(vec![
+            Field::new("year", DataType::Int64),
+            Field::new("id", DataType::Int64),
+            Field::new("Jan", DataType::Int64),
+            Field::new("Feb", DataType::Int64),
+        ]);
+        let scan_node = dummy_scan_node(scan_op.clone()).build();
+
+        let plan = LogicalPlan::Unpivot(
+            Unpivot::try_new(
+                scan_node.clone(),
+                vec![resolved_col("year")],
+                vec![resolved_col("Jan"), resolved_col("Feb")],
+                "month".to_string(),
+                "inventory".to_string(),
+            )
+            .unwrap(),
+        );
+
+        let plan = LogicalPlan::Project(
+            Project::try_new(plan.into(), vec![resolved_col("inventory").alias("year2")]).unwrap(),
+        )
+        .into();
+        let expected_scan = dummy_scan_node_with_pushdowns(
+            scan_op.clone(),
+            Pushdowns {
+                limit: None,
+                partition_filters: None,
+                columns: Some(Arc::new(vec![
+                    "year".to_string(),
+                    "Jan".to_string(),
+                    "Feb".to_string(),
+                ])),
+                filters: None,
+            },
+        )
+        .build();
+
+        let expected = LogicalPlan::Unpivot(
+            Unpivot::try_new(
+                expected_scan,
+                vec![resolved_col("year")],
+                vec![resolved_col("Jan"), resolved_col("Feb")],
+                "month".to_string(),
+                "inventory".to_string(),
+            )
+            .unwrap(),
+        );
+
+        let expected = LogicalPlan::Project(
+            Project::try_new(
+                expected.into(),
+                vec![resolved_col("inventory").alias("year2")],
+            )
+            .unwrap(),
+        )
+        .into();
+        assert_optimized_plan_eq(plan, expected).unwrap();
+    }
 }
