@@ -4,14 +4,14 @@ use common_error::{DaftError, DaftResult};
 use common_treenode::{DynTreeNode, Transformed, TreeNode};
 use daft_algebra::boolean::{combine_conjunction, split_conjunction};
 use daft_core::{join::JoinType, prelude::SchemaRef};
-use daft_dsl::{col, Expr, ExprRef, Operator, Subquery};
+use daft_dsl::{resolved_col, Column, Expr, ExprRef, Operator, ResolvedColumn, Subquery};
 use itertools::multiunzip;
 use uuid::Uuid;
 
 use super::OptimizerRule;
 use crate::{
     logical_plan::downcast_subquery,
-    ops::{Aggregate, Filter, Join, Project},
+    ops::{join::JoinPredicate, Aggregate, Filter, Join, Project},
     LogicalPlan, LogicalPlanRef,
 };
 
@@ -82,7 +82,7 @@ impl UnnestScalarSubquery {
                         if let Expr::Subquery(subquery) = e.as_ref() {
                             subqueries.insert(subquery.clone());
 
-                            Ok(Transformed::yes(col(subquery.semantic_id().id)))
+                            Ok(Transformed::yes(resolved_col(subquery.semantic_id().id)))
                         } else {
                             Ok(Transformed::no(e))
                         }
@@ -113,41 +113,51 @@ impl UnnestScalarSubquery {
                 // alias output column
                 let subquery_plan = Arc::new(LogicalPlan::Project(Project::try_new(
                     subquery_plan,
-                    vec![col(output_col.as_str()).alias(subquery_alias)],
+                    vec![resolved_col(output_col.as_str()).alias(subquery_alias)],
                 )?));
 
                 let (decorrelated_subquery, subquery_on, input_on) =
                     pull_up_correlated_cols(subquery_plan)?;
 
-                if subquery_on.is_empty() {
-                    // uncorrelated scalar subquery
-                    Ok(Arc::new(LogicalPlan::Join(Join::try_new(
-                        curr_input,
-                        decorrelated_subquery,
-                        vec![],
-                        vec![],
-                        None,
-                        JoinType::Inner,
-                        None,
-                        None,
-                        None,
-                        false,
-                    )?)))
+                let on_expr =
+                    combine_conjunction(input_on.into_iter().zip(subquery_on.into_iter()).map(
+                        |(i, s)| {
+                            let i_left = i
+                                .to_left_cols(curr_input.schema())
+                                .expect("input columns to be in curr_input");
+                            let s_right = s
+                                .to_right_cols(decorrelated_subquery.schema())
+                                .expect("subquery columns to be in decorrelated_subquery");
+
+                            i_left.eq(s_right)
+                        },
+                    ));
+
+                // use inner join when uncorrelated so that filter can be pushed into join and other optimizations
+                let join_type = if on_expr.is_none() {
+                    JoinType::Inner
                 } else {
-                    // correlated scalar subquery
-                    Ok(Arc::new(LogicalPlan::Join(Join::try_new(
-                        curr_input,
-                        decorrelated_subquery,
-                        input_on,
-                        subquery_on,
-                        None,
-                        JoinType::Left,
-                        None,
-                        None,
-                        None,
-                        false,
-                    )?)))
-                }
+                    JoinType::Left
+                };
+
+                let (curr_input, decorrelated_subquery, on_expr) = Join::deduplicate_join_columns(
+                    curr_input,
+                    decorrelated_subquery,
+                    on_expr,
+                    &[],
+                    join_type,
+                    Default::default(),
+                )?;
+
+                let on = JoinPredicate::try_new(on_expr)?;
+
+                Ok(Arc::new(LogicalPlan::Join(Join::try_new(
+                    curr_input,
+                    decorrelated_subquery,
+                    on,
+                    join_type,
+                    None,
+                )?)))
             })?;
 
         Ok(Transformed::yes((new_input, new_exprs)))
@@ -320,24 +330,34 @@ impl OptimizerRule for UnnestPredicateSubquery {
                         };
 
                         input_on.push(in_expr);
-                        subquery_on.push(col(output_col.as_str()));
+                        subquery_on.push(resolved_col(output_col.as_str()));
                     }
 
                     if subquery_on.is_empty() {
                         return Err(DaftError::ValueError("Expected IN/EXISTS subquery to be correlated, found uncorrelated subquery.".to_string()));
                     }
 
+                    let on_expr = combine_conjunction(input_on.into_iter().zip(subquery_on.into_iter()).map(
+                        |(i, s)| {
+                            let i_left = i
+                                .to_left_cols(curr_input.schema())
+                                .expect("input columns to be in curr_input");
+                            let s_right = s
+                                .to_right_cols(decorrelated_subquery.schema())
+                                .expect("subquery columns to be in decorrelated_subquery");
+
+                            i_left.eq(s_right)
+                        },
+                    ));
+
+                    let on = JoinPredicate::try_new(on_expr)?;
+
                     Ok(Arc::new(LogicalPlan::Join(Join::try_new(
                         curr_input,
                         decorrelated_subquery,
-                        input_on,
-                        subquery_on,
-                        None,
+                        on,
                         join_type,
                         None,
-                        None,
-                        None,
-                        false
                     )?)))
                 })?;
 
@@ -394,16 +414,26 @@ fn pull_up_correlated_cols(
                     {
                         match (left.as_ref(), right.as_ref()) {
                             (
-                                Expr::Column(subquery_col_name),
-                                Expr::OuterReferenceColumn(outer_col),
+                                Expr::Column(Column::Resolved(ResolvedColumn::Basic(
+                                    subquery_col_name,
+                                ))),
+                                Expr::Column(Column::Resolved(ResolvedColumn::OuterRef(
+                                    outer_field,
+                                    _,
+                                ))),
                             )
                             | (
-                                Expr::OuterReferenceColumn(outer_col),
-                                Expr::Column(subquery_col_name),
+                                Expr::Column(Column::Resolved(ResolvedColumn::OuterRef(
+                                    outer_field,
+                                    _,
+                                ))),
+                                Expr::Column(Column::Resolved(ResolvedColumn::Basic(
+                                    subquery_col_name,
+                                ))),
                             ) => {
                                 // remove correlated col from filter, use in join instead
-                                subquery_on.push(col(subquery_col_name.clone()));
-                                outer_on.push(col(outer_col.field.name.as_str()));
+                                subquery_on.push(resolved_col(subquery_col_name.clone()));
+                                outer_on.push(resolved_col(outer_field.name.as_str()));
 
                                 found_correlated_col = true;
                                 return false;
@@ -492,11 +522,13 @@ fn pull_up_correlated_cols(
         | LogicalPlan::Repartition(..)
         | LogicalPlan::Union(..)
         | LogicalPlan::Intersect(..)
-        | LogicalPlan::Sort(..) => Ok((plan.clone(), subquery_on, outer_on)),
+        | LogicalPlan::Sort(..)
+        | LogicalPlan::SubqueryAlias(..) => Ok((plan.clone(), subquery_on, outer_on)),
 
         // ops that cannot pull up correlated columns
         LogicalPlan::ActorPoolProject(..)
         | LogicalPlan::Limit(..)
+        | LogicalPlan::TopN(..)
         | LogicalPlan::Sample(..)
         | LogicalPlan::Source(..)
         | LogicalPlan::Explode(..)
@@ -504,7 +536,8 @@ fn pull_up_correlated_cols(
         | LogicalPlan::Pivot(..)
         | LogicalPlan::Concat(..)
         | LogicalPlan::Join(..)
-        | LogicalPlan::Sink(..) => {
+        | LogicalPlan::Sink(..)
+        | LogicalPlan::Window(..) => {
             if subquery_on.is_empty() {
                 Ok((plan.clone(), vec![], vec![]))
             } else {
@@ -533,7 +566,7 @@ fn get_missing_exprs(
             // another expression takes pull up column name, we rename the pull up column.
             let new_name = format!("{}-{}", expr.name(), Uuid::new_v4());
 
-            new_subquery_on.push(col(new_name.clone()));
+            new_subquery_on.push(resolved_col(new_name.clone()));
             missing_exprs.push(expr.alias(new_name));
         } else {
             // missing from schema, can keep original name
@@ -552,7 +585,7 @@ mod tests {
 
     use common_error::DaftResult;
     use daft_core::join::JoinType;
-    use daft_dsl::{col, Expr, OuterReferenceColumn, Subquery};
+    use daft_dsl::{unresolved_col, Column, Expr, PlanRef, ResolvedColumn, Subquery};
     use daft_schema::{dtype::DataType, field::Field};
 
     use super::{UnnestPredicateSubquery, UnnestScalarSubquery};
@@ -605,31 +638,29 @@ mod tests {
             DataType::Int64,
         )]));
 
-        let subquery = tbl2.aggregate(vec![col("key").max()], vec![])?;
+        let subquery = tbl2.aggregate(vec![unresolved_col("key").max()], vec![])?;
         let subquery_expr = Arc::new(Expr::Subquery(Subquery {
             plan: subquery.build(),
         }));
         let subquery_alias = subquery_expr.semantic_id(&subquery.schema()).id;
 
         let plan = tbl1
-            .filter(col("key").eq(subquery_expr))?
-            .select(vec![col("val")])?
+            .filter(unresolved_col("key").eq(subquery_expr))?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         let expected = tbl1
             .join(
-                subquery.select(vec![col("key").alias(subquery_alias.clone())])?,
-                vec![],
+                subquery.select(vec![unresolved_col("key").alias(subquery_alias.clone())])?,
+                None,
                 vec![],
                 JoinType::Inner,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
-            .filter(col("key").eq(col(subquery_alias)))?
-            .select(vec![col("key"), col("val")])?
-            .select(vec![col("val")])?
+            .filter(unresolved_col("key").eq(unresolved_col(subquery_alias)))?
+            .select(vec![unresolved_col("key"), unresolved_col("val")])?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         assert_scalar_optimized_plan_eq(plan, expected)?;
@@ -646,45 +677,52 @@ mod tests {
 
         let tbl2 = dummy_scan_node(dummy_scan_operator(vec![
             Field::new("outer_key", DataType::Int64),
-            Field::new("inner_key", DataType::Int64),
+            Field::new("inner_key2", DataType::Int64),
         ]));
 
-        let subquery = tbl2
-            .filter(col("inner_key").eq(Arc::new(Expr::OuterReferenceColumn(
-                OuterReferenceColumn {
-                    field: Field::new("inner_key", DataType::Int64),
-                    depth: 1,
-                },
+        let subquery =
+            tbl2.filter(unresolved_col("inner_key2").eq(Arc::new(Expr::Column(
+                Column::Resolved(ResolvedColumn::OuterRef(
+                    Field::new("inner_key", DataType::Int64),
+                    PlanRef::Unqualified,
+                )),
             ))))?
-            .aggregate(vec![col("outer_key").max()], vec![])?;
+            .aggregate(vec![unresolved_col("outer_key").max()], vec![])?;
         let subquery_expr = Arc::new(Expr::Subquery(Subquery {
             plan: subquery.build(),
         }));
         let subquery_alias = subquery_expr.semantic_id(&subquery.schema()).id;
 
         let plan = tbl1
-            .filter(col("outer_key").eq(subquery_expr))?
-            .select(vec![col("val")])?
+            .filter(unresolved_col("outer_key").eq(subquery_expr))?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         let expected = tbl1
             .join(
-                tbl2.aggregate(vec![col("outer_key").max()], vec![col("inner_key")])?
-                    .select(vec![
-                        col("outer_key").alias(subquery_alias.clone()),
-                        col("inner_key"),
-                    ])?,
-                vec![col("inner_key")],
-                vec![col("inner_key")],
+                tbl2.aggregate(
+                    vec![unresolved_col("outer_key").max()],
+                    vec![unresolved_col("inner_key2")],
+                )?
+                .select(vec![
+                    unresolved_col("outer_key").alias(subquery_alias.clone()),
+                    unresolved_col("inner_key2"),
+                ])?,
+                unresolved_col("inner_key")
+                    .eq(unresolved_col("inner_key2"))
+                    .into(),
+                vec![],
                 JoinType::Left,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
-            .filter(col("outer_key").eq(col(subquery_alias)))?
-            .select(vec![col("outer_key"), col("inner_key"), col("val")])?
-            .select(vec![col("val")])?
+            .filter(unresolved_col("outer_key").eq(unresolved_col(subquery_alias)))?
+            .select(vec![
+                unresolved_col("outer_key"),
+                unresolved_col("inner_key"),
+                unresolved_col("val"),
+            ])?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         assert_scalar_optimized_plan_eq(plan, expected)?;
@@ -699,30 +737,28 @@ mod tests {
         ]));
 
         let tbl2 = dummy_scan_node(dummy_scan_operator(vec![Field::new(
-            "key",
+            "key2",
             DataType::Int64,
         )]));
 
         let plan = tbl1
             .filter(Arc::new(Expr::InSubquery(
-                col("key"),
+                unresolved_col("key"),
                 Subquery { plan: tbl2.build() },
             )))?
-            .select(vec![col("val")])?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         let expected = tbl1
             .join(
                 tbl2,
-                vec![col("key")],
-                vec![col("key")],
+                unresolved_col("key").eq(unresolved_col("key2")).into(),
+                vec![],
                 JoinType::Semi,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
-            .select(vec![col("val")])?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         assert_predicate_optimized_plan_eq(plan, expected)?;
@@ -737,36 +773,36 @@ mod tests {
         ]));
 
         let tbl2 = dummy_scan_node(dummy_scan_operator(vec![Field::new(
-            "key",
+            "key2",
             DataType::Int64,
         )]));
 
         let subquery = tbl2
             .filter(
-                col("key").eq(Arc::new(Expr::OuterReferenceColumn(OuterReferenceColumn {
-                    field: Field::new("key", DataType::Int64),
-                    depth: 1,
-                }))),
+                unresolved_col("key2").eq(Arc::new(Expr::Column(Column::Resolved(
+                    ResolvedColumn::OuterRef(
+                        Field::new("key", DataType::Int64),
+                        PlanRef::Unqualified,
+                    ),
+                )))),
             )?
             .build();
 
         let plan = tbl1
             .filter(Arc::new(Expr::Exists(Subquery { plan: subquery })).not())?
-            .select(vec![col("val")])?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         let expected = tbl1
             .join(
                 tbl2,
-                vec![col("key")],
-                vec![col("key")],
+                unresolved_col("key").eq(unresolved_col("key2")).into(),
+                vec![],
                 JoinType::Anti,
                 None,
-                None,
-                None,
-                false,
+                Default::default(),
             )?
-            .select(vec![col("val")])?
+            .select(vec![unresolved_col("val")])?
             .build();
 
         assert_predicate_optimized_plan_eq(plan, expected)?;
