@@ -41,20 +41,25 @@ impl MicroPartition {
             _ => {}
         }
 
+        let left_on = left_on
+            .iter()
+            .map(|expr| BoundExpr::try_new(expr.clone(), &self.schema))
+            .try_collect::<Vec<_>>()?;
+        let right_on = right_on
+            .iter()
+            .map(|expr| BoundExpr::try_new(expr.clone(), &right.schema))
+            .try_collect::<Vec<_>>()?;
+
         // TODO(Kevin): short circuits are also possible for other join types
         if how == JoinType::Inner {
             let tv = match (&self.statistics, &right.statistics) {
                 (_, None) => TruthValue::Maybe,
                 (None, _) => TruthValue::Maybe,
                 (Some(l), Some(r)) => {
-                    let l_eval_stats = l.eval_expression_list(left_on, &self.schema)?;
-                    let r_eval_stats = r.eval_expression_list(right_on, &right.schema)?;
+                    let l_eval_stats = l.eval_expression_list(&left_on)?;
+                    let r_eval_stats = r.eval_expression_list(&right_on)?;
                     let mut curr_tv = TruthValue::Maybe;
-                    for (lc, rc) in l_eval_stats
-                        .columns
-                        .values()
-                        .zip(r_eval_stats.columns.values())
-                    {
+                    for (lc, rc) in l_eval_stats.into_iter().zip(&r_eval_stats) {
                         if lc.equal(rc)?.to_truth_value() == TruthValue::False {
                             curr_tv = TruthValue::False;
                             break;
@@ -75,15 +80,6 @@ impl MicroPartition {
         match (lt.as_slice(), rt.as_slice()) {
             ([], _) | (_, []) => Ok(Self::empty(Some(join_schema))),
             ([lt], [rt]) => {
-                let left_on = left_on
-                    .iter()
-                    .map(|expr| BoundExpr::try_new(expr.clone(), &self.schema))
-                    .try_collect::<Vec<_>>()?;
-                let right_on = right_on
-                    .iter()
-                    .map(|expr| BoundExpr::try_new(expr.clone(), &right.schema))
-                    .try_collect::<Vec<_>>()?;
-
                 let joined_table = table_join(lt, rt, &left_on, &right_on, how)?;
                 Ok(Self::new_loaded(
                     join_schema,
