@@ -1,7 +1,7 @@
 use common_error::{DaftError, DaftResult};
-use daft_dsl::ExprRef;
+use daft_dsl::{expr::bound_expr::BoundExpr, ExprRef};
 use daft_io::IOStatsContext;
-use daft_table::Table;
+use daft_recordbatch::RecordBatch;
 
 use crate::micropartition::MicroPartition;
 
@@ -17,14 +17,22 @@ impl MicroPartition {
 
         let tables = self.concat_or_get(io_stats)?;
 
+        let group_by = group_by
+            .iter()
+            .map(|expr| BoundExpr::try_new(expr.clone(), &self.schema))
+            .try_collect::<Vec<_>>()?;
+
+        let pivot_col = BoundExpr::try_new(pivot_col, &self.schema)?;
+        let values_col = BoundExpr::try_new(values_col, &self.schema)?;
+
         match tables.as_slice() {
             [] => {
-                let empty_table = Table::empty(Some(self.schema.clone()))?;
-                let pivoted = empty_table.pivot(group_by, pivot_col, values_col, names)?;
+                let empty_table = RecordBatch::empty(Some(self.schema.clone()))?;
+                let pivoted = empty_table.pivot(&group_by, pivot_col, values_col, names)?;
                 Ok(Self::empty(Some(pivoted.schema)))
             }
             [t] => {
-                let pivoted = t.pivot(group_by, pivot_col, values_col, names)?;
+                let pivoted = t.pivot(&group_by, pivot_col, values_col, names)?;
                 Ok(Self::new_loaded(
                     pivoted.schema.clone(),
                     vec![pivoted].into(),
