@@ -8,7 +8,7 @@ use arrow2::io::parquet::read::{column_iter_to_arrays, schema::infer_schema_with
 use common_error::DaftResult;
 use common_runtime::get_io_runtime;
 use daft_core::{prelude::*, utils::arrow::cast_array_for_daft_if_needed};
-use daft_dsl::ExprRef;
+use daft_dsl::{expr::bound_expr::BoundExpr, ExprRef};
 use daft_io::{IOClient, IOStatsRef};
 use daft_recordbatch::RecordBatch;
 use daft_stats::TruthValue;
@@ -29,9 +29,9 @@ use crate::{
     statistics,
     stream_reader::spawn_column_iters_to_table_task,
     utils::combine_stream,
-    JoinSnafu, OneShotRecvSnafu, UnableToConvertRowGroupMetadataToStatsSnafu,
-    UnableToCreateParquetPageStreamSnafu, UnableToParseSchemaFromMetadataSnafu,
-    UnableToRunExpressionOnStatsSnafu, PARQUET_MORSEL_SIZE,
+    JoinSnafu, OneShotRecvSnafu, UnableToBindExpressionSnafu,
+    UnableToConvertRowGroupMetadataToStatsSnafu, UnableToCreateParquetPageStreamSnafu,
+    UnableToParseSchemaFromMetadataSnafu, UnableToRunExpressionOnStatsSnafu, PARQUET_MORSEL_SIZE,
 };
 
 pub struct ParquetReaderBuilder {
@@ -120,6 +120,13 @@ pub fn build_row_ranges(
     let mut row_ranges = vec![];
     let mut curr_row_index = 0;
 
+    let predicate = predicate
+        .map(|p| BoundExpr::try_new(p, schema))
+        .transpose()
+        .with_context(|_| UnableToBindExpressionSnafu {
+            path: uri.to_string(),
+        })?;
+
     if let Some(row_groups) = row_groups {
         let mut rows_to_add: i64 = limit.unwrap_or(i64::MAX);
         for i in row_groups {
