@@ -15,9 +15,9 @@ use common_runtime::get_io_runtime;
 use daft_core::prelude::*;
 #[cfg(feature = "python")]
 use daft_core::python::PyTimeUnit;
-use daft_dsl::{optimization::get_required_columns, ExprRef};
+use daft_dsl::{expr::bound_expr::BoundExpr, optimization::get_required_columns, ExprRef};
 use daft_io::{parse_url, IOClient, IOStatsRef, SourceType};
-use daft_table::Table;
+use daft_recordbatch::RecordBatch;
 use futures::{
     future::{join_all, try_join_all},
     stream::BoxStream,
@@ -158,7 +158,7 @@ async fn read_parquet_single(
     metadata: Option<Arc<FileMetaData>>,
     delete_rows: Option<Vec<i64>>,
     chunk_size: Option<usize>,
-) -> DaftResult<Table> {
+) -> DaftResult<RecordBatch> {
     let field_id_mapping_provided = field_id_mapping.is_some();
     let mut columns_to_read = columns.clone();
     let columns_to_return = columns;
@@ -259,7 +259,7 @@ async fn read_parquet_single(
         let start_offset = start_offset.unwrap_or(0);
 
         for row in delete_rows.into_iter().map(|r| r as usize) {
-            if row >= start_offset && num_rows_to_read.map_or(true, |n| row < start_offset + n) {
+            if row >= start_offset && num_rows_to_read.is_none_or(|n| row < start_offset + n) {
                 let table_row = row - start_offset;
 
                 if table_row < table.len() {
@@ -291,9 +291,15 @@ async fn read_parquet_single(
     if let Some(predicate) = predicate {
         // If a predicate exists, we need to apply it before a limit and also keep all of the columns that it needs until it is applied
         // TODO ideally pipeline this with IO and before concatenating, rather than after
+        let predicate = BoundExpr::try_new(predicate, &table.schema)?;
         table = table.filter(&[predicate])?;
         if let Some(oc) = columns_to_return {
-            table = table.get_columns(&oc)?;
+            let oc_indices = oc
+                .iter()
+                .map(|name| table.schema.get_index(name))
+                .collect::<DaftResult<Vec<_>>>()?;
+
+            table = table.get_columns(&oc_indices);
         }
         if let Some(nr) = num_rows_to_return {
             table = table.head(nr)?;
@@ -375,7 +381,8 @@ async fn stream_parquet_single(
     metadata: Option<Arc<FileMetaData>>,
     delete_rows: Option<Vec<i64>>,
     maintain_order: bool,
-) -> DaftResult<impl Stream<Item = DaftResult<Table>> + Send> {
+    chunk_size: Option<usize>,
+) -> DaftResult<impl Stream<Item = DaftResult<RecordBatch>> + Send> {
     let field_id_mapping_provided = field_id_mapping.is_some();
     let columns_to_return = columns.map(|s| s.iter().map(|s| (*s).to_string()).collect_vec());
     let num_rows_to_return = num_rows;
@@ -417,6 +424,7 @@ async fn stream_parquet_single(
             metadata,
             maintain_order,
             io_stats,
+            chunk_size,
         )
         .await
     } else {
@@ -427,6 +435,9 @@ async fn stream_parquet_single(
             field_id_mapping,
         )
         .await?;
+
+        let builder = builder.set_chunk_size(chunk_size);
+
         let builder = builder.set_infer_schema_options(schema_infer_options);
 
         let builder = if let Some(columns) = &columns_to_read {
@@ -639,7 +650,7 @@ async fn read_parquet_single_into_arrow(
             }),
             _ => Ok(()),
         }?;
-    };
+    }
 
     let expected_num_columns = if let Some(columns) = &columns {
         columns.len()
@@ -675,7 +686,7 @@ pub fn read_parquet(
     multithreaded_io: bool,
     schema_infer_options: ParquetSchemaInferenceOptions,
     metadata: Option<Arc<FileMetaData>>,
-) -> DaftResult<Table> {
+) -> DaftResult<RecordBatch> {
     let runtime_handle = get_io_runtime(multithreaded_io);
 
     runtime_handle.block_on_current_thread(async {
@@ -761,7 +772,7 @@ pub fn read_parquet_bulk<T: AsRef<str>>(
     metadata: Option<Vec<Arc<FileMetaData>>>,
     delete_map: Option<HashMap<String, Vec<i64>>>,
     chunk_size: Option<usize>,
-) -> DaftResult<Vec<Table>> {
+) -> DaftResult<Vec<RecordBatch>> {
     let runtime_handle = get_io_runtime(multithreaded_io);
 
     let columns = columns.map(|s| s.iter().map(|v| v.as_ref().to_string()).collect::<Vec<_>>());
@@ -810,7 +821,7 @@ pub async fn read_parquet_bulk_async(
     metadata: Option<Vec<Arc<FileMetaData>>>,
     delete_map: Option<HashMap<String, Vec<i64>>>,
     chunk_size: Option<usize>,
-) -> DaftResult<Vec<DaftResult<Table>>> {
+) -> DaftResult<Vec<DaftResult<RecordBatch>>> {
     let task_stream = futures::stream::iter(uris.into_iter().enumerate().map(|(i, uri)| {
         let owned_columns = columns.clone();
         let owned_row_group = row_groups.as_ref().and_then(|rgs| rgs[i].clone());
@@ -873,7 +884,8 @@ pub async fn stream_parquet(
     metadata: Option<Arc<FileMetaData>>,
     maintain_order: bool,
     delete_rows: Option<Vec<i64>>,
-) -> DaftResult<BoxStream<'static, DaftResult<Table>>> {
+    chunk_size: Option<usize>,
+) -> DaftResult<BoxStream<'static, DaftResult<RecordBatch>>> {
     let stream = stream_parquet_single(
         uri.to_string(),
         columns,
@@ -887,6 +899,7 @@ pub async fn stream_parquet(
         metadata,
         delete_rows,
         maintain_order,
+        chunk_size,
     )
     .await?;
     Ok(Box::pin(stream))
@@ -955,7 +968,7 @@ pub fn read_parquet_into_pyarrow_bulk<T: AsRef<str>>(
     Ok(collected.into_iter().map(|(_, v)| v).collect())
 }
 
-pub async fn read_parquet_schema(
+pub async fn read_parquet_schema_and_metadata(
     uri: &str,
     io_client: Arc<IOClient>,
     io_stats: Option<IOStatsRef>,
@@ -968,7 +981,7 @@ pub async fn read_parquet_schema(
 
     let metadata = builder.metadata;
     let arrow_schema = infer_schema_with_options(&metadata, Some(schema_inference_options.into()))?;
-    let schema = Schema::try_from(&arrow_schema)?;
+    let schema = arrow_schema.into();
     Ok((schema, metadata))
 }
 
@@ -1014,7 +1027,7 @@ pub fn read_parquet_statistics(
     io_client: Arc<IOClient>,
     io_stats: Option<IOStatsRef>,
     field_id_mapping: Option<Arc<BTreeMap<i32, Field>>>,
-) -> DaftResult<Table> {
+) -> DaftResult<RecordBatch> {
     let runtime_handle = get_io_runtime(true);
 
     if uris.data_type() != &DataType::Utf8 {
@@ -1086,7 +1099,7 @@ pub fn read_parquet_statistics(
         )),
     ));
 
-    Table::from_nonempty_columns(vec![
+    RecordBatch::from_nonempty_columns(vec![
         uris.clone(),
         row_count_series.into_series(),
         row_group_series.into_series(),
@@ -1169,6 +1182,7 @@ mod tests {
                 None,
                 false,
                 None,
+                None,
             )
             .await?
             .collect::<Vec<_>>()
