@@ -1,18 +1,12 @@
-use std::{
-    cmp::max,
-    collections::HashSet,
-    fs::File,
-    io::{Read, Seek},
-    sync::Arc,
-};
+use std::{cmp::max, collections::HashSet, fs::File, sync::Arc};
 
 use arrow2::{bitmap::Bitmap, io::parquet::read};
 use common_error::DaftResult;
 use common_runtime::{get_compute_runtime, RuntimeTask};
 use daft_core::{prelude::*, utils::arrow::cast_array_for_daft_if_needed};
-use daft_dsl::ExprRef;
-use daft_io::IOStatsRef;
-use daft_table::Table;
+use daft_dsl::{expr::bound_expr::BoundExpr, ExprRef};
+use daft_io::{CountingReader, IOStatsRef};
+use daft_recordbatch::RecordBatch;
 use futures::{stream::BoxStream, StreamExt};
 use itertools::Itertools;
 use rayon::{
@@ -28,7 +22,7 @@ use crate::{
     read::{ArrowChunk, ArrowChunkIters, ParquetSchemaInferenceOptions},
     stream_reader::read::schema::infer_schema_with_options,
     utils::combine_stream,
-    UnableToConvertSchemaToDaftSnafu, PARQUET_MORSEL_SIZE,
+    PARQUET_MORSEL_SIZE,
 };
 
 fn prune_fields_from_schema(
@@ -65,11 +59,11 @@ fn arrow_chunk_to_table(
     predicate: Option<ExprRef>,
     original_columns: Option<&[String]>,
     original_num_rows: Option<usize>,
-) -> DaftResult<Table> {
+) -> DaftResult<RecordBatch> {
     let all_series = arrow_chunk
         .into_iter()
-        .zip(schema_ref.fields.iter())
-        .filter_map(|(mut arr, (f_name, _))| {
+        .zip(schema_ref.field_names())
+        .filter_map(|(mut arr, f_name)| {
             if (*index_so_far + arr.len()) < row_range_start {
                 // No need to process arrays that are less than the start offset
                 return None;
@@ -79,8 +73,7 @@ fn arrow_chunk_to_table(
                 let offset = row_range_start.saturating_sub(*index_so_far);
                 arr = arr.sliced(offset, arr.len() - offset);
             }
-            let series_result =
-                Series::try_from((f_name.as_str(), cast_array_for_daft_if_needed(arr)));
+            let series_result = Series::try_from((f_name, cast_array_for_daft_if_needed(arr)));
             Some(series_result)
         })
         .collect::<DaftResult<Vec<_>>>()?;
@@ -96,8 +89,8 @@ fn arrow_chunk_to_table(
         .into());
     }
 
-    let mut table = Table::new_with_size(
-        Schema::new(all_series.iter().map(|s| s.field().clone()).collect())?,
+    let mut table = RecordBatch::new_with_size(
+        Schema::new(all_series.iter().map(|s| s.field().clone())),
         all_series,
         len,
     )
@@ -126,9 +119,15 @@ fn arrow_chunk_to_table(
 
     // Apply pushdowns if needed
     if let Some(predicate) = predicate {
+        let predicate = BoundExpr::try_new(predicate, &table.schema)?;
         table = table.filter(&[predicate])?;
         if let Some(oc) = &original_columns {
-            table = table.get_columns(oc)?;
+            let oc_indices = oc
+                .iter()
+                .map(|name| table.schema.get_index(name))
+                .collect::<DaftResult<Vec<_>>>()?;
+
+            table = table.get_columns(&oc_indices);
         }
         if let Some(nr) = original_num_rows {
             table = table.head(nr)?;
@@ -148,7 +147,7 @@ pub fn spawn_column_iters_to_table_task(
     original_columns: Option<Vec<String>>,
     original_num_rows: Option<usize>,
     delete_rows: Option<Vec<i64>>,
-    output_sender: tokio::sync::mpsc::Sender<DaftResult<Table>>,
+    output_sender: tokio::sync::mpsc::Sender<DaftResult<RecordBatch>>,
     permit: tokio::sync::OwnedSemaphorePermit,
     channel_size: usize,
 ) -> RuntimeTask<DaftResult<()>> {
@@ -177,7 +176,7 @@ pub fn spawn_column_iters_to_table_task(
 
     compute_runtime.spawn(async move {
         if deserializer_handles.is_empty() {
-            let empty = Table::new_with_size(schema_ref.clone(), vec![], rg_range.num_rows);
+            let empty = RecordBatch::new_with_size(schema_ref.clone(), vec![], rg_range.num_rows);
             let _ = output_sender.send(empty).await;
             return Ok(());
         }
@@ -224,62 +223,8 @@ pub fn spawn_column_iters_to_table_task(
     })
 }
 
-struct CountingReader<R> {
-    reader: R,
-    count: usize,
-    io_stats: Option<IOStatsRef>,
-}
-
-impl<R> CountingReader<R> {
-    fn update_count(&mut self) {
-        if let Some(ios) = &self.io_stats {
-            ios.mark_bytes_read(self.count);
-            self.count = 0;
-        }
-    }
-}
-
-impl<R> Read for CountingReader<R>
-where
-    R: Read + Seek,
-{
-    #[inline]
-    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
-        let read = self.reader.read(buf)?;
-        self.count += read;
-        Ok(read)
-    }
-    #[inline]
-    fn read_vectored(&mut self, bufs: &mut [std::io::IoSliceMut<'_>]) -> std::io::Result<usize> {
-        let read = self.reader.read_vectored(bufs)?;
-        self.count += read;
-        Ok(read)
-    }
-    #[inline]
-    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> std::io::Result<usize> {
-        let read = self.reader.read_to_end(buf)?;
-        self.count += read;
-        Ok(read)
-    }
-}
-
-impl<R> Seek for CountingReader<R>
-where
-    R: Read + Seek,
-{
-    #[inline]
-    fn seek(&mut self, pos: std::io::SeekFrom) -> std::io::Result<u64> {
-        self.reader.seek(pos)
-    }
-}
-
-impl<R> Drop for CountingReader<R> {
-    fn drop(&mut self) {
-        self.update_count();
-    }
-}
-
 #[allow(clippy::too_many_arguments)]
+#[allow(clippy::type_complexity)]
 pub fn local_parquet_read_into_column_iters(
     uri: &str,
     columns: Option<&[String]>,
@@ -319,11 +264,7 @@ pub fn local_parquet_read_into_column_iters(
             file_size: size as usize,
         });
     }
-    let mut reader = CountingReader {
-        reader,
-        count: 0,
-        io_stats,
-    };
+    let mut reader = CountingReader::new(reader, io_stats);
 
     let metadata = match metadata {
         Some(m) => m,
@@ -339,10 +280,7 @@ pub fn local_parquet_read_into_column_iters(
             path: uri.to_string(),
         })?;
     let schema = prune_fields_from_schema(schema, columns)?;
-    let daft_schema =
-        Schema::try_from(&schema).with_context(|_| UnableToConvertSchemaToDaftSnafu {
-            path: uri.to_string(),
-        })?;
+    let daft_schema = Schema::from(&schema);
 
     let row_ranges = build_row_ranges(
         num_rows,
@@ -435,10 +373,7 @@ pub fn local_parquet_read_into_arrow(
             path: uri.to_string(),
         })?;
     let schema = prune_fields_from_schema(schema, columns)?;
-    let daft_schema =
-        Schema::try_from(&schema).with_context(|_| UnableToConvertSchemaToDaftSnafu {
-            path: uri.to_string(),
-        })?;
+    let daft_schema = Schema::from(&schema);
     let chunk_size = chunk_size.unwrap_or(PARQUET_MORSEL_SIZE);
     let max_rows = metadata.num_rows.min(num_rows.unwrap_or(metadata.num_rows));
 
@@ -488,7 +423,6 @@ pub fn local_parquet_read_into_arrow(
             if (curr_index + arr.len()) < row_range.start {
                 // throw arrays less than what we need
                 curr_index += arr.len();
-                continue;
             } else if curr_index < row_range.start {
                 let offset = row_range.start.saturating_sub(curr_index);
                 arrays_so_far.push(arr.sliced(offset, arr.len() - offset));
@@ -534,7 +468,7 @@ pub async fn local_parquet_read_async(
     schema_infer_options: ParquetSchemaInferenceOptions,
     metadata: Option<Arc<parquet2::metadata::FileMetaData>>,
     chunk_size: Option<usize>,
-) -> DaftResult<(Arc<parquet2::metadata::FileMetaData>, Table)> {
+) -> DaftResult<(Arc<parquet2::metadata::FileMetaData>, RecordBatch)> {
     let (send, recv) = tokio::sync::oneshot::channel();
     let uri = uri.to_string();
     rayon::spawn(move || {
@@ -572,8 +506,8 @@ pub async fn local_parquet_read_async(
                 .collect::<Result<Vec<_>, _>>()?;
             Ok((
                 metadata,
-                Table::new_with_size(
-                    Schema::new(converted_arrays.iter().map(|s| s.field().clone()).collect())?,
+                RecordBatch::new_with_size(
+                    Schema::new(converted_arrays.iter().map(|s| s.field().clone())),
                     converted_arrays,
                     num_rows_read,
                 )?,
@@ -599,11 +533,12 @@ pub async fn local_parquet_stream(
     metadata: Option<Arc<parquet2::metadata::FileMetaData>>,
     maintain_order: bool,
     io_stats: Option<IOStatsRef>,
+    chunk_size: Option<usize>,
 ) -> DaftResult<(
     Arc<parquet2::metadata::FileMetaData>,
-    BoxStream<'static, DaftResult<Table>>,
+    BoxStream<'static, DaftResult<RecordBatch>>,
 )> {
-    let chunk_size = PARQUET_MORSEL_SIZE;
+    let chunk_size = chunk_size.unwrap_or(PARQUET_MORSEL_SIZE);
     let (metadata, schema_ref, row_ranges, column_iters) = local_parquet_read_into_column_iters(
         uri,
         columns.as_deref(),
