@@ -1,3 +1,5 @@
+use std::sync::Arc;
+
 use common_error::{DaftError, DaftResult};
 use daft_core::{
     join::JoinType,
@@ -39,10 +41,6 @@ impl PyRecordBatch {
         })
     }
 
-    pub fn cast_to_schema(&self, schema: &PySchema) -> PyResult<Self> {
-        Ok(self.record_batch.cast_to_schema(&schema.schema)?.into())
-    }
-
     pub fn eval_expression_list(&self, py: Python, exprs: Vec<PyExpr>) -> PyResult<Self> {
         let converted_exprs = self.pyexprs_to_bound(exprs)?;
         py.allow_threads(|| {
@@ -487,13 +485,21 @@ impl PyRecordBatch {
             });
         }
 
-        let num_rows = pycolumns.first().unwrap().series.len();
+        let first = pycolumns.first().unwrap().series.len();
+
+        let num_rows = pycolumns.iter().map(|c| c.series.len()).max().unwrap();
 
         let (fields, columns) = pycolumns
             .into_iter()
             .map(|s| (s.series.field().clone(), s.series))
             .unzip::<_, _, Vec<Field>, Vec<Series>>();
 
+        if first == 0 {
+            return Ok(Self {
+                record_batch: RecordBatch::empty(Some(Arc::new(Schema::new(fields)))).unwrap(),
+            });
+        }
+
         Ok(Self {
             record_batch: RecordBatch::new_with_broadcast(Schema::new(fields), columns, num_rows)?,
         })
