@@ -144,12 +144,12 @@ pub enum DataType {
     FixedShapeTensor(Box<DataType>, Vec<u64>),
 
     /// A logical type for sparse tensors with variable shapes.
-    #[display("SparseTensor({_0})")]
-    SparseTensor(Box<DataType>),
+    #[display("SparseTensor[{_0}; indices_offset: {_1}]")]
+    SparseTensor(Box<DataType>, bool),
 
     /// A logical type for sparse tensors with the same shape.
-    #[display("FixedShapeSparseTensor[{_0}; {_1:?}]")]
-    FixedShapeSparseTensor(Box<DataType>, Vec<u64>),
+    #[display("FixedShapeSparseTensor[{_0}; {_1:?}; indices_offset: {_2}]")]
+    FixedShapeSparseTensor(Box<DataType>, Vec<u64>, bool),
 
     #[cfg(feature = "python")]
     Python,
@@ -242,25 +242,13 @@ impl DataType {
                 arrow2::datatypes::Field::new("item", field.to_arrow()?, true),
             ))),
             Self::Map { key, value } => {
+                // To comply with the Arrow spec, Neither the "entries" field nor the "key" field may be nullable.
+                // See https://github.com/apache/arrow/blob/apache-arrow-20.0.0/format/Schema.fbs#L138
                 let struct_type = ArrowType::Struct(vec![
-                    // We never allow null keys in maps for several reasons:
-                    // 1. Null typically represents the absence of a value, which doesn't make sense for a key.
-                    // 2. Null comparisons can be problematic (similar to how f64::NAN != f64::NAN).
-                    // 3. It maintains consistency with common map implementations in arrow (no null keys).
-                    // 4. It simplifies map operations
-                    //
-                    // This decision aligns with the thoughts of team members like Jay and Sammy, who argue that:
-                    // - Nulls in keys could lead to unintuitive behavior
-                    // - If users need to count or group by null values, they can use other constructs like
-                    //   group_by operations on non-map types, which offer more explicit control.
-                    //
-                    // By disallowing null keys, we encourage more robust data modeling practices and
-                    // provide a clearer semantic meaning for map types in our system.
-                    arrow2::datatypes::Field::new("key", key.to_arrow()?, true),
+                    arrow2::datatypes::Field::new("key", key.to_arrow()?, false),
                     arrow2::datatypes::Field::new("value", value.to_arrow()?, true),
                 ]);
-
-                let struct_field = arrow2::datatypes::Field::new("entries", struct_type, true);
+                let struct_field = arrow2::datatypes::Field::new("entries", struct_type, false);
 
                 Ok(ArrowType::map(struct_field, false))
             }
@@ -311,6 +299,7 @@ impl DataType {
             FixedSizeList(child_dtype, size) => {
                 FixedSizeList(Box::new(child_dtype.to_physical()), *size)
             }
+            Struct(fields) => Struct(fields.iter().map(|field| field.to_physical()).collect()),
             Map { key, value } => List(Box::new(Struct(vec![
                 Field::new("key", key.to_physical()),
                 Field::new("value", value.to_physical()),
@@ -338,12 +327,12 @@ impl DataType {
                 Box::new(*dtype.clone()),
                 usize::try_from(shape.iter().product::<u64>()).unwrap(),
             ),
-            SparseTensor(dtype) => Struct(vec![
+            SparseTensor(dtype, _) => Struct(vec![
                 Field::new("values", List(Box::new(*dtype.clone()))),
                 Field::new("indices", List(Box::new(Self::UInt64))),
                 Field::new("shape", List(Box::new(Self::UInt64))),
             ]),
-            FixedShapeSparseTensor(dtype, shape) => Struct(vec![
+            FixedShapeSparseTensor(dtype, shape, _) => Struct(vec![
                 Field::new("values", List(Box::new(*dtype.clone()))),
                 {
                     let largest_index = std::cmp::max(shape.iter().product::<u64>(), 1) - 1;
@@ -435,15 +424,6 @@ impl DataType {
         }
     }
 
-    #[inline]
-    pub fn fixed_size(&self) -> Option<usize> {
-        match self {
-            Self::FixedSizeList(_, size) => Some(*size),
-            Self::Embedding(_, size) => Some(*size),
-            _ => None,
-        }
-    }
-
     #[inline]
     pub fn is_integer(&self) -> bool {
         matches!(
@@ -477,6 +457,11 @@ impl DataType {
         }
     }
 
+    #[inline]
+    pub fn is_embedding(&self) -> bool {
+        matches!(self, Self::Embedding(..))
+    }
+
     #[inline]
     pub fn is_tensor(&self) -> bool {
         matches!(self, Self::Tensor(..))
@@ -536,6 +521,106 @@ impl DataType {
         }
     }
 
+    #[inline]
+    pub fn is_int8(&self) -> bool {
+        matches!(self, Self::Int8)
+    }
+
+    #[inline]
+    pub fn is_int16(&self) -> bool {
+        matches!(self, Self::Int16)
+    }
+
+    #[inline]
+    pub fn is_int32(&self) -> bool {
+        matches!(self, Self::Int32)
+    }
+
+    #[inline]
+    pub fn is_int64(&self) -> bool {
+        matches!(self, Self::Int64)
+    }
+
+    #[inline]
+    pub fn is_uint8(&self) -> bool {
+        matches!(self, Self::UInt8)
+    }
+
+    #[inline]
+    pub fn is_uint16(&self) -> bool {
+        matches!(self, Self::UInt16)
+    }
+
+    #[inline]
+    pub fn is_uint32(&self) -> bool {
+        matches!(self, Self::UInt32)
+    }
+
+    #[inline]
+    pub fn is_uint64(&self) -> bool {
+        matches!(self, Self::UInt64)
+    }
+
+    #[inline]
+    pub fn is_float32(&self) -> bool {
+        matches!(self, Self::Float32)
+    }
+
+    #[inline]
+    pub fn is_float64(&self) -> bool {
+        matches!(self, Self::Float64)
+    }
+
+    #[inline]
+    pub fn is_decimal128(&self) -> bool {
+        matches!(self, Self::Decimal128(_, _))
+    }
+
+    #[inline]
+    pub fn is_timestamp(&self) -> bool {
+        matches!(self, Self::Timestamp(..))
+    }
+
+    #[inline]
+    pub fn is_date(&self) -> bool {
+        matches!(self, Self::Date)
+    }
+
+    #[inline]
+    pub fn is_time(&self) -> bool {
+        matches!(self, Self::Time(..))
+    }
+
+    #[inline]
+    pub fn is_duration(&self) -> bool {
+        matches!(self, Self::Duration(..))
+    }
+
+    #[inline]
+    pub fn is_interval(&self) -> bool {
+        matches!(self, Self::Interval)
+    }
+
+    #[inline]
+    pub fn is_binary(&self) -> bool {
+        matches!(self, Self::Binary)
+    }
+
+    #[inline]
+    pub fn is_fixed_size_binary(&self) -> bool {
+        matches!(self, Self::FixedSizeBinary(_))
+    }
+
+    #[inline]
+    pub fn is_fixed_size_list(&self) -> bool {
+        matches!(self, Self::FixedSizeList(..))
+    }
+
+    #[inline]
+    pub fn is_struct(&self) -> bool {
+        matches!(self, Self::Struct(..))
+    }
+
     #[inline]
     pub fn is_extension(&self) -> bool {
         matches!(self, Self::Extension(..))
@@ -658,6 +743,143 @@ impl DataType {
         let val: DataTypePayload = serde_json::from_str(input)?;
         Ok(val.datatype)
     }
+
+    /// If the datatype variant has a `size` property, return it.
+    /// For example, `FixedSizeBinary` and `FixedSizeList` have a size property.
+    pub fn fixed_size(&self) -> DaftResult<usize> {
+        match self {
+            Self::FixedSizeBinary(size) => Ok(*size),
+            Self::FixedSizeList(_, size) => Ok(*size),
+            Self::Embedding(_, size) => Ok(*size),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `fixed_size` property",
+            ))),
+        }
+    }
+    /// if the datatype variant has a shape, return it.
+    /// For example, `FixedShapeImage` and `FixedShapeTensor` have a fixed shape.
+    pub fn fixed_shape(&self) -> DaftResult<Vec<u64>> {
+        match self {
+            Self::FixedShapeImage(_, height, width) => Ok(vec![*height as u64, *width as u64]),
+            Self::FixedShapeTensor(_, shape) => Ok(shape.clone()),
+            Self::FixedShapeSparseTensor(_, shape, _) => Ok(shape.clone()),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `fixed_shape` property",
+            ))),
+        }
+    }
+
+    /// if the datatype contains a timeunit, return it.
+    pub fn time_unit(&self) -> DaftResult<TimeUnit> {
+        match self {
+            Self::Timestamp(unit, _) => Ok(*unit),
+            Self::Time(unit) => Ok(*unit),
+            Self::Duration(unit) => Ok(*unit),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `time_unit` property",
+            ))),
+        }
+    }
+
+    /// if the datatype contains a timezone, return it.
+    pub fn time_zone(&self) -> DaftResult<Option<&str>> {
+        match self {
+            Self::Timestamp(_, timezone) => Ok(timezone.as_deref()),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `time_zone` property",
+            ))),
+        }
+    }
+
+    /// if the datatype contains an image mode, return it.
+    /// For example, `Image` and `FixedShapeImage` have an image mode.
+    pub fn image_mode(&self) -> DaftResult<Option<ImageMode>> {
+        match self {
+            Self::Image(mode) => Ok(*mode),
+            Self::FixedShapeImage(mode, ..) => Ok(Some(*mode)),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have an `image_mode` property",
+            ))),
+        }
+    }
+
+    /// if the datatype contains an inner datatype, return it.
+    /// For example, `List` and `FixedSizeList` have an inner datatype.
+    pub fn dtype(&self) -> DaftResult<&Self> {
+        match self {
+            Self::List(dtype) | Self::FixedSizeList(dtype, _) => Ok(dtype),
+            Self::Embedding(dtype, _) => Ok(dtype),
+            Self::Extension(_, dtype, _) => Ok(dtype),
+            Self::Tensor(dtype) => Ok(dtype),
+            Self::SparseTensor(dtype, _) => Ok(dtype),
+            Self::FixedShapeTensor(dtype, _) => Ok(dtype),
+            Self::FixedShapeSparseTensor(dtype, _, _) => Ok(dtype),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `dtype` property",
+            ))),
+        }
+    }
+
+    /// if the datatype is a struct, return its fields.
+    pub fn fields(&self) -> DaftResult<&[Field]> {
+        match self {
+            Self::Struct(fields) => Ok(fields),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `fields` property",
+            ))),
+        }
+    }
+
+    /// if the datatype is a decimal, return its precision.
+    pub fn precision(&self) -> DaftResult<usize> {
+        match self {
+            Self::Decimal128(precision, _) => Ok(*precision),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `precision` property",
+            ))),
+        }
+    }
+
+    /// if the datatype is a decimal, return its scale.
+    pub fn scale(&self) -> DaftResult<usize> {
+        match self {
+            Self::Decimal128(_, scale) => Ok(*scale),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `scale` property",
+            ))),
+        }
+    }
+
+    /// if the datatype is a sparse tensor, return whether it uses offset indices.
+    pub fn use_offset_indices(&self) -> DaftResult<bool> {
+        match self {
+            Self::SparseTensor(_, use_offset) => Ok(*use_offset),
+            Self::FixedShapeSparseTensor(_, _, use_offset) => Ok(*use_offset),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `use_offset_indices` property",
+            ))),
+        }
+    }
+
+    /// if the datatype is a map, return its key type.
+    pub fn key_type(&self) -> DaftResult<&Self> {
+        match self {
+            Self::Map { key, .. } => Ok(key),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `key_type` property",
+            ))),
+        }
+    }
+
+    /// if the datatype is a map, return its value type.
+    pub fn value_type(&self) -> DaftResult<&Self> {
+        match self {
+            Self::Map { value, .. } => Ok(value),
+            _ => Err(DaftError::TypeError(format!(
+                "DataType {self:?} does not have a `value_type` property",
+            ))),
+        }
+    }
 }
 
 #[expect(
