@@ -2,24 +2,50 @@ use std::sync::Arc;
 
 use common_error::{DaftError, DaftResult};
 use common_file_formats::FileFormat;
+use daft_core::prelude::*;
 use daft_logical_plan::OutputFileInfo;
 use daft_micropartition::MicroPartition;
 use daft_recordbatch::RecordBatch;
 
-use crate::{AsyncFileWriter, WriterFactory};
+use crate::{
+    parquet_writer::{create_native_parquet_writer, native_parquet_writer_supported},
+    AsyncFileWriter, WriterFactory,
+};
+
+enum WriterType {
+    Native,
+    Pyarrow,
+}
 
 /// PhysicalWriterFactory is a factory for creating physical writers, i.e. parquet, csv writers.
 pub struct PhysicalWriterFactory {
     output_file_info: OutputFileInfo,
-    native: bool, // TODO: Implement native writer
+    schema: SchemaRef,
+    writer_type: WriterType,
 }
 
 impl PhysicalWriterFactory {
-    pub fn new(output_file_info: OutputFileInfo) -> Self {
-        Self {
+    pub fn new(
+        output_file_info: OutputFileInfo,
+        file_schema: SchemaRef,
+        native_enabled: bool,
+    ) -> DaftResult<Self> {
+        let writer_type = if native_enabled
+            && native_parquet_writer_supported(
+                output_file_info.file_format,
+                &output_file_info.root_dir,
+                &file_schema,
+            )? {
+            WriterType::Native
+        } else {
+            WriterType::Pyarrow
+        };
+
+        Ok(Self {
             output_file_info,
-            native: false,
-        }
+            schema: file_schema,
+            writer_type,
+        })
     }
 }
 
@@ -32,19 +58,22 @@ impl WriterFactory for PhysicalWriterFactory {
         file_idx: usize,
         partition_values: Option<&RecordBatch>,
     ) -> DaftResult<Box<dyn AsyncFileWriter<Input = Self::Input, Result = Self::Result>>> {
-        match self.native {
-            true => unimplemented!(),
-            false => {
-                let writer = create_pyarrow_file_writer(
-                    &self.output_file_info.root_dir,
-                    file_idx,
-                    self.output_file_info.compression.as_ref(),
-                    self.output_file_info.io_config.as_ref(),
-                    self.output_file_info.file_format,
-                    partition_values,
-                )?;
-                Ok(writer)
-            }
+        match self.writer_type {
+            WriterType::Native => create_native_writer(
+                &self.output_file_info.root_dir,
+                file_idx,
+                &self.schema,
+                self.output_file_info.file_format,
+                partition_values,
+            ),
+            WriterType::Pyarrow => create_pyarrow_file_writer(
+                &self.output_file_info.root_dir,
+                file_idx,
+                self.output_file_info.compression.as_ref(),
+                self.output_file_info.io_config.as_ref(),
+                self.output_file_info.file_format,
+                partition_values,
+            ),
         }
     }
 }
@@ -76,3 +105,21 @@ pub fn create_pyarrow_file_writer(
         )),
     }
 }
+
+fn create_native_writer(
+    root_dir: &str,
+    file_idx: usize,
+    schema: &SchemaRef,
+    file_format: FileFormat,
+    partition_values: Option<&RecordBatch>,
+) -> DaftResult<Box<dyn AsyncFileWriter<Input = Arc<MicroPartition>, Result = Option<RecordBatch>>>>
+{
+    match file_format {
+        FileFormat::Parquet => {
+            create_native_parquet_writer(root_dir, schema, file_idx, partition_values)
+        }
+        _ => Err(DaftError::ComputeError(
+            "Unsupported file format for native write".to_string(),
+        )),
+    }
+}
