@@ -153,7 +153,10 @@ pub mod pylib {
             .expect("Failed to acquire write lock on function registry");
         functions_registry.register::<daft_functions::numeric::NumericFunctions>();
         functions_registry.register::<daft_functions::float::FloatFunctions>();
+        functions_registry.register::<daft_functions::uri::UriFunctions>();
         functions_registry.register::<daft_image::functions::ImageFunctions>();
+        functions_registry.register::<daft_functions_list::ListFunctions>();
+        functions_registry.register::<daft_functions_utf8::Utf8Functions>();
         functions_registry.register::<daft_functions_json::JsonFunctions>();
 
         Ok(())
