@@ -8,6 +8,7 @@ import pytest
 
 import daft
 from daft.expressions import col
+from tests.conftest import get_tests_daft_runner_name
 from tests.cookbook.assets import COOKBOOK_DATA_CSV
 
 COLUMNS = [
@@ -42,9 +43,10 @@ def service_requests_csv_pd_df():
     return pd.read_csv(COOKBOOK_DATA_CSV, keep_default_na=False)[COLUMNS]
 
 
-@pytest.fixture(scope="module", params=[1, 2])
+@pytest.fixture(
+    scope="module",
+    params=[1, 2] if get_tests_daft_runner_name() != "native" else [1],
+)
 def repartition_nparts(request):
-    """Adds a `n_repartitions` parameter to test cases which provides the number of
-    partitions that the test case should repartition its dataset into for testing
-    """
+    """Adds a `n_repartitions` parameter to test cases which provides the number of partitions that the test case should repartition its dataset into for testing."""
     return request.param
