@@ -84,7 +84,7 @@ def test_create_dataframe_empty_list() -> None:
 def test_create_dataframe_list(valid_data: list[dict[str, float]]) -> None:
     df = daft.from_pylist(valid_data)
     assert len(df) == len(valid_data)
-    assert len(df._preview.preview_partition) == len(valid_data)
+    assert len(df._preview.partition) == len(valid_data)
     assert set(df.column_names) == set(COL_NAMES)
 
 
@@ -92,7 +92,7 @@ def test_create_dataframe_list_data_longer_than_preview(valid_data: list[dict[st
     valid_data = valid_data * 3
     df = daft.from_pylist(valid_data)
     assert len(df) == len(valid_data)
-    assert len(df._preview.preview_partition) == 8
+    assert len(df._preview.partition) == 8
     assert set(df.column_names) == set(COL_NAMES)
 
 
@@ -136,7 +136,7 @@ def test_create_dataframe_pydict(valid_data: list[dict[str, float]]) -> None:
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
     df = daft.from_pydict(pydict)
     assert len(df) == len(valid_data)
-    assert len(df._preview.preview_partition) == len(valid_data)
+    assert len(df._preview.partition) == len(valid_data)
     assert set(df.column_names) == set(COL_NAMES)
 
 
@@ -145,7 +145,7 @@ def test_create_dataframe_pydict_data_longer_than_preview(valid_data: list[dict[
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
     df = daft.from_pydict(pydict)
     assert len(df) == len(valid_data)
-    assert len(df._preview.preview_partition) == 8
+    assert len(df._preview.partition) == 8
     assert set(df.column_names) == set(COL_NAMES)
 
 
@@ -180,7 +180,7 @@ def test_create_dataframe_arrow(valid_data: list[dict[str, float]], multiple) ->
     if multiple:
         t = pa.concat_tables(t)
     assert len(df) == len(t)
-    assert len(df._preview.preview_partition) == (8 if multiple else len(t))
+    assert len(df._preview.partition) == (8 if multiple else len(t))
     assert set(df.column_names) == set(t.column_names)
     casted_field = t.schema.field("variety").with_type(pa.large_string())
     expected = t.cast(t.schema.set(t.schema.get_field_index("variety"), casted_field))
@@ -274,7 +274,7 @@ def test_create_dataframe_pandas(valid_data: list[dict[str, float]], multiple) -
     if multiple:
         pd_df = pd.concat(pd_df).reset_index(drop=True)
     assert len(df) == len(pd_df)
-    assert len(df._preview.preview_partition) == (8 if multiple else len(pd_df))
+    assert len(df._preview.partition) == (8 if multiple else len(pd_df))
     assert set(df.column_names) == set(pd_df.columns)
     # Check roundtrip.
     pd.testing.assert_frame_equal(df.to_pandas(), pd_df)
@@ -323,22 +323,14 @@ def test_create_dataframe_pandas_py_object(valid_data: list[dict[str, float]]) -
             DataType.map(DataType.string(), DataType.int64()),
             id="pa_map",
         ),
-        # TODO(Colin): Enable this test once cast_array_for_daft_if_needed in src/daft-core/src/utils/arrow.rs supports nested maps
-        # pytest.param(
-        #     pa.array(
-        #         [{"a": {"b": 1}, "c": {"d": 2}}, {"e": {"f": 3}, "g": {"h": 4}}],
-        #         type=pa.map_(pa.string(), pa.map_(pa.string(), pa.int64())),
-        #     ),
-        #     DataType.map(
-        #         DataType.struct(
-        #             {
-        #                 "key": DataType.string(),
-        #                 "value": DataType.map(DataType.struct({"key": DataType.string(), "value": DataType.int64()})),
-        #             }
-        #         )
-        #     ),
-        #     id="pa_nested_map",
-        # ),
+        pytest.param(
+            pa.array(
+                [[("a", [("b", 1)]), ("c", [("d", 2)])], [("e", [("f", 3)]), ("g", [("h", 4)])]],
+                type=pa.map_(pa.string(), pa.map_(pa.string(), pa.int64())),
+            ),
+            DataType.map(DataType.string(), DataType.map(DataType.string(), DataType.int64())),
+            id="pa_nested_map",
+        ),
         pytest.param(
             pa.chunked_array([pa.array([[1, 2, 3], [1, 2], [1]])]),
             DataType.list(DataType.int64()),
