@@ -6,7 +6,6 @@ import pytest
 import daft
 from daft import col
 from daft.datatype import DataType
-from daft.errors import ExpressionTypeError
 from tests.conftest import get_tests_daft_runner_name
 from tests.utils import sort_arrow_table
 
@@ -725,8 +724,23 @@ def test_join_all_null(join_strategy, join_type, expected, make_df, repartition_
     [None, "hash", "sort_merge", "sort_merge_aligned_boundaries", "broadcast"],
     indirect=True,
 )
-@pytest.mark.parametrize("join_type", ["inner", "left", "right", "outer"])
-def test_join_null_type_column(join_strategy, join_type, make_df, with_morsel_size):
+@pytest.mark.parametrize(
+    "join_type,expected",
+    [
+        ("inner", {"id": [], "values_left": [], "values_right": []}),
+        ("left", {"id": [None, None, None], "values_left": ["a1", "b1", "c1"], "values_right": [None, None, None]}),
+        ("right", {"id": [None, None, None], "values_left": [None, None, None], "values_right": ["a2", "b2", "c2"]}),
+        (
+            "outer",
+            {
+                "id": [None, None, None, None, None, None],
+                "values_left": ["a1", "b1", "c1", None, None, None],
+                "values_right": [None, None, None, "a2", "b2", "c2"],
+            },
+        ),
+    ],
+)
+def test_join_null_type_column(join_strategy, join_type, expected, make_df, with_morsel_size):
     skip_invalid_join_strategies(join_strategy, join_type)
 
     daft_df = make_df(
@@ -742,8 +756,10 @@ def test_join_null_type_column(join_strategy, join_type, make_df, with_morsel_si
         }
     )
 
-    with pytest.raises((ExpressionTypeError, ValueError)):
-        daft_df.join(daft_df2, on="id", how=join_type, strategy=join_strategy)
+    daft_df = daft_df.join(daft_df2, on="id", how=join_type, strategy=join_strategy).sort(
+        ["values_left", "values_right"]
+    )
+    assert pa.Table.from_pydict(daft_df.to_pydict()) == pa.Table.from_pydict(expected)
 
 
 @pytest.mark.parametrize("repartition_nparts", [1, 2, 4])
@@ -860,8 +876,44 @@ def test_join_semi_anti_different_names(
     )
 
 
-@pytest.mark.parametrize("join_type", ["inner", "left", "right", "outer"])
-def test_join_true_join_keys(join_type, make_df, with_morsel_size):
+@pytest.mark.parametrize(
+    "join_type,expected_dtypes",
+    [
+        (
+            "inner",
+            {
+                "id": DataType.int64(),
+                "values": DataType.string(),
+                "right.values": DataType.string(),
+            },
+        ),
+        (
+            "left",
+            {
+                "id": DataType.int64(),
+                "values": DataType.string(),
+                "right.values": DataType.string(),
+            },
+        ),
+        (
+            "right",
+            {
+                "id": DataType.float64(),
+                "values": DataType.string(),
+                "right.values": DataType.string(),
+            },
+        ),
+        (
+            "outer",
+            {
+                "id": DataType.float64(),
+                "values": DataType.string(),
+                "right.values": DataType.string(),
+            },
+        ),
+    ],
+)
+def test_join_true_join_keys(join_type, expected_dtypes, make_df, with_morsel_size):
     daft_df = make_df(
         {
             "id": [1, 2, 3],
@@ -878,9 +930,9 @@ def test_join_true_join_keys(join_type, make_df, with_morsel_size):
     result = daft_df.join(daft_df2, left_on=["id", "values"], right_on=["id", col("values").str.left(1)], how=join_type)
 
     assert result.schema().column_names() == ["id", "values", "right.values"]
-    assert result.schema()["id"].dtype == daft_df.schema()["id"].dtype
-    assert result.schema()["values"].dtype == daft_df.schema()["values"].dtype
-    assert result.schema()["right.values"].dtype == daft_df2.schema()["values"].dtype
+    assert result.schema()["id"].dtype == expected_dtypes["id"]
+    assert result.schema()["values"].dtype == expected_dtypes["values"]
+    assert result.schema()["right.values"].dtype == expected_dtypes["right.values"]
 
 
 @pytest.mark.parametrize(
@@ -1208,30 +1260,12 @@ def test_cross_join(left_partitions, right_partitions, make_df, with_morsel_size
     ],
 )
 def test_join_empty(join_type, repartition_nparts, left, right, expected, make_df, with_morsel_size):
-    left = pa.Table.from_pydict(
-        left,
-        schema=pa.schema(
-            [
-                ("a", pa.int32()),
-                ("b", pa.string()),
-            ]
-        ),
-    )
     left_df = make_df(
         left,
         repartition=repartition_nparts,
         repartition_columns=["a"],
     )
 
-    right = pa.Table.from_pydict(
-        right,
-        schema=pa.schema(
-            [
-                ("c", pa.int32()),
-                ("d", pa.string()),
-            ]
-        ),
-    )
     right_df = make_df(
         right,
         repartition=repartition_nparts,
@@ -1246,11 +1280,121 @@ def test_join_empty(join_type, repartition_nparts, left, right, expected, make_d
         right_on = ["c"]
 
     result = left_df.join(right_df, left_on=left_on, right_on=right_on, how=join_type)
-    if join_type in ["inner", "left", "right", "outer", "cross"]:
-        result = result.sort(["a", "b", "c", "d"])
-    else:
-        result = result.sort(["a", "b"])
 
-    expected_result = expected[join_type]
+    sort_by = ["a", "b", "c", "d"] if join_type in ["inner", "left", "right", "outer", "cross"] else ["a", "b"]
 
-    assert result.to_pydict() == expected_result
+    assert sort_arrow_table(pa.Table.from_pydict(result.to_pydict()), *sort_by) == sort_arrow_table(
+        pa.Table.from_pydict(expected[join_type]), *sort_by
+    )
+
+
+@pytest.mark.parametrize(
+    "join_type,expected",
+    [
+        ("inner", {"a": [1, 1, 2], "b": ["a", "a", "b"], "c": [1.0, 1.0, 2.0], "d": ["g", "h", "j"]}),
+        (
+            "left",
+            {
+                "a": [1, 1, 2, 3, 4, 5, 6],
+                "b": ["a", "a", "b", "c", "d", "e", "f"],
+                "c": [1.0, 1.0, 2.0, None, None, None, None],
+                "d": ["g", "h", "j", None, None, None, None],
+            },
+        ),
+        (
+            "right",
+            {
+                "a": [1, 1, 2, None, None, None],
+                "b": ["a", "a", "b", None, None, None],
+                "c": [1.0, 1.0, 2.0, 1.5, 2.1, 2.5],
+                "d": ["g", "h", "j", "i", "k", "l"],
+            },
+        ),
+        (
+            "outer",
+            {
+                "a": [1, 1, 2, 3, 4, 5, 6, None, None, None],
+                "b": ["a", "a", "b", "c", "d", "e", "f", None, None, None],
+                "c": [1.0, 1.0, 2.0, None, None, None, None, 1.5, 2.1, 2.5],
+                "d": ["g", "h", "j", None, None, None, None, "i", "k", "l"],
+            },
+        ),
+        (
+            "anti",
+            {
+                "a": [3, 4, 5, 6],
+                "b": ["c", "d", "e", "f"],
+            },
+        ),
+        (
+            "semi",
+            {
+                "a": [1, 2],
+                "b": ["a", "b"],
+            },
+        ),
+    ],
+)
+@pytest.mark.parametrize("repartition_nparts", [1, 2, 4])
+def test_join_different_join_key_types(join_type, expected, repartition_nparts, make_df, with_morsel_size):
+    left_df = make_df(
+        {"a": [1, 2, 3, 4, 5, 6], "b": ["a", "b", "c", "d", "e", "f"]},
+        repartition=repartition_nparts,
+        repartition_columns=["a"],
+    )
+
+    right_df = make_df(
+        {"c": [1.0, 1.0, 1.5, 2.0, 2.1, 2.5], "d": ["g", "h", "i", "j", "k", "l"]},
+        repartition=repartition_nparts,
+        repartition_columns=["c"],
+    )
+
+    result = left_df.join(right_df, left_on="a", right_on="c", how=join_type)
+
+    sort_by = ["a", "b", "c", "d"] if join_type in ["inner", "left", "right", "outer"] else ["a", "b"]
+
+    assert sort_arrow_table(pa.Table.from_pydict(result.to_pydict()), *sort_by) == sort_arrow_table(
+        pa.Table.from_pydict(expected), *sort_by
+    )
+
+
+# Tests issue 4113.
+def test_self_join_with_projection():
+    df1 = daft.from_pydict(
+        {
+            "pattern_id_l": ["a"],
+            "pattern_id_r": ["b"],
+        }
+    )
+
+    df2 = daft.from_pydict(
+        {
+            "id": ["1", "2", "3"],
+            "pattern_id": ["a", "b", "c"],
+        }
+    )
+
+    result_df = (
+        df1.join(
+            df2,
+            left_on="pattern_id_l",
+            right_on="pattern_id",
+        )
+        .select(daft.col("id").alias("id_l"), daft.col("pattern_id_r"))
+        .join(
+            df2,
+            left_on="pattern_id_r",
+            right_on="pattern_id",
+        )
+        .select(
+            daft.col("id_l"),
+            daft.col("id").alias("id_r"),
+        )
+    )
+
+    expected = {
+        "id_l": ["1"],
+        "id_r": ["2"],
+    }
+
+    assert pa.Table.from_pydict(result_df.to_pydict()) == pa.Table.from_pydict(expected)
