@@ -10,8 +10,8 @@ import daft
 from daft.datatype import DataType, TimeUnit
 from daft.expressions import col, lit
 from daft.expressions.testing import expr_structurally_equal
+from daft.recordbatch import MicroPartition
 from daft.series import Series
-from daft.table import MicroPartition
 
 
 @pytest.mark.parametrize(
@@ -40,7 +40,7 @@ def test_make_lit(data, expected_dtype) -> None:
     assert literal.name() == "literal"
     empty_table = MicroPartition.empty()
     lit_table = empty_table.eval_expression_list([literal])
-    series = lit_table.get_column("literal")
+    series = lit_table.get_column_by_name("literal")
     assert series.datatype() == expected_dtype
     repr_out = repr(literal)
 
@@ -118,11 +118,38 @@ def test_repr_functions_sign() -> None:
     assert repr_out == repr(copied)
 
 
+def test_repr_functions_signum() -> None:
+    a = col("a")
+    y = a.signum()
+    repr_out = repr(y)
+    assert repr_out == "sign(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
+def test_repr_functions_negate() -> None:
+    a = col("a")
+    y = a.negate()
+    repr_out = repr(y)
+    assert repr_out == "negative(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
+def test_repr_functions_negative() -> None:
+    a = col("a")
+    y = a.negative()
+    repr_out = repr(y)
+    assert repr_out == "negative(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
 def test_repr_functions_round() -> None:
     a = col("a")
     y = a.round()
     repr_out = repr(y)
-    assert repr_out == "round(col(a))"
+    assert repr_out == "round(col(a), lit(0))"
     copied = copy.deepcopy(y)
     assert repr_out == repr(copied)
 
@@ -158,9 +185,9 @@ def test_repr_functions_log10() -> None:
 
 def test_repr_functions_log() -> None:
     a = col("a")
-    y = a.log()
+    y = a.log(2)
     repr_out = repr(y)
-    assert repr_out == "log(col(a))"
+    assert repr_out == "log(col(a), lit(2))"
     copied = copy.deepcopy(y)
     assert repr_out == repr(copied)
 
@@ -174,6 +201,15 @@ def test_repr_functions_ln() -> None:
     assert repr_out == repr(copied)
 
 
+def test_repr_functions_log1p() -> None:
+    a = col("a")
+    y = a.log1p()
+    repr_out = repr(y)
+    assert repr_out == "log1p(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
 def test_repr_functions_shift_left() -> None:
     a = col("a")
     b = col("b")
@@ -200,7 +236,12 @@ def test_repr_functions_shift_right() -> None:
         "sin",
         "cos",
         "tan",
+        "csc",
+        "sec",
         "cot",
+        "sinh",
+        "cosh",
+        "tanh",
         "arcsin",
         "arccos",
         "arctan",
@@ -254,6 +295,15 @@ def test_repr_functions_arcsinh() -> None:
     assert repr_out == repr(copied)
 
 
+def test_repr_functions_unix_date() -> None:
+    a = col("a")
+    y = a.dt.unix_date()
+    repr_out = repr(y)
+    assert repr_out == "unix_date(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
 def test_repr_functions_day() -> None:
     a = col("a")
     y = a.dt.day()
@@ -272,6 +322,15 @@ def test_repr_functions_month() -> None:
     assert repr_out == repr(copied)
 
 
+def test_repr_functions_quarter() -> None:
+    a = col("a")
+    y = a.dt.quarter()
+    repr_out = repr(y)
+    assert repr_out == "quarter(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
 def test_repr_functions_year() -> None:
     a = col("a")
     y = a.dt.year()
@@ -290,6 +349,33 @@ def test_repr_functions_day_of_week() -> None:
     assert repr_out == repr(copied)
 
 
+def test_repr_functions_day_of_month() -> None:
+    a = col("a")
+    y = a.dt.day_of_month()
+    repr_out = repr(y)
+    assert repr_out == "day_of_month(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
+def test_repr_functions_day_of_year() -> None:
+    a = col("a")
+    y = a.dt.day_of_year()
+    repr_out = repr(y)
+    assert repr_out == "day_of_year(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
+def test_repr_functions_week_of_year() -> None:
+    a = col("a")
+    y = a.dt.week_of_year()
+    repr_out = repr(y)
+    assert repr_out == "week_of_year(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
 def test_repr_functions_exp() -> None:
     a = col("a")
     y = a.exp()
@@ -299,6 +385,15 @@ def test_repr_functions_exp() -> None:
     assert repr_out == repr(copied)
 
 
+def test_repr_functions_expm1() -> None:
+    a = col("a")
+    y = a.expm1()
+    repr_out = repr(y)
+    assert repr_out == "expm1(col(a))"
+    copied = copy.deepcopy(y)
+    assert repr_out == repr(copied)
+
+
 def test_repr_functions_sqrt() -> None:
     a = col("a")
     y = a.sqrt()
@@ -507,7 +602,7 @@ def test_datetime_lit_different_timeunits(timeunit, expected) -> None:
     pa_table = pa.table({"dt": pa_array})
     mp = MicroPartition.from_arrow(pa_table)
 
-    series = mp.get_column("dt")
+    series = mp.get_column_by_name("dt")
     output = repr(series)
 
     # Series repr is very long, so we just check the last line which contains the timestamp
@@ -656,3 +751,53 @@ def test_list_value_counts_degenerate():
 
     # Check the result for null values
     assert result_null.to_pydict() == {"value_counts": [[], []]}
+
+
+@pytest.mark.parametrize(
+    # we don't need to test all types, just a few as a sanity check. The rest are tested in the sql tests
+    "sql, actual",
+    [
+        ("int32", DataType.int32()),
+        ("int64", DataType.int64()),
+        ("float32", DataType.float32()),
+        ("real", DataType.float32()),
+        ("float(5)", DataType.float32()),
+        ("float64", DataType.float64()),
+        ("float", DataType.float64()),
+        ("double", DataType.float64()),
+        ("double 10", DataType.float64()),
+        ("float(30)", DataType.float64()),
+        ("text", DataType.string()),
+        ("string", DataType.string()),
+        ("varchar", DataType.string()),
+    ],
+)
+def test_cast_sql_string(sql, actual):
+    expr = col("a").cast(sql)
+    actual = col("a").cast(actual)
+    df = daft.from_pydict({"a": [1, 2, 3]}).select(expr)
+    actual_df = daft.from_pydict({"a": [1, 2, 3]}).select(actual)
+    assert df.schema() == actual_df.schema()
+
+
+@pytest.mark.parametrize(
+    # Some sql expressions were failing as documented previously
+    "sql",
+    ["char", "character", "char varying", "character varying"],
+)
+def test_cast_sql_string_failing(sql):
+    with pytest.raises(Exception):
+        expr = col("a").cast(sql)
+        daft.from_pydict({"a": [1, 2, 3]}).select(expr)
+
+
+def test_drop_nan():
+    dd = {"n": [1.0, 2.0, None]}
+    df = daft.from_pydict(dd)
+    assert df.drop_nan().to_pydict() == {"n": [1.0, 2.0]}
+
+
+def test_drop_nan_no_nans():
+    dd = {"n": [1, 1, 2]}
+    df = daft.from_pydict(dd)
+    assert df.drop_nan().to_pydict() == dd
