@@ -26,7 +26,7 @@ app.add_exception_handler(RateLimitExceeded, rate_limit_exceeded_handler)
 @app.get(OBJECT_KEY_URL)
 @limiter.shared_limit(limit_value="80/second", scope="my_shared_limit")
 async def rate_limited_bucket_get(request: Request, item_id: str):
-    """This endpoint will just echo the `item_id` and return that as the response body"""
+    """This endpoint will just echo the `item_id` and return that as the response body."""
     result = item_id.encode("utf-8")
     return Response(
         status_code=200,
