@@ -8,7 +8,7 @@ import pytest
 from daft.datatype import DataType
 from daft.expressions import ExpressionsProjection, col
 from daft.logical.schema import Schema
-from daft.table import MicroPartition
+from daft.recordbatch import MicroPartition
 
 DATA = {
     "int": ([1, 2, None], DataType.int64()),
@@ -97,7 +97,7 @@ def test_repr():
     assert (
         without_escape.replace("\r", "")
         == """╭─────────────┬─────────╮
-│ Column Name ┆ Type    │
+│ column_name ┆ type    │
 ╞═════════════╪═════════╡
 │ int         ┆ Int64   │
 ├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌┤
@@ -117,7 +117,7 @@ def test_repr_html():
     assert (
         out_repr
         == f"""<table class="dataframe">
-<thead><tr><th {TH_STYLE}>Column Name</th><th {TH_STYLE}>Type</th></tr></thead>
+<thead><tr><th {TH_STYLE}>column_name</th><th {TH_STYLE}>type</th></tr></thead>
 <tbody>
 <tr><td {TD_STYLE}>int</td><td {TD_STYLE}>Int64</td></tr>
 <tr><td {TD_STYLE}>float</td><td {TD_STYLE}>Float64</td></tr>
